<?php

namespace Punishment\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Punishment\Form\PunishmentForm;
use Punishment\Model\PunishmentModel;
use Punishment\Repository\PunishmentRepository;
use Setup\Model\JobHistory;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class PunishmentController extends HrisController {

    private $jobHistoryId = [
        'C' => 30,
        'G' => 31,
        'P' => 32,
        'T' => 8,
        'R' => 33
    ];

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(PunishmentRepository::class);
        $this->initializeForm(PunishmentForm::class);
    }

    public function indexAction(){
        $request = $this->getRequest();

        if ($request->isPost()) {

            $data = iterator_to_array($this->repository->fetchAll(), false);
            return new JsonModel(['success'=>true, 'data'=>$data]);

        }

    }

    public function addAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $this->form->setData($request->getPost());

            if ($this->form->isValid()) {

                $punishment = new PunishmentModel();
                $punishment->exchangeArrayFromForm($this->form->getData());
                $punishment->punishmentId = ((int) Helper::getMaxId($this->adapter, PunishmentModel::TABLE_NAME, PunishmentModel::PUNISHMENT_ID)) + 1;
                $punishment->createdDate = Helper::getcurrentExpressionDate();
                $punishment->startDate = date('Y-m-d', strtotime($punishment->startDate)); 
                $punishment->endDate = date('Y-m-d', strtotime($punishment->endDate));
                $punishment->status = 'E';
                $punishment->createdBy = $this->employeeId;

                if($request->getPost()['payrollGenerate']){
                    $this->repository->impactPayroll($punishment->employeeId, 'N');
                    $punishment->impactPayroll = 'Y';
                }else{
                    $this->repository->impactPayroll($punishment->employeeId, 'Y');
                    $punishment->impactPayroll = 'N';
                }

                // Add job history
                $this->repository->addJobHistory($punishment->employeeId, 
                                                $punishment->startDate, 
                                                $this->jobHistoryId[$punishment->punishmentType],
                                                $punishment->endDate);

                $punishment->jobHistoryId = ((int) Helper::getMaxId($this->adapter, JobHistory::TABLE_NAME, JobHistory::JOB_HISTORY_ID));
                $this->repository->add($punishment);

                $this->flashmessenger()->addMessage("Punishment Sucessfully created");
                return $this->redirect()->toRoute("punishment");
            }

        }

        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FULL_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FULL_NAME", "ASC", " ", FALSE, TRUE);

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => $employees
        ]);
    }

    public function editAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("basicSalary", array("action"=>"index"));
        }

        $data = iterator_to_array($this->repository->fetchById($id));

        $model = new PunishmentModel();
        $model->exchangeArrayFromDB($data);

        if ($request->isPost())
        {

            $punishment = new PunishmentModel();
            $postData = $request->getPost();

            $this->form->setData($postData);

            if ($this->form->isValid())
            {

                $punishment->exchangeArrayFromForm($this->form->getData());
                $punishment->modifiedDate = Helper::getcurrentExpressionDate();
                $punishment->modifiedBy = $this->employeeId;
                $punishment->startDate = date('Y-m-d', strtotime($punishment->startDate));
                $punishment->endDate = date('Y-m-d', strtotime($punishment->endDate));

                if($request->getPost()['payrollGenerate']){
                    $this->repository->impactPayroll($punishment->employeeId, 'N');
                    $punishment->impactPayroll = 'Y';
                }else{
                    $this->repository->impactPayroll($punishment->employeeId, 'Y');
                    $punishment->impactPayroll = 'N';
                }

                $this->repository->edit($punishment, $id);
                $this->repository->editJobHistory($model->jobHistoryId, 
                                                  $this->jobHistoryId[$punishment->punishmentType], 
                                                  $punishment->startDate, 
                                                  $punishment->endDate, 
                                                  $punishment->employeeId);

                $this->flashmessenger()->addMessage("Punishment Sucessfully updated");
                return $this->redirect()->toRoute("punishment");
            }
        }

        $model->startDate = date('d-M-Y', strtotime($model->startDate)); 
        $model->endDate = date('d-M-Y', strtotime($model->endDate));
        $this->form->bind($model);

        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FULL_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FULL_NAME", "ASC", " ", FALSE, TRUE);
        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => $employees,
            'id' => $id,
            'impactPayroll' => $model->impactPayroll
        ]);
    }

    public function deleteAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("punishment", array("action"=>"index"));
        }

        $this->repository->delete($id);

        $this->flashmessenger()->addMessage("Punishment deleted successfully");
        return $this->redirect()->toRoute("punishment", array("action"=>"index"));
    }

    public function revokeAction(){
        $postData = $this->getRequest()->getPost();

        // Get punishment information
        $punishmentData = iterator_to_array($this->repository->fetchById($postData['punishmentId']));

        $this->repository->impactPayroll($punishmentData['EMPLOYEE_ID'], 'Y');

        // Add job history
        $this->repository->addJobHistory($punishmentData['EMPLOYEE_ID'], 
                                        date("Y-m-d"), 
                                        34,
                                        null);

        $revokeHistory = ((int) Helper::getMaxId($this->adapter, JobHistory::TABLE_NAME, JobHistory::JOB_HISTORY_ID));
        // Modify punishment data
        $this->repository->revoke($postData['punishmentId'], $this->employeeId, $postData['revokeRemarks'], $revokeHistory);


        $this->flashmessenger()->addMessage("Punishment revoked successfully");
        return $this->redirect()->toRoute("punishment", array("action"=>"index"));
    }

}