<?php

namespace Recruitment\Controller;

use Application\Controller\HrisController;
use Application\Helper\Helper;
use Recruitment\Form\UserInformationForm;
use Recruitment\Model\UserInformationModel;
use Recruitment\Repository\UserInformationRepository;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Zend\View\Model\JsonModel;

class UserInformationController extends HrisController {

    function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(UserInformationRepository::class);
        $this->initializeForm(UserInformationForm::class);
    }

    public function indexAction()
    {
        $request = $this->getRequest();

        if ($request->isPost()) 
        {
            // Get data from database
            $list = iterator_to_array($this->repository->allUsers(), false);

            return new JsonModel(['success' => true, 'data' => $list, 'error' => '']);
        }
    }

    public function editAction()
    {
        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0)
        {
            return $this->redirect()->toRoute("userinformation", array("action"=>"index"));
        }

        if ($request->isPost()) 
        {
            $postData = $request->getPost();
            $this->form->setData($postData);

            if ($this->form->isValid()) 
            {
                $userInfo = new UserInformationModel();
                $userInfo->exchangeArrayFromForm($this->form->getData());
                $userInfo->modifiedDt = Helper::getcurrentExpressionDate();

                // Update data
                $this->repository->edit($userInfo, $id);

                $this->flashmessenger()->addMessage("User Successfully Updated!!!");
                return $this->redirect()->toRoute("userinformation", array("action"=>"edit", "id"=>$id));
            }
        }

        $data = iterator_to_array($this->repository->fetchUserById($id));

        $model = new UserInformationModel();
        $model->exchangeArrayFromDB($data);
        $this->form->bind($model);

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'id' => $id,
            'passwordDtl' => $model->password,
        ]);
    }

    public function checkUserNameAction() {
        try {
            $request = $this->getRequest();
            $userName = $request->getPost('userName');
            $userId = $request->getPost('userId');
            
            $returnData = $this->repository->checkUserNameAvailability($userName,$userId);

            $availability = 'YES';
            if ($returnData) {
                $availability = 'NO';
            }

            return new JsonModel(['success' => true, 'data' => $availability, 'error' => '']);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

}