<?php
namespace Application\Helper;

use Application\Model\Model;
use DateTime;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\ResultSet\ResultSet;
use Zend\Db\Sql\Expression;
use Zend\Db\Sql\Sql;
use System\Repository\SystemSettingRepository;

class Helper {

    const ORACLE_DATE_FORMAT = "DD-MON-YYYY";
    const ORACLE_TIME_FORMAT = "HH:MI AM";
    const ORACLE_TIMESTAMP_FORMAT = "HH24:MI";
    const MYSQL_DATE_FORMAT = "";
    const PHP_DATE_FORMAT = "d-M-Y";
    const PHP_TIME_FORMAT = "h:i A";
    const FLOAT_ROUNDING_DIGIT_NO = 2;
    const UPLOAD_DIR = __DIR__ . "/../../../../public/uploads";
    const UPLOAD_DIR_EMP = __DIR__ . "/../../../../public/uploads/profile";
    const SH_DIR = __DIR__ . "/../../../../public/sh";

    const decodeExceptionList = array("Ekadashi");

//  method to add flashmessage to view
    public static function addFlashMessagesToArray($context, $return) {
        $flashMessenger = $context->flashMessenger();
        if ($flashMessenger->hasMessages()) {
            $return['messages'] = $flashMessenger->getMessages();
        }
        return $return;
    }

//  method to generate maxId
    public static function getMaxId(AdapterInterface $adapter, $tableName, $columnName) {
        $sql = new Sql($adapter);
        $select = $sql->select();
        $select->from($tableName);
        $select->columns(["MAX({$columnName}) AS MAX_{$columnName}"], false);

        $statement = $sql->prepareStatementForSqlObject($select);

        $result = $statement->execute();
        $row = $result->current();
        return $row["MAX_{$columnName}"];
    }

    public static function convertColumnDateFormat(AdapterInterface $adapter, Model $table, $attrs = null, $timeAttrs = null, $shortForm = null, $timeIntervalAttrs = null) {
        $temp = get_object_vars($table);
        if ($attrs != null) {
            foreach ($attrs as $attr) {
                unset($temp[$attr]);
            }
        }

        if ($timeAttrs != null) {
            foreach ($timeAttrs as $attr) {
                unset($temp[$attr]);
            }
        }

        unset($temp['mappings']);

        if ($timeAttrs != null) {
            foreach ($timeAttrs as $attr) {
                unset($temp[$attr]);
            }
        }
        if ($timeIntervalAttrs != null) {
            foreach ($timeIntervalAttrs as $attr) {
                unset($temp[$attr]);
            }
        }

        $attributes = array_keys($temp);

        $tempCols = [];

        if ($attrs != null) {
            foreach ($attrs as $attr) {
//                array_push($tempCols, Helper::appendDateFormat($adapter, $table->mappings[$attr], self::ORACLE_DATE_FORMAT));
                array_push($tempCols, Helper::appendDateFormat($adapter, $table->mappings[$attr], self::ORACLE_DATE_FORMAT, $shortForm));
            }
        }

        if ($timeAttrs != null) {
            foreach ($timeAttrs as $attr) {
                array_push($tempCols, Helper::appendDateFormat($adapter, $table->mappings[$attr], self::ORACLE_TIME_FORMAT, $shortForm));
            }
        }

        if ($timeIntervalAttrs != null) {
            foreach ($timeIntervalAttrs as $attr) {
                array_push($tempCols, Helper::appendDateFormat($adapter, $table->mappings[$attr], self::ORACLE_TIMESTAMP_FORMAT, $shortForm));
            }
        }

        foreach ($attributes as $attribute) {
            array_push($tempCols, Helper::columnExpression($table->mappings[$attribute], $shortForm, null));
        }
        return $tempCols;
    }

    public static function appendDateFormat($adapter, $columnName, $format, $shortForm = null) {
        $pre = "";
        if ($shortForm != null && strlen($shortForm) != 0) {
            $pre = $shortForm . ".";
        }
        $tempStr = "";
        switch (strtolower($adapter->getPlatform()->getName())) {
            case strtolower("Oracle"):
                $tempStr = "INITCAP(TO_CHAR({$pre}{$columnName}, '{$format}')) AS {$columnName}";
                break;

            case strtolower("Mysql"):

                break;

            case strtolower("odbc"):
                $tempStr = "INITCAP(TO_CHAR({$pre}{$columnName}, '{$format}')) AS {$columnName}";
                break;

        }
        return $tempStr;
    }

    public static function dateExpression($columnName, $shortForm = null) {
        $format = Helper::ORACLE_DATE_FORMAT;
        $pre = "";
        if ($shortForm != null && strlen($shortForm) != 0) {
            $pre = $shortForm . ".";
        }
        $tempStr = "INITCAP(TO_CHAR({$pre}{$columnName}, '{$format}')) AS {$columnName}";
        return new Expression($tempStr);
    }

    public static function timeExpression($columnName, $shortForm = null) {
        $format = Helper::ORACLE_TIME_FORMAT;
        $pre = "";
        if ($shortForm != null && strlen($shortForm) != 0) {
            $pre = $shortForm . ".";
        }
        $tempStr = "INITCAP(TO_CHAR({$pre}{$columnName}, '{$format}')) AS {$columnName}";
        return new Expression($tempStr);
    }

    public static function datetimeExpression($columnName, $shortForm = null) {
        $format = Helper::ORACLE_DATE_FORMAT . " " . self::ORACLE_TIME_FORMAT;
        $pre = "";
        if ($shortForm != null && strlen($shortForm) != 0) {
            $pre = $shortForm . ".";
        }
        $tempStr = "INITCAP(TO_CHAR({$pre}{$columnName}, '{$format}')) AS {$columnName}";
        return new Expression($tempStr);
    }

    public static function columnExpression($columnName, $shortForm = null, $function = null, $customColumnName = null) {
        $pre = "";
        if ($shortForm != null && strlen($shortForm) != 0) {
            $pre = $shortForm . ".";
        }

        if ($customColumnName == null) {
            $customColumnName = $columnName;
        }
        if ($function == NULL) {
            $tempStr = "{$pre}{$columnName} AS {$columnName}";
        } else {
            $tempStr = "${function}({$pre}{$columnName}) AS {$customColumnName}";
        }
        return new Expression($tempStr);
    }

    public static function getExpressionDate($dateStr) {
        $format = Helper::ORACLE_DATE_FORMAT;
        return new Expression("TO_DATE('{$dateStr}', '{$format}')"
        );
    }

    public static function getExpressionDateHana($dateStr) {
        $format = Helper::ORACLE_DATE_FORMAT;
        return new Expression("TO_DATE('{$dateStr}', 'YYYY-MM-DD')"
        );
    }

    public static function getExpressionDateHanaDDMONYYYY($dateStr) {
         return new Expression("TO_DATE('{$dateStr}', 'DD-MON-YYYY')"
        );
    }

    public static function getExpressionTime($dateStr, $format = null) {
        if ($format == null) {
            $format = Helper::ORACLE_TIME_FORMAT;
        }
        return new Expression("TO_TIME('{$dateStr}', '{$format}')");
    }

    public static function getExpressionTimeHANA($dateStr, $format = null) {
        if ($format == null) {
            $format = Helper::ORACLE_TIME_FORMAT;
        }
        return new Expression("TO_TIMESTAMP('{$dateStr}', 'HH:MI AM')");
    }

    public static function getExpressionDateTime($dateStr, $format = null) {
        if ($format == null) {
            $format = Helper::ORACLE_DATE_FORMAT . " " . Helper::ORACLE_TIME_FORMAT;
        }
        return new Expression("TO_TIMESTAMP('{$dateStr}', '{$format}')");
    }

    public static function getExpressionDateTimeHana($timeStr, $dateStr = null, $format = null) {
        if ($format == null) {
            $format = Helper::ORACLE_DATE_FORMAT . " " . Helper::ORACLE_TIME_FORMAT;
        }
        if($timeStr == null || $timeStr == ''){
            return new Expression("TO_TIME('{$timeStr}', '{$format}')");
        }
        else{
            return new Expression("TO_TIMESTAMP('{$dateStr} {$timeStr}', '{$format}')");
        }
        
    }

    public static function getTimestampHana($timeStr, $dateStr = null, $format = null) {
        if ($format == null) {
            $format = Helper::ORACLE_DATE_FORMAT . " " . Helper::ORACLE_TIME_FORMAT;
        }
        if($timeStr == null || $timeStr == ''){
            return "TO_TIME('{$timeStr}', '{$format}')";
        }
        else{
            return "TO_TIMESTAMP('{$dateStr} {$timeStr}', '{$format}')";
        }
        
    }

    public static function getcurrentExpressionDate() {
        $currentDate = date(self::PHP_DATE_FORMAT);
        return self::getExpressionDate($currentDate);
    }

    public static function getcurrentExpressionTime() {
        $currentTime = date(self::PHP_TIME_FORMAT);
        return self::getExpressionTime($currentTime);
    }

    public static function getcurrentExpressionDateTime() {
        date_default_timezone_set('Asia/Kathmandu');
        $currentDate = date(self::PHP_DATE_FORMAT . " " . self::PHP_TIME_FORMAT);
        return self::getExpressionDateTime($currentDate);
    }

    public static function getcurrentMonthDayExpression() {
        $currentDate = date('d-M');
        $format = 'DD-MON';
        return new Expression("TO_DATE('{$currentDate}','{$format}')");
    }

    public static function getCurrentDate() {
        $currentDate = date(self::PHP_DATE_FORMAT);
        return $currentDate;
    }

    public static function renderCustomView() {
        return function ($object) {
            $elems = $object->getValueOptions();
            $counter = 1;
            $name = $object->getName();
            $atts = $object->getAttributes();
            $disabled = '';
            if (in_array('disabled', $atts)) {
                $disabled = 'disabled';
            }
            foreach ($elems as $key => $value) {
                $temp = (($object->getValue() == "") && ($counter == $object->getCheckedValue())) || ($object->getValue() == $key) ? 'checked=checked' : '';

                echo "<div class = 'md-radio'> <input {$temp} {$disabled} type = 'radio' value = '{$key}' name = '{$name}' id = '{$name}+{$value}' class = 'md-radiobtn radioButton'><label for = '$name+$value'>
                            <span></span>
                            <span class = 'check'></span>
                            <span class = 'box'></span> $value
                        </label> </div>";
                $counter++;
            }
        };
    }

    public static function renderCustomViewForCheckbox() {
        return function ($object) {
            $elems = $object->getValueOptions();
            $counter = 1;
            $name = $object->getName();
            $atts = $object->getAttributes();
            $disabled = '';
            if (in_array('disabled', $atts)) {
                $disabled = 'disabled';
            }
            foreach ($elems as $key => $value) {
                $temp = '';
                if ($object->getValue() == "") {
                    if ($counter == $object->getCheckedValue()) {
                        $temp = 'checked=checked';
                    }
                } else {
                    if (in_array($key, $object->getValue())) {
                        $temp = 'checked=checked';
                    }
                }
                echo "<div class = 'md-checkbox'>";
                echo "<input $temp $disabled type = 'checkbox' value = '$key' name = '$name' id = '$name+$value' class = 'md-check'>";

                echo "<label for = '$name+$value'>
                <span></span>
                <span class = 'check'></span>
                <span class = 'box'></span> $value
                </label>";
                echo "</div>";
                $counter++;
            }
        };
    }

    public static function hydrate($class, ResultSet $resultSet) {
        $tempArray = [];
        foreach ($resultSet as $item) {
            $model = new $class();
            $model->exchangeArrayFromDB(
                $item->getArrayCopy());
            array_push($tempArray, $model);
        }
        return $tempArray;
    }
    /*
     * This function return the raw result in array or object array form
     */

    public static function extractDbData($rawArray, bool $inArray = false, string $arrangeWithKey = null): array {
        $extractedArray = [];
        foreach ($rawArray as $item) {
            if ($inArray) {
                $item = $item->getArrayCopy();
            }
            if ($arrangeWithKey == null) {
                array_push($extractedArray, $item);
            } else {
                $extractedArray[$item[$arrangeWithKey]] = $item;
            }
        }
        return $extractedArray;
    }

    public static function generateUniqueName() {
        $date = new DateTime();
        $t = $date->getTimestamp();
        return $t + rand(0, 1000);
    }

    public static function maintainFloatNumberFormat($floatNumber) {
        return number_format($floatNumber, self::FLOAT_ROUNDING_DIGIT_NO, '.', '');
    }

    public static function hoursToMinutes($formattedHours) {
        if (isset($formattedHours) && strpos($formattedHours, ":") > 0) {
            list($hours, $minutes) = explode(':', $formattedHours);
            return $hours * 60 + $minutes;
        } else {
            return 0;
        }
    }

    public static function minutesToHours($minutes) {
        $hours = (int) ($minutes / 60);
        $minutes -= $hours * 60;
        return sprintf("%d:%02.0f", $hours, $minutes);
    }

    public static function dateDiff($date1, $date2) {
        $fromDate = \DateTime::createFromFormat(Helper::PHP_DATE_FORMAT, $date1);
        $toDate = \DateTime::createFromFormat(Helper::PHP_DATE_FORMAT, $date2);
        $interval = $fromDate->diff($toDate);
        return $interval->format('%a');
    }

    public static function encryptPassword($password) {
        return new Expression("FN_ENCRYPT_PASSWORD('$password')");
    }
	
	public static function uploadFilesVacancy($receivedFiles,$folder) {
        $citizenshipDir = getcwd() . '/public/uploads/documents/'.$folder;
        $empFileDir = getcwd() . '/public/uploads';

        if (!file_exists($citizenshipDir)) {
            mkdir($citizenshipDir, 0777, true);
        }
            $imageName = $receivedFiles['name'];
            $newImageName = time().$receivedFiles['name'];
            $extension = explode('.', $receivedFiles['name']);
            $extension = end($extension);
            $path = 'uploads/documents/'.$folder. "/";
            $empFilePath = $empFileDir. "/" . $newImageName;
            $movingPath = $citizenshipDir. "/" . $newImageName;

        $filesStoring = [
            'imageName' => $imageName,
            'newImageName' => $newImageName,
            'tmp_name' => $receivedFiles['tmp_name'],
            'path' => $path,
            'extension' => $extension,
            'empFilePath' => $empFilePath,
            'movingPath' => $movingPath,
            'folder' => $folder
        ];
        return $filesStoring;
    }

    /**
     * Converts english number to nepali and viceversa
     */
    public static function numConverter($number, $to="english"){
        $eng_number = array(
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9"
        );
        $nep_number = array(
            "०",
            "१",
            "२",
            "३",
            "४",
            "५",
            "६",
            "७",
            "८",
            "९"
        );

        if($to=="english"){
            return str_replace($nep_number, $eng_number, $number);
        }else{
            return str_replace($eng_number, $nep_number, $number);
        }
    }

    /**
     * Converts Nepali date to english and vice versa
    */
    public static function convertDateLanguage($date, $language){

        if($date){
            if($language =="nepali"){
                $dateArray = explode('/', date_format(date_create($date), "Y/m/d"));
            }else{
                $dateArray = explode('/', $date);
            }

            $dateConversion = Helper::numConverter($dateArray[0], $language) . '/' . 
                                Helper::numConverter($dateArray[1], $language)  . '/' . 
                                Helper::numConverter($dateArray[2], $language);

            return $dateConversion;
        }
    }

    /**
     * Converts Nepali time to english and vice versa
    */
    public static function convertTimeLanguage($time, $language){
        if($time){
            $timeArr = explode(':', $time);

            $timeConversion = Helper::numConverter($timeArr[0], $language) . ':' . 
                                Helper::numConverter($timeArr[1], $language);

            return $timeConversion;
        }
    }

    /**
     * Convert english ad no into nepali
     */

     public static function convertAdNo($adNo){

        $eng_number = array(
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9"
        );

        $currentNumber = null;
        $convertedNumber = null;

        for($i = 0; $i<strlen($adNo); $i++){

            if(in_array($adNo[$i], $eng_number)){
                $currentNumber = $currentNumber . $adNo[$i];
            }else{
                $conversion = Helper::numConverter($currentNumber, 'nepali');
                $currentNumber = null;
                $convertedNumber = $convertedNumber . $conversion . $adNo[$i];
            }
            
            if($i == strlen($adNo) - 1){
                $conversion = Helper::numConverter($currentNumber, 'nepali');
                $convertedNumber = $convertedNumber . $conversion;
            }

        }

        return $convertedNumber;

    }

    public static function nepaliAmPm($start_time, $end_time) {

        if (strpos($start_time, 'AM')) {


            return 'बिहानको '. str_replace('AM', '',$start_time) . ' बजे ';


        } else {

            return 'दिउसोको '. str_replace('PM', '',$start_time) . ' बजे ';

        }

    }

    // Start at variable is used to by pass certain useless data at beginning index
    public static function decodeBase64List($list, $index=null, $startAt=0)
    {
        // decode all index from database
        for($i=$startAt; $i<count($list); $i++)
        {
            if($index){
                // Check if data is actually encoded
                foreach($index as $item){
                    $list[$i][$item] = Helper::decodingUTF($list[$i][$item]);
                } 
            }else{
                // Check if data is actually encoded
                $list[$i] = Helper::decodingUTF($list[$i]);
            }
        }

        return $list;
    }

    //=============== ENCODING CODE ================
    /**
     * ENCODING UTF 
     * 
     * @ PARAMETER  |  STRING
     * 
     * @@ THIS HELPER WORKS PROCESS
     *    - EVERY FIRST CHARACTER CHECKED IF ASCII OR UTF
     *    - IF UTF THEN ENCODING
     *    - ELSE ASCII THEN LEFT AS IT IS
     * 
     * 
     * */

    public static function encodingUTF($parameter) {

        $first_character = mb_substr($parameter,0, 1, "UTF8");

        /**
         * ASCII TABLE MAX 127 ELSE UTF
         * */

        if (ord($first_character) > 127) {

            $encoding_parameter = base64_encode($parameter);

        } else {

            $encoding_parameter = $parameter;

        }

        return $encoding_parameter;


    }

    /**
     * DECODING STRING
     * 
     * @ PARAMETER  |  STRING
     * 
     * @@ THIS HELPER WORKS PROCESS
     *    - CHECK PARAMETER ENCODED OR NOT
     *    - IF YES DECODE
     *    - ELSE LEFT AS IT IS
     * 
     * 
     * */

    public static function decodingUTF($parameter) {



        /*
            // Character (string)   Base64_encode (length)
            // 1                    4 (with == padding)
            // 2                    4 (with =  padding)
            // 3                    4 (without padding)
            // 4                    8 (with == padding)
            // 5                    8 (with =  padding)
            // 6                    8 (without padding)
            // 7                    12(with == padding)
            // 8                    12(with =  padding)
            // 9                    12(without padding)
            // 10                   16(with == padding)
            // 11                   16(with =  padding)
        */

        if (Helper::checkEncodedBase64($parameter)) {

            $decode_parameter = base64_decode($parameter);

        } else {

            $decode_parameter = $parameter;

        }

        return $decode_parameter;

    }

    /**
     * 
     * CHECK IF CHARACTER HAS '=' PADDING 
     * 
     * */
    private static function checkEncodingPadding($string) {

        if ( strpos($string, '=') ) {

            return true;

        } 

        return false;

    }

    /**
     * 
     * CHECK DECODE STRING SHOULD HAVE 1 LESS CHARACTER THAN BASE64_CODE STRING
     * 
     * */
    private static function checkEncodedBase64($string)
    {
        $original_string_length = strlen($string);

        $base64_decoded_length  = strlen( base64_decode($string) );

        // Some exceptional case: Non encoded code is also considered as encoded one
        if(in_array($string, Helper::decodeExceptionList)){
            return false;
        }

        /**
         * BASE64_ENCODED ARE ALWAYS EVEN STARTS WITH 4 BYTE AND MULTIPLE WITH 4 ..... RESULT [4,8,12,16,20] 
         * */

        if (($original_string_length % 4) == 0) {
        /**
         * IF ORIGINAL_STRING_LENGTH LENGTH GREATER THAN BASE64_DECODED_LENGTH THEN STRING IS BASE64_ENCODED
         * */

            if ($original_string_length > $base64_decoded_length) {

                /**
                 * IF STRING HAS PADDING '='
                 * */
                if (Helper::checkEncodingPadding($string)) {

                    /* YES ENCODED STRING */
                    return true;


                } else {

                    if (strpos($string, ' ')) {

                        return false;


                    } else {

                        // $divide_by_3           = $base64_decoded_length % 3;
                        // $multply_and_modulos_4 = ($divide_by_3 * 4) % 4;

                        if (base64_decode($string) != '') {


                                if (mb_detect_encoding(base64_decode($string)) == '') {

                                    return false;

                                } else {

                                    return true;

                                }

                        } else {

                            return false;

                        }

                    }

                }


            } else {

                return false;
            }

        } else {

            return false;

        }

    }

    //=============== ENCODING CODE ================


    /**
     * 
     * GET AGE DIFFERENCE FROM NEPALI DATE
     * 
     * */

    public static function DateDiffByNepaliDate($start_date, $end_date) {

        if ($start_date == '' || $end_date == '') {

            return false;

        }

        $date1 = date_create($start_date);
        $date2 = date_create($end_date);

        /***
        * GET DIFFERENCE IN NEPALI
        */
        $intval = date_diff($date2, $date1);

        // TOTAL DAYS
        $days =  $intval->format("%a");


        /**
        * FINDING AGE BY ADDING ABOVE DAYS WITH TILL CERTAIN DATE
        */
        // $days = 151;

        $start_date = new \DateTime($end_date);
        $end_date = (new \DateTime($end_date))->add(new \DateInterval("P{$days}D") );
        $dd = date_diff($start_date,$end_date);
        // return $dd;
        $age = $dd->y." years ".$dd->m." months ".$dd->d." days";

        return $age;

    }

    /**
     * GET RETIREMENT DATE FROM DOB
     * 
     * */
    public static function getRetirementDate($dob, $age, $current_date) {

        $disect_age = explode(' ', $age);
        $disect_current_date = explode(' ', $current_date);

        /*** AGE YEAR ***/ 
        $age_year   = $disect_age[0];

        /*** NOC RETIREMENT AGE IS 58 ***/
        $left_retirement_year = 58 - $age_year;

        if ($left_retirement_year > 0) {

            $disect_dob = explode('-', $dob);
            $retirement =  $disect_current_date[0] + $left_retirement_year .'-'.$disect_dob[1].'-'.$disect_dob[2];
            
            return $retirement;

        }

        return false;      

    }

    public static function getDataFromDiffDate($start_date, $end_date) {

        if ($start_date == '' || $end_date == '') {

            return false;

        }

        $date1 = date_create($start_date);
        $date2 = date_create($end_date);

        /*** GET DIFFERENCE IN NEPALI ***/
        $intval = date_diff($date2, $date1);

        // TOTAL DAYS
        $days =  $intval->format("%a");

        $start_date = new \DateTime($end_date);
        $end_date = (new \DateTime($end_date))->add(new \DateInterval("P{$days}D") );
        $dd = date_diff($start_date,$end_date);
        return round($dd->days/30);

    }

    /**
     * INDEXING WITH MONTH NAME TO NUMBER
     * 
     * STARTING MONTH : SHRAWAN
     * 
     * */
    public static function monthToIndex($month_name)
    {   

        $get_month_index = [
                'shrawan' => 1,
                'bhadra'  => 2,
                'ashwin'  => 3,
                'kartik'  => 4,
                'mangsir' => 5,
                'paush'   => 6,
                'magh'    => 7,
                'falgun'  => 8,
                'chaitra' => 9,
                'bhaisakh'=> 10,
                'jestha'  => 11,
                'ashadh'  => 12
            ];



        $month_name = strtolower($month_name);

        return $get_month_index[$month_name];
    }

    /**
     * INDEXING WITH NUMBER TO MONTH NAME
     * 
     * STARTING MONTH : SHRAWAN
     * 
     * */
    public static function indexToMonth($month_index)
    {   

        $get_month_index = [
                1  => 'shrawan',
                2  => 'bhadra',
                3  => 'ashwin',
                4  => 'kartik',
                5  => 'mangsir',
                6  => 'paush',
                7  => 'magh',
                8  => 'falgun',
                9  => 'chaitra',
                10 => 'bhaisakh',
                11 => 'jestha',
                12 => 'ashadh'
            ];

        return $get_month_index[$month_index];
    }


    public static function getEmployeeID(AdapterInterface $adapter, $code = null) {

        if (count($code) == 0) {

            return false;

        }

        // $sql = new Sql($adapter);
        // $select = $sql->select();
        // $select->from('HRIS_EMPLOYEES');
        // $select->columns(['EMPLOYEE_ID'], FALSE);
        // $select->where(['EMPLOYEE_CODE' => $code]);

        $sql = "SELECT * FROM HRIS_EMPLOYEES WHERE EMPLOYEE_CODE LIKE '%".$code."%'";

        // $statement = $sql->prepareStatementForSqlObject($select);
        $statement = $adapter->query($sql);
        
        $result = $statement->execute();
        $row    = $result->current();
        // echo "<pre>";
        // print_r($row);
        // die;
        // echo "<pre>";
        // print_r($row);
        // die;
        return $row;
    }

    public static function checkEmployeeLoanID(AdapterInterface $adapter, $employee_id, $loan_id) {

        if (count($loan_id) == 0 || is_null($employee_id)) {

            return false;

        }

        $sql = "SELECT * FROM HRIS_EMPLOYEE_LOAN_REQUEST WHERE EMPLOYEE_ID = {$employee_id} AND LOAN_ID = {$loan_id}";

        $statement = $adapter->query($sql);
        $result    = $statement->execute();
        $row       = $result->current();
        
        if ($row) {
            return TRUE;
        }
        return FALSE;
    }

    public static function getExcelDate($date) {

        $segment = explode('.', $date);

        return $segment[0] . '-' . 
               ((strlen((string)$segment[1]) == 1) ? '0' . (string)$segment[1] : (string)$segment[1]) . '-' . 
               ((strlen((string)$segment[2]) == 1) ? '0' . (string)$segment[2] : (string)$segment[2]);

    }

    public static function getPreference(AdapterInterface $adapter, $key){
        $repository = new SystemSettingRepository($adapter);
        
        return $repository->fetchByKey($key)['VALUE'];
    }

    public static function sparrowMessage(AdapterInterface $adapter,$to, $text){

        $sendSMS = self::getPreference($adapter, 'SEND_SMS');
        
        if($sendSMS == 'Y'){

            $token = self::getPreference($adapter, 'SPARROW_TOKEN');
            $from = self::getPreference($adapter, 'SPARROW_FROM');

            // var_dump($token);
            // var_dump($from);

            // STEP 1
            $args = http_build_query(array(
                'token' => $token,
                'from'  => $from,
                'to'    => $to,
                'text'  => $text));
        
            $url = "http://api.sparrowsms.com/v2/sms/";
        
            # Make the call using API.
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS,$args);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        
            // Response
            $response = curl_exec($ch);
            $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            curl_close($ch);

            return $response;
        }
    }
}
