<?php
namespace Application\Helper;

use Application\Helper\Helper;
use DateTime;
use Setup\Repository\AdvanceRepository;
use Setup\Repository\EmployeeRepository;
use Setup\Repository\LoanRepository;
use Setup\Repository\LoanRestrictionRepository;
use Zend\Db\Adapter\AdapterInterface;

class LoanAdvanceHelper{
    
    public static function getLoanList(AdapterInterface $adapter,$employeeId){
        $employeeId = $employeeId;
        $loanRepo = new LoanRepository($adapter);
        $loanRestrictionRepo = new LoanRestrictionRepository($adapter);
        $employeeRepo = new EmployeeRepository($adapter);
        
        $employeeDetail = $employeeRepo->fetchById($employeeId);
        
        $position = $employeeDetail['POSITION_ID'];
        $serviceType = $employeeDetail['SERVICE_TYPE_ID'];
        $designation = $employeeDetail['DESIGNATION_ID'];
        
        $salary = (int)$employeeDetail['SALARY'];

        if($employeeDetail['JOIN_DATE']==null){
            $joinDate = new DateTime();
        }else{
            $joinDate = DateTime::createFromFormat(Helper::PHP_DATE_FORMAT, $employeeDetail['JOIN_DATE']);
        }
        
        $currentDate = new DateTime();
        
        $different = date_diff($joinDate,$currentDate);
        $yr = $different->format('%y');
        $mn = $different->format('%m');
        $days = $different->format('%d');
        $mnPercentage = 0.083;
        
        $mnInPer = $mn * $mnPercentage;
        $totalYr =(double)($yr+$mnInPer);
        $actTotalYr = round($totalYr,2);

        $loanList = $loanRepo->fetchActiveRecord();

        $loanResultList = [];
        $loanResultList[null] = "   - - - Please Select - - -";
        foreach($loanList as $loanRow){

            $loanId = $loanRow['LOAN_ID'];
            $restrictionDtl = $loanRestrictionRepo->getByLoanId($loanId);
            
            if ($restrictionDtl) {

                $positionList    = explode(",", $restrictionDtl['position']);
                $serviceTypeList = explode(",", $restrictionDtl['serviceType']);
                $designationList = explode(",", $restrictionDtl['designation']);
                $salaryRange     = explode(",", $restrictionDtl['salaryRange']);
                
            
                if ( count($salaryRange) > 1 ) {
                    $salaryFrom = (int)$salaryRange[0];
                } else {
                    $salaryFrom="";
                }

                if ( count($salaryRange) == 2 ) {
                    $salaryTo = (int)$salaryRange[1];
                }

                if ( count($salaryRange) == 1 ||  count($salaryRange) != 2 ) {
                    $salaryTo = "";
                }
            
                $workingPeriod =  explode(",",$restrictionDtl['workingPeriod']);

                if ( count($workingPeriod) > 1 ) {
                    $workingPeriodFrom = (int)$workingPeriod[0];
                } else {
                    $workingPeriodFrom="";
                }

                if ( count($workingPeriod) == 2 ) {
                    $workingPeriodTo = (int)$workingPeriod[1];
                }

                if ( count($workingPeriod) ==1 ||  count($workingPeriod) != 2 ) {
                    $workingPeriodTo = "";
                }

                if ( !$loanRow['ELIGIBLE_PERIOD'] ) {
                    $loanRow['ELIGIBLE_PERIOD'] = 0;
                }

                if ( $actTotalYr>=$loanRow['ELIGIBLE_PERIOD'] && 
                     !in_array($position,$positionList) && 
                     !in_array($serviceType, $serviceTypeList) && 
                     !in_array($designation,$designationList)  && 
                     ($salary >= ($salaryFrom ? $salaryFrom : $salary) && $salary <= ($salaryTo ? $salaryTo : $salary)) && 
                     ($actTotalYr >= ($workingPeriodFrom ? $workingPeriodFrom : $actTotalYr) && $actTotalYr <= ($workingPeriodTo ? $workingPeriodTo : $actTotalYr))
                    ) 
                    {
                    
                        $loanResultList[$loanRow['LOAN_ID']] = $loanRow['LOAN_NAME'];

                    }
            }

        }

        return $loanResultList;
    }

    public static function getLoanListBy(AdapterInterface $adapter,$employeeId,$column, $value){
        $employeeId = $employeeId;
        $loanRepo = new LoanRepository($adapter);
        $loanRestrictionRepo = new LoanRestrictionRepository($adapter);
        $employeeRepo = new EmployeeRepository($adapter);
        
        $employeeDetail = $employeeRepo->fetchById($employeeId);
        
        $position = $employeeDetail['POSITION_ID'];
        $serviceType = $employeeDetail['SERVICE_TYPE_ID'];
        $designation = $employeeDetail['DESIGNATION_ID'];
        
        $salary = (int)$employeeDetail['SALARY'];

        if($employeeDetail['JOIN_DATE']==null){
            $joinDate = new DateTime();
        }else{
            $joinDate = DateTime::createFromFormat(Helper::PHP_DATE_FORMAT, $employeeDetail['JOIN_DATE']);
        }
        
        $currentDate = new DateTime();
        
        $different = date_diff($joinDate,$currentDate);
        $yr = $different->format('%y');
        $mn = $different->format('%m');
        $days = $different->format('%d');
        $mnPercentage = 0.083;

        
        $mnInPer = $mn * $mnPercentage;
        $totalYr =(double)($yr+$mnInPer);
        $actTotalYr = round($totalYr,2);
        
        
        $loanList = $loanRepo->fetchActiveRecordByArgument($column, $value);
        // echo "<pre>";print_r($loanList); die;

        $loanResultList = [];
        // $loanResultList[null] = "   - - - Please Select - - -";
        foreach($loanList as $loanRow){

            $loanId = $loanRow['LOAN_ID'];
            $restrictionDtl = $loanRestrictionRepo->getByLoanId($loanId);
            $positionList = explode(",",$restrictionDtl['position']);
            $serviceTypeList =  explode(",",$restrictionDtl['serviceType']);
            $designationList = explode(",",$restrictionDtl['designation']);
            $salaryRange =  explode(",",$restrictionDtl['salaryRange']);
            
            if(count($salaryRange)>1){
                $salaryFrom = (int)$salaryRange[0];
            }else{
                $salaryFrom="";
            }
            if(count($salaryRange)==2){
               $salaryTo = (int)$salaryRange[1];
            }
            if(count($salaryRange)==1 ||  count($salaryRange)!=2){
                $salaryTo = "";
            }
            
            $workingPeriod =  explode(",",$restrictionDtl['workingPeriod']);
            if(count($workingPeriod)>1){
                $workingPeriodFrom = (int)$workingPeriod[0];
            }else{
                $workingPeriodFrom="";
            }
            if(count($workingPeriod)==2){
               $workingPeriodTo = (int)$workingPeriod[1];
            }
            if(count($workingPeriod)==1 ||  count($workingPeriod)!=2){
                $workingPeriodTo = "";
            }
            if(!$loanRow['ELIGIBLE_PERIOD']){
                $loanRow['ELIGIBLE_PERIOD'] = 0;
            }
            if($actTotalYr>=$loanRow['ELIGIBLE_PERIOD'] && !in_array($position,$positionList) && !in_array($serviceType, $serviceTypeList) && !in_array($designation,$designationList)  && ($salary>=($salaryFrom?$salaryFrom:$salary) && $salary<=($salaryTo?$salaryTo:$salary)) && ($actTotalYr>=($workingPeriodFrom?$workingPeriodFrom:$actTotalYr) && $actTotalYr<=($workingPeriodTo?$workingPeriodTo:$actTotalYr))){
                $loanResultList[$loanRow['LOAN_ID']] = $loanRow['LOAN_NAME'];
            }

        }

        return $loanResultList;
    }

    public static function getLoanListDetails(AdapterInterface $adapter,$employeeId,$column, $value){
        $employeeId = $employeeId;
        $loanRepo = new LoanRepository($adapter);
        $loanRestrictionRepo = new LoanRestrictionRepository($adapter);
        $employeeRepo = new EmployeeRepository($adapter);
        
        $employeeDetail = $employeeRepo->fetchById($employeeId);
        
        $position    = $employeeDetail['POSITION_ID'];
        $serviceType = $employeeDetail['SERVICE_TYPE_ID'];
        $designation = $employeeDetail['DESIGNATION_ID'];
        
        $salary = (int)$employeeDetail['SALARY'];

        if($employeeDetail['JOIN_DATE']==null){
            $joinDate = new DateTime();
        }else{
            $joinDate = DateTime::createFromFormat(Helper::PHP_DATE_FORMAT, $employeeDetail['JOIN_DATE']);
        }
        
        $currentDate = new DateTime();
        
        $different = date_diff($joinDate,$currentDate);
        $yr = $different->format('%y');
        $mn = $different->format('%m');
        $days = $different->format('%d');
        $mnPercentage = 0.083;
        
        $mnInPer = $mn * $mnPercentage;
        $totalYr =(double)($yr+$mnInPer);
        $actTotalYr = round($totalYr,2);

        $loanList = $loanRepo->fetchActiveRecordByArgument($column, $value);

        $loanResultList = [];
        foreach($loanList as $loanRow){

            $loanId = $loanRow['LOAN_ID'];
            $restrictionDtl = $loanRestrictionRepo->getByLoanId($loanId);
            $positionList = explode(",",$restrictionDtl['position']);
            $serviceTypeList =  explode(",",$restrictionDtl['serviceType']);
            $designationList = explode(",",$restrictionDtl['designation']);
            $salaryRange =  explode(",",$restrictionDtl['salaryRange']);
            
            if(count($salaryRange)>1){
                $salaryFrom = (int)$salaryRange[0];
            }else{
                $salaryFrom="";
            }
            if(count($salaryRange)==2){
               $salaryTo = (int)$salaryRange[1];
            }
            if(count($salaryRange)==1 ||  count($salaryRange)!=2){
                $salaryTo = "";
            }
            
            $workingPeriod =  explode(",",$restrictionDtl['workingPeriod']);
            if(count($workingPeriod)>1){
                $workingPeriodFrom = (int)$workingPeriod[0];
            }else{
                $workingPeriodFrom="";
            }
            if(count($workingPeriod)==2){
               $workingPeriodTo = (int)$workingPeriod[1];
            }
            if(count($workingPeriod)==1 ||  count($workingPeriod)!=2){
                $workingPeriodTo = "";
            }
            if(!$loanRow['ELIGIBLE_PERIOD']){
                $loanRow['ELIGIBLE_PERIOD'] = 0;
            }
            if($actTotalYr>=$loanRow['ELIGIBLE_PERIOD'] && !in_array($position,$positionList) && !in_array($serviceType, $serviceTypeList) && !in_array($designation,$designationList)  && ($salary>=($salaryFrom?$salaryFrom:$salary) && $salary<=($salaryTo?$salaryTo:$salary)) && ($actTotalYr>=($workingPeriodFrom?$workingPeriodFrom:$actTotalYr) && $actTotalYr<=($workingPeriodTo?$workingPeriodTo:$actTotalYr))){
                $loanResultList[$loanRow['LOAN_ID']] = $loanRow['LOAN_NAME'];
            }

        }

        return $loanResultList;
    }
    
    public static function getAdvanceList(AdapterInterface $adapter,$employeeId){
        $employeeId = $employeeId;
        $employeeRepo = new EmployeeRepository($adapter);
        
        $advanceRepo = new AdvanceRepository($adapter);
        $employeeDetail = $employeeRepo->fetchById($employeeId);
        $salary = $employeeDetail['SALARY'];
        
        $advanceResult = $advanceRepo->fetchActiveRecord();
        $advanceList = [];
        
        foreach($advanceResult as $advanceRow){
            $minSalaryAmt = $advanceRow['MIN_SALARY_AMT'];
            $maxSalaryAmt = $advanceRow['MAX_SALARY_AMT'];
            
            if($salary>=$minSalaryAmt && $salary<=$maxSalaryAmt){
                $advanceList[$advanceRow['ADVANCE_ID']] = $advanceRow['ADVANCE_NAME'];
            }
        }
        return $advanceList;
    }

    public static function getLoanMapData($loanId, $loanAmount)
    {
        $array = [];
        
        if ($loanId == 1 || $loanId == 12) {

            /* LOAN ITEM */
            $array['LOAN_ITEM']   = 'landLoan';

            /* MONTHLY VALUE ID */
            $array['MTH_ID']      = 38;

        } else if ($loanId == 3 || $loanId == 13) {

            $array['LOAN_ITEM']   = 'socialLoan';
            $array['MTH_ID']      = 41;

        } else if ($loanId == 4) {

            $array['LOAN_ITEM']   = 'motorCycleLoan';
            $array['MTH_ID']      = 40;
       
        } else if ($loanId == 5 || $loanId == 6) {

            $array['LOAN_ITEM']   = 'marriageLoan';
            $array['MTH_ID']      = 46;

        } else if ($loanId == 7) {

            $array['LOAN_ITEM']   = 'educationLoan';
            $array['MTH_ID']      = 42;

        } else if ($loanId == 8) { 

            $array['LOAN_ITEM']   = 'medicalLoan';
            $array['MTH_ID']      = '';
            // $monthlyData['MTH_ID'] = 41;
            
        } else if ($loanId == 2 || $loanId == 9) {
            
            $array['LOAN_ITEM']   = 'hml';

            if ($loanId == 2) {

                $array['MTH_ID']      = 39;
                
            } else {
                
                $array['MTH_ID']      = 21;

            }
            
        } else if ($loanId == 10) {

            $array['LOAN_ITEM']   = 'familyInsuranceLoan';
            $array['MTH_ID']      = 43;
            
        } else if ($loanId == 11) {
            
            $array['LOAN_ITEM']   = 'vehiclePurchaseLoan';
            $array['MTH_ID']      = 48;

        }  else if ($loanId == 14) {
            
            $array['LOAN_ITEM']   = 'modernTechnology';
            $array['MTH_ID']      = 47;

        } 

        /* LOAN AMOUNT */
        $array['LOAN_AMOUNT'] = $loanAmount;


        return $array;

    }


}