<?php

namespace Appraisal\Controller;

use Zend\Mvc\Controller\AbstractActionController;
use Zend\Db\Adapter\AdapterInterface;
use Appraisal\Form\QuestionsEmployeeForm;
use Appraisal\Model\QuestionsEmployee;
use Appraisal\Repository\QuestionsEmployeeRepository;
use Zend\Authentication\AuthenticationService;
use Zend\Form\Annotation\AnnotationBuilder;
use Application\Helper\Helper;
use Application\Helper\EntityHelper;
use Appraisal\Model\Heading;
use Setup\Repository\EmployeeRepository;
use Application\Custom\CustomViewModel;
use Appraisal\Model\QuestionOption;
use Appraisal\Repository\QuestionOptionRepository;


class QuestionsEmployeeController extends AbstractActionController {

    private $adapter;
    private $repository;
    private $form;
    private $employeeId;
    private $userId;

    public function __construct(AdapterInterface $adapter) {
        $this->adapter = $adapter;
        $this->repository = new QuestionsEmployeeRepository($adapter);
        $auth = new AuthenticationService();
        $this->employeeId = $auth->getStorage()->read()["employee_id"];
        $this->userId = $auth->getStorage()->read()["user_id"];
    }

    public function initializeForm() {
        $form = new QuestionsEmployeeForm();
        $builder = new AnnotationBuilder();
        $this->form = $builder->createForm($form);
    }

    public function indexAction() {
        $result = $this->repository->fetchAll();
        $list = [];
        foreach ($result as $row) {
            array_push($list, $row);
        }
        // echo '<pre>';
        // print_r($list);die;
        return Helper::addFlashMessagesToArray($this, [
                    'questions' => $list,
                    'employeeId'=> $this->employeeId,
        ]);
    }

    public function addAction() {
        $this->initializeForm();
        // echo '<pre>';
        $request = $this->getRequest();
        if ($request->isPost()) {
            $question = new QuestionsEmployee();
            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                $data=$request->getPost();

                $question->exchangeArrayFromForm($this->form->getData());
                
                $question->questionType=$data['questionType'];
                $question->status='E';
                $question->createdBy=$this->employeeId;
                $question->createdDate=date('Y-m-d');
                $question->questionId = ((int) Helper::getMaxId($this->adapter, QuestionsEmployee::TABLE_NAME, QuestionsEmployee::QUESTION_ID)) + 1;                
                $this->repository->add($question);
                $this->flashmessenger()->addMessage("Appraisal Question Successfully added!!!");
                return $this->redirect()->toRoute("questionsEmployee");
            }
        }
        return Helper::addFlashMessagesToArray($this, [
                    'form' => $this->form,
                    'customRenderer' => Helper::renderCustomView()
        ]);
    }

    public function editAction() {
        $this->initializeForm();
        $id = $this->params()->fromRoute('id');
        if ($id == 0) {
            $this->redirect()->toRoute('questionsEmployee');
        }
        // print_r($id);die;
        $request = $this->getRequest();
        $question = new QuestionsEmployee();
        if (!$request->isPost()) {
            $question->exchangeArrayFromDB($this->repository->fetchById($id)->getArrayCopy());
            // print_r($question->questionType);die;
            $this->form->bind($question);
        } else {
            // print_r($request->getPost());die;
            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                $question->exchangeArrayFromForm($this->form->getData());
                $question->modifiedDate = date('Y-m-d');
                $question->modifiedBy = $this->employeeId;
                // print_r($question->questionId);die;
                $this->repository->edit($question,$question->questionId);
                $this->flashmessenger()->addMessage("Appraisal Question Successfully Updated!!!");
                return $this->redirect()->toRoute("questionsEmployee");
            }
        }
        return Helper::addFlashMessagesToArray($this, [
                    'form' => $this->form,
                    'headings' => EntityHelper::getTableKVListWithSortOption($this->adapter, Heading::TABLE_NAME, Heading::HEADING_ID, [Heading::HEADING_EDESC], [Heading::STATUS => 'E'], Heading::HEADING_EDESC, "ASC", NULL, FALSE, TRUE),
                    'question' => $question,
                    'customRenderer' => Helper::renderCustomView(),
                    'id' => $id
        ]);
    }

    public function deleteAction() {
        $id = $this->params()->fromRoute('id');
        $questionOptionRepo = new QuestionOptionRepository($this->adapter);
        if ($id === 0) {
            $this->redirect()->toRoute('questionsEmployee');
        }
        $this->repository->delete($id);
        $this->flashmessenger()->addMessage("Appraisal Question Successfully Deleted!!!");
        return $this->redirect()->toRoute("questionsEmployee");
    }

}
