<?php
namespace Appraisal\Repository;

use Application\Repository\RepositoryInterface;
use Zend\Db\TableGateway\TableGateway;
use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;
use Application\Helper\Helper;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Select;
use Zend\Db\Sql\Expression;
use Application\Model\Model;
use Setup\Model\HrEmployees;
use Appraisal\Model\AppraisalAnswer;
use Appraisal\Model\AppraisalEmp;
use Appraisal\Model\AppraisalRemarks;
use Appraisal\Model\AppraisalEval;
use Appraisal\Model\AppraisalEvalDetails;

use Application\Helper\EntityHelper;

use Appraisal\Model\Heading;
use Appraisal\Model\Question;
use Appraisal\Model\StageQuestion; 
use Appraisal\Model\Stage;

class AppraisalRepository extends HrisRepository implements RepositoryInterface {
    protected $adapter;
    protected $tableGateway;
    protected $remarkTableGateway;

    public function __construct(AdapterInterface $adapter) {
        $this->adapter = $adapter;
        $this->tableGateway = new TableGateway(AppraisalEmp::TABLE_NAME,$adapter);
        $this->remarkTableGateway= new TableGateway(AppraisalRemarks::TABLE_NAME,$adapter);
        $this->evalTableGateway= new TableGateway(AppraisalEval::TABLE_NAME,$adapter);
        $this->evalDetailsTableGateway= new TableGateway(AppraisalEvalDetails::TABLE_NAME,$adapter);        
    }

    public function add(Model $model) {
        // echo '<pre>';
        // print_r($model->getArrayCopyForDB());die;
        $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function addDetailRemarks(Model $model){
        // print_r($model);die;
        $this->remarkTableGateway->insert($model->getArrayCopyForDB());
    }

    public function appraisalDetails($appraisalId){
        $sql=" select
                        e.employee_code,
                        e.full_name,
                        e2.full_name as supervisor_full_name,
                        hp.position_name as position_name,
                        hfl.functional_level_edesc as functional_level_edesc,
                        hae.*,
                        hfy.fiscal_year_name,
                        hb.branch_name as branch_name,
                        hd.department_name department_name,
                        bs_date(hae.requested_date) as requested_date_nepali,
                            
                        hp2.position_name as supervisor_position_name,
                        hfl2.functional_level_edesc as supervisor_functional_level_edesc,
                        hb2.branch_name as supervisor_branch_name,
                        hd2.department_name supervisor_department_name
                from hris_appraisal_employee hae 
                left join hris_employees e on hae.employee_id=e.employee_id 
                left join hris_positions hp on hp.position_id=hae.position_id 
                left join hris_employees e2 on e2.employee_id=hae.supervisor_id 
                left join hris_functional_levels hfl on hfl.functional_level_id=hae.functional_level_id 
                left join hris_fiscal_years hfy on hfy.fiscal_year_id=hae.fiscal_year_id 
                left join hris_departments hd on hd.department_id=hae.department_id 
                left join hris_branches hb on hb.branch_id=hae.branch_id 
                
                left join hris_functional_levels hfl2 on hfl2.functional_level_id=e2.functional_level_id 
                left join hris_positions hp2 on hp2.position_id=e2.position_id 
                left join hris_departments hd2 on hd2.department_id=e2.department_id 
                left join hris_branches hb2 on hb2.branch_id=e2.branch_id 
                where hae.appraisal_id={$appraisalId};";
        $result=$this->rawQuery($sql);
        return $result[0];
    }

    public function getRemarkDataFromDb($appraisalId){
        $sql="select * from hris_appraisal_remarks_detail where appraisal_id={$appraisalId}";
        $result=$this->rawQuery($sql);
        return $result;
    }

    // public function addDetailRemarks($data){
    //     $appraisalId=$data['appraisalId'];
    //     $id=$data['id'];
    //     $workPerformed=$data['workPerformed'];
    //     $workAchieved=$data['workAchieved'];
    //     $status=$data['status'];

    //     $sql="insert into HRIS_APPRAISAL_REMARKS_DETAIL (APPRAISAL_ID,ID,WORK_PERFORMED_REMARK,WORK_ACHIEVED_REMARK,STATUS)
    //      values($appraisalId,$id,'$workPerformed','$workAchieved','$status')";

    //     //  print_r($sql);die;


    //      $result=$this->rawQuery($sql);

    //      return 'success';
    // }

    public function update(Model $model,$appraisalId){
        $this->tableGateway->update($model->getArrayCopyForDB(),[AppraisalEmp::APPRAISAL_ID=>$appraisalId]);
    }

    public function updateRemark(Model $model,$appraisalId,$id){
        $this->remarkTableGateway->update($model->getArrayCopyForDB(),[AppraisalRemarks::APPRAISAL_ID=>$appraisalId,AppraisalRemarks::APP_REMARK_ID=>$id]);
    }

    public function insertAppraisalEval(Model $model){
        $this->evalTableGateway->insert($model->getArrayCopyForDB());
    }

    public function insertAppraisalEvalDetails(Model $model){
        $this->evalDetailsTableGateway->insert($model->getArrayCopyForDB());
    }

    public function delete($id) {
        
    }

    public function edit(Model $model, $id) {
    }

    public function fetchAll() {
        
    }

    public function fetchById($id) {
        
    }

    public function getEmpDet($empId){

        // print_r($empId);die;
        
        $sql="select
                e.employee_id,
                 e.employee_code,
                e.full_name,
                hp.position_id,
                hp.position_name,
                hfl.functional_level_id,
                hfl.functional_level_edesc,
                e.department_id,
                hd.department_name,
                e.branch_id,
                hb.branch_name,
                e.location_id,
                hl.location_edesc
            from hris_employees e 
            left join hris_positions hp on hp.position_id=e.position_id 
            left join hris_functional_levels hfl on hfl.functional_level_id=e.functional_level_id 
            left join hris_departments hd on hd.department_id=e.department_id
            left join hris_branches hb on hb.branch_id=e.branch_id
            left join hris_locations hl on hl.location_id=e.location_id
            where e.employee_id={$empId}";

        $rowset = EntityHelper::rawQueryResult($this->adapter, $sql);
        return $rowset->current();

    }


    // FOR POTENTIAL SUPERVISOR
    // public function getSupervisors($employeeId){
    //     $sql="select
    //             e.employee_id,e.employee_code,e.full_name
    //                     from hris_employees e 
    //             left join hris_functional_levels hfl on e.functional_level_id=hfl.functional_level_id 
    //             left join hris_functional_levels hfl1 on e.acting_functional_level_id=hfl1.functional_level_id 
    //             where (case when acting_functional_level_id is null 
    //                 then hfl.order_id 
    //                 else hfl1.order_id 
    //                 end) < (select
    //                     order_id 
    //                 from hris_functional_levels 
    //                 where functional_level_id = (select
    //                     case when acting_functional_level_id is null 
    //                     then functional_level_id 
    //                     else acting_functional_level_id 
    //                     end 
    //                     from hris_employees 
    //                     where employee_id={$employeeId}) ) 
    //             and e.status='E' ";
    //             $result=$this->rawQuery($sql);

    //             // print_r($result);die;
    //     return $result;
    // }

    public function getSupervisors(){
        $sql="select
                e.employee_id,e.employee_code,e.full_name
                        from hris_employees e 
                where e.status='E'
                ";
                $result=$this->rawQuery($sql);

                // print_r($result);die;
        return $result;
    }

    public function getAppraisalReq($employeeId){
        $sql="select Row_Number() OVER(ORDER BY appraisal_id) as sn,hae.*,e.full_name,hfy.fiscal_year_name from hris_appraisal_employee hae
        left join hris_employees e on hae.employee_id=e.employee_id 
        left join hris_fiscal_years hfy on hae.fiscal_year_id=hfy.fiscal_year_id
         where hae.supervisor_id={$employeeId} and hae.application_stage='RQ' and hae.status='E';";
        $result=$this->rawQuery($sql);
        // print_r($result);die;
        return $result;
    }


    public function getAppraisalRevReq($employeeId){
        $sql="select Row_Number() OVER(ORDER BY appraisal_id) as sn,hae.*,e.full_name,hfy.fiscal_year_name,e2.full_name as supervisor_full_name from hris_appraisal_employee hae
        left join hris_employees e on hae.employee_id=e.employee_id 
        left join hris_employees e2 on hae.supervisor_id=e2.employee_id 
        left join hris_fiscal_years hfy on hae.fiscal_year_id=hfy.fiscal_year_id
         where hae.reviewer_id={$employeeId} and hae.application_stage='SU' and hae.status='E';";
         $result=$this->rawQuery($sql);
        //  print_r($result);die;
        return $result;
    }

    

    

    public function cancelReq($employeeId,$appraisalId){
        $sql="update hris_appraisal_employee set status='C' where appraisal_id={$appraisalId} and employee_id={$employeeId}";
        
        
        $this->rawQuery($sql);
        return 'success';
    }

    public function getAppraisalReqSupervisor($employeeId){
        $sql="select hae.appraisal_id,hae.appraisal_from_date,hae.appraisal_to_date,e.full_name,e1.full_name as supervisor_name,hae.status
        from hris_appraisal_employee hae
                left join hris_employees e on e.employee_id=hae.employee_id 
                left join hris_employees e1 on e1.employee_id=hae.supervisor_id
                where hae.supervisor_id={$employeeId} and hae.status='RQ'
     ";
    //  print_r($sql);die;
     $result=$this->rawQuery($sql);
    //  print_r($result);die;
     return $result;
    }

    public function checkAppReq($employeeId){
        $sql="select employee_id,fiscal_year_id from hris_appraisal_employee
        where employee_id={$employeeId} and status='E'
        ";
        
        $result=$this->rawQuery($sql);
        //  print_r($result);die;
        return $result;
    }


    //##########################################################################################################################
    // FOR EVALUATION PART

    // not required must delete 
    public function getAppraisalForEval($appraisalId){
        $sql="select
                hae.*,
            e1.full_name as employee_full_name ,
            e1.position_id as employee_position,
            e1.department_id as employee_department,
            e2.full_name as supervisor_full_name,
            e2.position_id as supervisor_position_id,
            e2.department_id as supervisor_department_id,
            hd.department_name as supervisor_department_name,
            hp.position_name as supervisor_position_name

                    from hris_appraisal_employee hae 
        left join hris_employees e1 on e1.employee_id=hae.employee_id 
        left join hris_employees e2 on e2.employee_id=hae.supervisor_id 
        left join hris_positions hp on e2.position_id=hp.position_id
        left join hris_departments hd on e2.department_id=hd.department_id

        ";
        $result=$this->rawQuery($sql);
        return $result;
    }

    //this is for supervisor evaluation
    public function getQuestions(){
        $sql="select * from HRIS_APPRAISAL_QUESTIONS_EMPLOYEE where status='E' and question_type='SR' order by order_id asc";
        // here SR refers to questions for Supervisor/Reviewer and RC refers to questions for Review Committee 
        $result=$this->rawQuery($sql);
        return $result;
    }

    

    //this is for reviewer evaluation
    public function getQuestionsEval($appraisalId){
        $sql="select
        t1.question_id,
        t1.question_edesc,
        t1.status,
        t1.order_id,
        t1.detail_id,
        t1.marks su_marks,
        t1.remark as su_remarks,
        t1.total_mark as su_total_mark,
        t2.remark as re_remarks,
        t2.marks as re_marks,
        t2.total_mark as re_total_mark 
   from (select
        haqe.*,
       hae.appraisal_id,
       haed.marks,
       haed.detail_id,
       hae.remark,
       hae.total_mark 
       from HRIS_APPRAISAL_QUESTIONS_EMPLOYEE haqe 
       left join hris_appraisal_evaluation_details haed on haqe.question_id=haed.question_id 
       left join hris_appraisal_evaluation hae on hae.evaluation_id=haed.evaluation_id 
       where haqe.status='E' 
       and hae.appraisal_id={$appraisalId} 
       and hae.evaluator_role='SU' and haqe.question_type='SR'
       order by order_id asc ) t1 
   left join (select
        haqe.*,
       hae.appraisal_id,
       haed.marks,
       haed.detail_id,
       hae.remark,
       hae.total_mark 
       from HRIS_APPRAISAL_QUESTIONS_EMPLOYEE haqe 
       left join hris_appraisal_evaluation_details haed on haqe.question_id=haed.question_id 
       left join hris_appraisal_evaluation hae on hae.evaluation_id=haed.evaluation_id 
       where haqe.status='E' and haqe.question_type='SR'
       and hae.appraisal_id={$appraisalId}
       and hae.evaluator_role='RE' 
       order by order_id asc ) t2 on t1.question_id = t2.question_id ";
        
        $result=$this->rawQuery($sql);
        return $result;
    }

    public function getCommiteeQuestion(){
        $sql="select * from hris_appraisal_questions_employee where status='E' and question_type='C1'";
        $result=$this->rawQuery($sql);
        return $result;
    }

    
}
