<?php
namespace Appraisal\Repository;

use Zend\Db\TableGateway\TableGateway;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Expression;
use Appraisal\Model\QuestionsEmployee;
use Zend\Db\Sql\Select;
use Application\Repository\RepositoryInterface;
use Application\Helper\EntityHelper;

class QuestionsEmployeeRepository implements RepositoryInterface{
    private $tableGateway;
    private $adapter;
    
    public function __construct(AdapterInterface $adapter) {
        $this->tableGateway = new TableGateway(QuestionsEmployee::TABLE_NAME,$adapter);
        $this->adapter = $adapter;
    }

    public function add(\Application\Model\Model $model) {
        // print_r($model);die;
        $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function delete($id) {
        $this->tableGateway->update([QuestionsEmployee::STATUS=>'D'],[QuestionsEmployee::QUESTION_ID=>$id]);
    }

    public function edit(\Application\Model\Model $model, $id) {
        $data = $model->getArrayCopyForDB();
        unset($data[QuestionsEmployee::QUESTION_ID]);
        unset($data[QuestionsEmployee::CREATED_DATE]);
        unset($data[QuestionsEmployee::STATUS]);
        // print_r($id);die;
        $this->tableGateway->update($data,[QuestionsEmployee::QUESTION_ID=>$id]);
    }

    public function fetchAll() {
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->from(['AQ' => "HRIS_APPRAISAL_QUESTIONS_EMPLOYEE"]);        
        $select->where(["AQ.STATUS='E'"]);
        $select->order("AQ.QUESTION_TYPE");
        $select->order("AQ.ORDER_ID");
        $caseExpression = new Expression(
            "CASE
                WHEN AQ.QUESTION_TYPE='SR' THEN 'SUPERVISOR/REVIEWER QUESTION'
                WHEN AQ.QUESTION_TYPE='C1' THEN 'REVIEWER COMMITTEE 1'
                WHEN AQ.QUESTION_TYPE='C2' THEN 'REVIEWER COMMITTEE 2'
            END AS QUESTION_TYPE_DESC,*"
        );
        
        $select->columns([$caseExpression]);
        $statement = $sql->prepareStatementForSqlObject($select);
        // print_r($statement->getSql());die;
        $result = $statement->execute();
        // print_r($result);die;
        return $result;
    }

    public function fetchById($id) {
        $rowset = $this->tableGateway->select(function(Select $select) use($id){
            $select->where([QuestionsEmployee::QUESTION_ID => $id, QuestionsEmployee::STATUS => 'E']);
        });
        return $result = $rowset->current();
    }

    public function fetchByHeadingId($headingId){
        $rowset= $this->tableGateway->select(function(Select $select) use($headingId) {
            $select->where([Question::STATUS=>'E',Question::HEADING_ID=>$headingId]);
            $select->order(Question::QUESTION_ID." ASC");
        });
        $result = [];
        foreach($rowset as $row){
            array_push($result,$row);
        }
        return $result;
    }
}