<?php

namespace AttendanceManagement\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use AttendanceManagement\Form\AttendanceOverrideForm;
use AttendanceManagement\Model\AttendanceOverrideModel;
use AttendanceManagement\Repository\AttendanceOverrideRepository;
use Zend\View\Model\JsonModel;

class AttendanceOverrideController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(AttendanceOverrideRepository::class);
        $this->initializeForm(AttendanceOverrideForm::class);
    }

    public function indexAction(){
        $request = $this->getRequest();

        if ($request->isPost()) {

            $data = iterator_to_array($this->repository->fetchAll($this->getLocationIfManager()), false);

            return new JsonModel(['data'=>$data, 'success'=>true]);
        }
    }

    public function addAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {
            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                $model = new AttendanceOverrideModel();
                $model->exchangeArrayFromForm($this->form->getData());
                $model->overrideId = ((int) Helper::getMaxId($this->adapter, AttendanceOverrideModel::TABLE_NAME, AttendanceOverrideModel::OVERRIDE_ID)) + 1;
                $model->createdBy = $this->employeeId;
                $model->attendanceDt = date_format(date_create($model->attendanceDt), "Y-m-d");
                $model->createdDate = Helper::getcurrentExpressionDate();
                $model->isForceLeave = $request->getPost()['IS_FORCE_LEAVE'];
                
                $this->repository->add($model);
                $this->repository->reAttendance($model->attendanceDt, $model->employeeId);

                return new JsonModel(["success"=>true]);
            }
        }

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => $this->getEmployeeList()
        ]);
    }

    public function editAction(){
        $id = (int) $this->params()->fromRoute("id");
        if ($id === 0) {
            return $this->redirect()->toRoute("attendancebyhr");
        }

        $request = $this->getRequest();

        $data = $this->repository->fetchById($id); 
        $prevModel = new AttendanceOverrideModel();
        $prevModel->exchangeArrayFromDB($data);

        if($request->isPost()){
            $model = new AttendanceOverrideModel();
            $postData = $request->getPost();
            
            $this->form->setData($postData);

            if ($this->form->isValid()) {
                $model->exchangeArrayFromForm($this->form->getData());
                $model->attendanceDt = date_format(date_create($model->attendanceDt), "Y-m-d");

                $this->repository->edit($model, $id);

                // Revert previous date and employee data
                $this->repository->reAttendance($prevModel->attendanceDt, $prevModel->employeeId);
                $this->repository->reAttendance($model->attendanceDt, $model->employeeId);

                return new JsonModel(["success"=>true]);
            }
        }

        $this->form->bind($prevModel);

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'id' => $id,
            'employees' => $this->getEmployeeList(),
            'attendanceDate' => date_format(date_create($prevModel->attendanceDt), "d-M-Y"),
        ]);        
    }

    public function deleteAction(){
        $id = (int) $this->params()->fromRoute("id");
        if ($id === 0) {
            return $this->redirect()->toRoute("attendancebyhr");
        }

        $data = $this->repository->fetchById($id); 
        $model = new AttendanceOverrideModel();
        $model->exchangeArrayFromDB($data);

        $this->repository->delete($id);
        $this->repository->reAttendance($model->attendanceDt, $model->employeeId);

        $this->flashmessenger()->addMessage("Attendance Override Successfully Deleted!!!");
        return $this->redirect()->toRoute("attendanceOverride");
    }

    public function getInAndOutTimeAction(){

        $request = $this->getRequest();
        $postData = $request->getPost();

        $date = date_format(date_create($postData['attendanceDate']), "Y-m-d");

        $thumbData = $this->repository->getThumbTime($postData['employeeId'], $date);
        $attendanceData = $this->repository->getAttendanceTime($postData['employeeId'], $date);

        return new JsonModel(['success'=>true, 'thumbData'=>$thumbData, 'attendanceDate'=>$attendanceData]);

    }

    /**
     * Check for url if it is from either HR side or manager side and returns location_id if it is from manager side
     */
    private function getLocationIfManager(){
        $url = explode('/', $_SERVER['REQUEST_URI']);

        if(in_array("attendanceOverrideManager", $url)){
            return $this->repository->getLocationId($this->employeeId)['LOCATION_ID'];
        }

        return null;
    }

    /**
     * It gets all employees if from HR side and returns only location users if from manager side
     */
    private function getEmployeeList(){
        $condition = [
            "STATUS" => "E",
            "RETIRED_FLAG" => "N",
        ];

        $locationId = $this->getLocationIfManager();

        if($locationId){
            $condition["LOCATION_ID"] = $locationId;
        }

        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FIRST_NAME", "MIDDLE_NAME", "LAST_NAME"], $condition, "FIRST_NAME", "ASC", " ", FALSE, TRUE);

        return $employees;
    }

    public function forceLeaveAction(){

        $postData = $this->getRequest()->getPost();
        $this->repository->addForceLeave($postData);

        return new JsonModel(['success'=>true]);

    }

}