<?php
namespace AttendanceManagement\Repository;

use Application\Helper\EntityHelper;
use Application\Model\Model;
use Application\Repository\RepositoryInterface;
use AttendanceManagement\Model\AttendanceOverrideModel;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\TableGateway\TableGateway;

class AttendanceOverrideRepository{

    private $tableGateway;
    private $adapter;

    public function __construct(AdapterInterface $adapter) {
        $this->tableGateway = new TableGateway(AttendanceOverrideModel::TABLE_NAME, $adapter);
        $this->adapter = $adapter;
    }

    public function add(Model $model){
        $data = $model->getArrayCopyForDB();
        $this->tableGateway->insert($data);
    }

    public function edit(Model $model, $id){
        $array = $model->getArrayCopyForDB();
        $this->tableGateway->update($array, [AttendanceOverrideModel::OVERRIDE_ID => $id]);
    }

    public function fetchAll($locationId){

        $sql = "
            SELECT 
            OVERRIDE_ID AS OVERRIDE_ID,
            HE.FIRST_NAME || ' ' || ifnull(HE.MIDDLE_NAME,'') || ' ' || HE.LAST_NAME    AS EMPLOYEE_NAME,
            HAO.ATTENDANCE_DT AS ATTENDANCE_DATE,
            TO_CHAR(HAO.IN_TIME, 'MON DD, YYYY HH:MI AM') AS IN_TIME,
            TO_CHAR(HAO.OUT_TIME, 'MON DD, YYYY HH:MI AM') AS OUT_TIME,
            HE2.FIRST_NAME || ' ' || ifnull(HE2.MIDDLE_NAME,'') || ' ' || HE2.LAST_NAME    AS CREATED_BY,
            HAO.CREATED_DATE
            FROM HRIS_ATTENDANCE_OVERRIDE HAO
            LEFT JOIN HRIS_EMPLOYEES HE ON HAO.EMPLOYEE_ID = HE.EMPLOYEE_ID
            LEFT JOIN HRIS_EMPLOYEES HE2 ON HAO.CREATED_BY = HE2.EMPLOYEE_ID
            WHERE HAO.STATUS = 'E'
        ";

        if($locationId){
            $sql = $sql . " AND HE.LOCATION_ID = {$locationId}";
        }

        $sql = EntityHelper::heirarchy($sql, 'HE');

        $statement = $this->adapter->query($sql); 
        return $statement->execute();

    }

    public function fetchById($id){
        $sql = "
            SELECT 
            OVERRIDE_ID,
            EMPLOYEE_ID,
            ATTENDANCE_DT,
            TO_CHAR(HAO.IN_TIME, 'yyyy-MM-dd hh24:mm') AS IN_TIME,
            TO_CHAR(HAO.OUT_TIME, 'yyyy-MM-dd hh24:mm') AS OUT_TIME,
            CREATED_BY,
            CREATED_DATE,
            IN_REMARKS,
            OUT_REMARKS
            FROM HRIS_ATTENDANCE_OVERRIDE HAO
            WHERE HAO.STATUS = 'E' AND OVERRIDE_ID = {$id}
        ";

        $statement = $this->adapter->query($sql); 
        return $statement->execute()->current();
    }

    public function delete($id){
        $sql = "
            UPDATE HRIS_ATTENDANCE_OVERRIDE SET STATUS = 'D' WHERE OVERRIDE_ID = {$id}
        ";

        $statement = $this->adapter->query($sql); 
        $statement->execute();
        
    }

    public function reAttendance($date, $employeeId){

        $sql = "CALL HRIS_REATTENDANCE('{$date}', {$employeeId}, NULL)";

        $statement = $this->adapter->query($sql); 
        $statement->execute();

    }

    public function getLocationId($employeeId){

        $sql = "SELECT LOCATION_ID FROM HRIS_EMPLOYEES WHERE EMPLOYEE_ID = {$employeeId}";

        $statement = $this->adapter->query($sql); 
        return $statement->execute()->current();
    }

    public function getThumbTime($employeeId, $attendanceDate){
        $sql = "SELECT 
                TO_CHAR(IN_TIME_OUT_TIME('{$employeeId}', '{$attendanceDate}', 'IN'), 'YYYY-MM-DD HH:MI AM') AS IN_TIME, 
                TO_CHAR(IN_TIME_OUT_TIME('{$employeeId}', '{$attendanceDate}', 'OUT'), 'YYYY-MM-DD HH:MI AM') AS OUT_TIME 
                FROM DUMMY
            ";

        $statement = $this->adapter->query($sql); 
        return $statement->execute()->current();
    }

    public function getAttendanceTime($employeeId, $attendanceDate){
        $sql = "SELECT 
                TO_CHAR(IN_TIME, 'YYYY-MM-DD HH24:MI') AS IN_TIME, 
                TO_CHAR(OUT_TIME, 'YYYY-MM-DD HH24:MI') AS OUT_TIME 
                FROM HRIS_ATTENDANCE_DETAIL 
                WHERE EMPLOYEE_ID = {$employeeId} AND ATTENDANCE_DT = '{$attendanceDate}'
            ";
            
        $statement = $this->adapter->query($sql); 
        return $statement->execute()->current();
    }

}