<?php
namespace LeaveManagement\Repository;

use Application\Helper\EntityHelper;
use Application\Model\Model;
use Application\Repository\HrisRepository;
use LeaveManagement\Model\LeaveAssign;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Sql;

class LeaveAssignRepository extends HrisRepository {

    public function __construct(AdapterInterface $adapter, $tableName = null) {
        if ($tableName == null) {
            $tableName = LeaveAssign::TABLE_NAME;
        }
        parent::__construct($adapter, $tableName);
    }

    public function add(Model $model) {
        $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function edit(Model $model, $id) {
        $this->tableGateway->update($model->getArrayCopyForDB(), [LeaveAssign::LEAVE_ID => $id[0], LeaveAssign::EMPLOYEE_ID => $id[1]]);
              
        $boundedParameter = [];
        $boundedParameter['employeeId'] = $id[1];
        $boundedParameter['leaveId'] = $id[0];
       
        $sql="CALL
              HRIS_RECALCULATE_LEAVE(?,?);
            ";
        $this->executeStatement($sql, $boundedParameter);
    }

    public function fetchAll() {
        return $this->tableGateway->select();
    }

    public function fetchByEmployeeId($id) {
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->from(['A' => LeaveAssign::TABLE_NAME])
            ->join(['S' => 'HRIS_LEAVE_MASTER_SETUP'], 'A.LEAVE_ID=S.LEAVE_ID');
        $statement = $sql->prepareStatementForSqlObject($select);
        $result = $statement->execute();
        return $result;
    }

    public function filter($branchId, $departmentId, $genderId, $designationId, $serviceTypeId, $employeeId, $companyId, $positionId, $employeeTypeId, $leaveId): array {

        $searchCondition = EntityHelper::getSearchConditonBounded($companyId, $branchId, $departmentId, $positionId, $designationId, $serviceTypeId, null, $employeeTypeId, $employeeId, $genderId);

        $boundedParameter = [];
        $boundedParameter['leaveId1'] = $leaveId;
        $boundedParameter['leaveId2'] = $leaveId;
        $boundedParameter=array_merge($boundedParameter, $searchCondition['parameter']);

        $sql = "SELECT C.COMPANY_NAME,
                  B.BRANCH_NAME,
                  DEP.DEPARTMENT_NAME,
                  E.EMPLOYEE_ID,
                  E.EMPLOYEE_CODE,
                  E.FULL_NAME,
                  ELA.LEAVE_ID,
                  ELA.PREVIOUS_YEAR_BAL,
                  ELA.TOTAL_DAYS,
                  ELA.BALANCE,
                  LS.IS_MONTHLY,
                  MC.MONTH_EDESC
                FROM HRIS_EMPLOYEES E
                LEFT JOIN (SELECT * FROM HRIS_EMPLOYEE_LEAVE_ASSIGN WHERE LEAVE_ID   = ?) ELA
                ON (E.EMPLOYEE_ID = ELA.EMPLOYEE_ID)
                LEFT JOIN HRIS_LEAVE_MASTER_SETUP LS
                ON(LS.LEAVE_ID= ?)
               LEFT JOIN HRIS_LEAVE_MONTH_CODE MC ON
               (MC.LEAVE_YEAR_MONTH_NO=ELA.FISCAL_YEAR_MONTH_NO AND LS.LEAVE_YEAR=MC.LEAVE_YEAR_ID)
                LEFT JOIN HRIS_COMPANY C
                ON (E.COMPANY_ID=C.COMPANY_ID)
                LEFT JOIN HRIS_BRANCHES B
                ON (E.BRANCH_ID=B.BRANCH_ID)
                LEFT JOIN HRIS_DEPARTMENTS DEP
                ON (E.DEPARTMENT_ID=DEP.DEPARTMENT_ID)
                WHERE 1            =1 AND E.STATUS='E'
                {$searchCondition['sql']}
                    AND (CASE 
           WHEN ELA.FISCAL_YEAR_MONTH_NO IS NOT NULL THEN 
         (SELECT LEAVE_YEAR_MONTH_NO FROM HRIS_LEAVE_MONTH_CODE WHERE 
         (select 
            case when current_date>max(to_date) then
            max(to_date)
            else 
            current_date
            end
            from HRIS_LEAVE_MONTH_CODE) 
            BETWEEN FROM_DATE AND TO_DATE)
           END=ELA.FISCAL_YEAR_MONTH_NO 
          OR 
           CASE 
           WHEN ELA.FISCAL_YEAR_MONTH_NO IS  NULL THEN 
         1
         ELSE
         2
           END=1
          )
                ORDER BY C.COMPANY_NAME,B.BRANCH_NAME,DEP.DEPARTMENT_NAME,E.FULL_NAME,MC.LEAVE_YEAR_MONTH_NO";
        return $this->rawQuery($sql, $boundedParameter);
    }

    public function filterByLeaveEmployeeId($leaveId, $employeeId, $fiscalYearMonthNo = null) {
        $result = $fiscalYearMonthNo == null ? $this->tableGateway->select([LeaveAssign::LEAVE_ID => $leaveId, LeaveAssign::EMPLOYEE_ID => $employeeId]) : $this->tableGateway->select([LeaveAssign::LEAVE_ID => $leaveId, LeaveAssign::EMPLOYEE_ID => $employeeId, LeaveAssign::FISCAL_YEAR_MONTH_NO => $fiscalYearMonthNo]);

        return $result->current();
    }

    public function fetchById($id) {
        $rowset = $this->tableGateway->select([LeaveAssign::EMPLOYEE_LEAVE_ASSIGN_ID => $id]);
        return $rowset->current();
    }

    public function delete($id) {
        $this->tableGateway->delete([LeaveAssign::EMPLOYEE_LEAVE_ASSIGN_ID => $id]);
    }

    public function updatePreYrBalance($employeeId, $leaveId, $preYrBalance, $totalDays, $balance) {
        $this->tableGateway->update([LeaveAssign::PREVIOUS_YEAR_BAL => $preYrBalance, LeaveAssign::TOTAL_DAYS => $totalDays, LeaveAssign::BALANCE => $balance], [LeaveAssign::EMPLOYEE_ID => $employeeId, LeaveAssign::LEAVE_ID => $leaveId]);
    }
    
    
    public function editMonthlyLeave($employeeId,$leaveDetails,$monthId,$totalDays=null,$previousBalance=null){
        $boundedParams = [];
        $monthlyDays=($totalDays !=null )?$totalDays:$leaveDetails['DEFAULT_DAYS'];
        
         $sql = "CALL HRIS_ASSIGN_MONTHLY_LEAVE(
                $monthlyDays,
                {$leaveDetails['LEAVE_ID']},
                $monthId,
                $employeeId,
                '{$leaveDetails['CARRY_FORWARD']}',
                $previousBalance

            )";
         $this->executeStatement($sql);
    }

    public function checkEmployeeValidity($employeeId, $field, $values){

        $sql = "SELECT EMPLOYEE_ID FROM HRIS_EMPLOYEES WHERE EMPLOYEE_ID = {$employeeId} AND {$field} in ({$values})";

        return count(iterator_to_array($this->adapter->query($sql)->execute(), false)) == 0 ? false : true;
    }

    public function assignLeaveOnSetup($leaveMaster, $previousYearId){

        $conditionSql = "SELECT EMPLOYEE_ID FROM HRIS_EMPLOYEES WHERE STATUS='E' AND RETIRED_FLAG='N' AND RESIGNED_FLAG='N'";

        // Filter employees based on set conditions
        if($leaveMaster->branchId){
            $conditionSql = $conditionSql . " AND BRANCH_ID IN ({$leaveMaster->branchId})";
        }
        if($leaveMaster->departmentId){
            $conditionSql = $conditionSql . " AND DEPARTMENT_ID IN ({$leaveMaster->departmentId})";
        }
        if($leaveMaster->designationId){
            $conditionSql = $conditionSql . " AND DESIGNATION_ID IN ({$leaveMaster->designationId})";
        }
        if($leaveMaster->positionId){
            $conditionSql = $conditionSql . " AND POSITION_ID IN ({$leaveMaster->positionId})";
        }
        if($leaveMaster->serviceTypeId){
            $conditionSql = $conditionSql . " AND SERVICE_TYPE_ID IN ({$leaveMaster->serviceTypeId})";
        }
        if($leaveMaster->employeeType){
            $conditionSql = $conditionSql . " AND EMPLOYEE_TYPE IN ({$leaveMaster->employeeType})";
        }
        if($leaveMaster->genderId){
            $conditionSql = $conditionSql . " AND GENDER_ID IN ({$leaveMaster->genderId})";
        }
        if($leaveMaster->employeeId){
            $conditionSql = $conditionSql . " AND EMPLOYEE_ID IN ({$leaveMaster->employeeId})";
        }
        
        // Grab previous year balance and assign accordingly
        if($leaveMaster->isMonthly == 'N'){

            $prevBalQuery = "
                SELECT BALANCE INTO V_PREV_BALANCE DEFAULT 0 FROM hris_employee_leave_assign 
                WHERE EMPLOYEE_ID = E.EMPLOYEE_ID AND LEAVE_ID = {$previousYearId};
            ";

            $insertQuery = "

                IF '{$leaveMaster->isProdataBasis}' = 'N' THEN
                    V_BALANCE = CEIL(({$leaveMaster->defaultDays}/12) * (13-V_MONTH_NO));
                ELSE
                    V_BALANCE = {$leaveMaster->defaultDays};
                END IF;

                insert into HRIS_EMPLOYEE_LEAVE_ASSIGN 
                (employee_id, leave_id, previous_year_bal, total_days, balance, created_dt)
                values(e.employee_id, {$leaveMaster->leaveId}, V_PREV_BALANCE, V_BALANCE, 
                V_PREV_BALANCE + V_BALANCE, current_date);    
            ";
        }else{

            $prevBalQuery = "
                select balance into v_prev_balance default 0 from hris_employee_leave_assign
                where employee_id = e.employee_id and leave_id = {$previousYearId} and fiscal_year_month_no = 
                (select max(fiscal_year_month_no) from hris_employee_leave_assign where employee_id = e.employee_id and leave_id = {$previousYearId});
            ";

            // Ensures that people with latest join date will only get calculated number of monthly holidays
            $insertQuery = "
                CALL HRIS_ASSIGN_MONTHLY_LEAVE(
                    {$leaveMaster->defaultDays},
                    {$leaveMaster->leaveId},
                    V_MONTH_NO,
                    E.EMPLOYEE_ID,
                    '{$leaveMaster->carryForward}',
                    V_PREV_BALANCE
                );
            ";
        }

        if($previousYearId == null){
            $prevBalQuery = '';
        }

        $insertQuery = "
            DO
            BEGIN

                DECLARE V_PREV_BALANCE DECIMAL(7) DEFAULT 0;
                DECLARE V_MONTH_NO INTEGER DEFAULT 1;
                DECLARE V_JOIN_DATE DATE DEFAULT CURRENT_DATE;
                DECLARE V_START_DATE DATE DEFAULT CURRENT_DATE;
                DECLARE V_END_DATE DATE DEFAULT CURRENT_DATE;
                DECLARE V_BALANCE INTEGER DEFAULT 0;
                
                DECLARE CURSOR EMPLOYEES FOR
                    {$conditionSql};

                select start_date into v_start_date default current_date from hris_leave_years 
                where leave_year_id = {$leaveMaster->fiscalYear} and status = 'E';
                
                select end_date into v_end_date default current_date from hris_leave_years 
                where leave_year_id = {$leaveMaster->fiscalYear} and status = 'E';
                    
                FOR E AS EMPLOYEES DO

                    select join_date into v_join_date from hris_employees where employee_id = e.employee_id and status = 'E';

                    select month_no into v_month_no default 1 from hris_month_code where
                    v_join_date between from_date and to_date
                    and v_join_date between v_start_date and v_end_date;

                    {$prevBalQuery}

                    SELECT JOIN_DATE INTO V_JOIN_DATE DEFAULT CURRENT_DATE FROM HRIS_EMPLOYEES WHERE EMPLOYEE_ID = E.EMPLOYEE_ID AND STATUS = 'E';

                    {$insertQuery}
                
                END FOR;
                
            END;
        ";

        // echo "<pre>";
        // print_r($insertQuery); die;
        
        $this->adapter->query($insertQuery)->execute();
        
    }

    public function assignLeaveAndHolidayOnEmployeeSetup($leaveIds, $employeeId){

        $sql = "
                DO
                BEGIN
                DECLARE v_fiscal_year_id NUMBER(7, 0);
                DECLARE v_month_id NUMBER(7, 0);
                DECLARE v_current_month_count NUMBER(7);
                DECLARE v_default_days NUMBER(7, 0);
                DECLARE v_prodata_days NUMBER(7, 0);
                
                DECLARE CURSOR leaves FOR 
                SELECT leave_id, default_days, is_prodata_basis, is_monthly, carry_forward 
                FROM hris_leave_master_setup
                WHERE LEAVE_ID IN ($leaveIds);
                
                -- Cursor for holidays
                DECLARE CURSOR holidays FOR
                SELECT holiday_id
                FROM hris_holiday_master_setup
                WHERE assign_on_employee_setup = 'Y' AND status = 'E' AND start_date >= current_date;

                delete from hris_employee_leave_assign where employee_id = {$employeeId};
                delete from hris_employee_holiday where employee_id = {$employeeId};
                
                SELECT leave_year_id, month_id
                INTO v_fiscal_year_id, v_month_id
                FROM hris_leave_month_code
                WHERE current_date BETWEEN from_date AND TO_DATE;
                
                SELECT month_no
                INTO v_current_month_count
                FROM (SELECT month_no, month_id
                    FROM hris_leave_month_code
                    WHERE leave_year_id = v_fiscal_year_id
                    ORDER BY from_date) months
                WHERE month_id = v_month_id;
            
                -- Cursor for leaves
            
                FOR leave AS leaves DO
                
                v_default_days := leave.default_days;
                
                if leave.is_monthly='N' then
                    v_prodata_days := leave.default_days;
                
                    IF leave.is_prodata_basis = 'Y' THEN
                        v_prodata_days := ROUND(v_default_days * ( (13 - v_current_month_count) / 12));
                    END IF;
                
                    INSERT INTO hris_employee_leave_assign (employee_id, leave_id, previous_year_bal, total_days, balance, created_dt)
                    VALUES ({$employeeId}, leave.leave_id, 0, v_prodata_days, v_prodata_days, current_date);
                
                else
                
                    CALL HRIS_ASSIGN_MONTHLY_LEAVE(
                                v_default_days,
                                leave.leave_id,
                                v_current_month_count,
                                {$employeeId},
                                leave.carry_forward,
                                0
                            );
                
                end if;
                
                END FOR;
            
            
                FOR holiday AS holidays DO
                INSERT INTO hris_employee_holiday (employee_id, holiday_id)
                VALUES ({$employeeId}, holiday.holiday_id);
                END FOR;
            
            END;
        ";

        $this->adapter->query($sql)->execute();

    }

    public function currentYear(){

        $sql = "select fiscal_year_id from hris_fiscal_years where current_date between start_date and end_date and status = 'E'";

        return $this->adapter->query($sql)->execute()->current()['FISCAL_YEAR_ID']; 

    }
    
}
