<?php
namespace Loan;

use Application\Controller\ControllerFactory;
use Loan\Controller\LoanStatus;
use Loan\Controller\LoanApply;
use Loan\Controller\LoanCashPayment;
use Loan\Controller\LoanReport;
use Loan\Controller\LoanDocument;
use Loan\Controller\LoanStatusTest;
use Loan\Controller\LoanCashPaymentTest;
use Zend\Router\Http\Segment;

return [
    'router' => [
        'routes' => [
            'loanStatus' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loan/status[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanStatus::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanApply' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loan/apply[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanApply::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanCashPayment' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loan/cash[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanCashPayment::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanReport' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loan/report[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanReport::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanDocument' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loan/document[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanDocument::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanStatusTest' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loantest/status[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanStatusTest::class,
                        'action' => 'index'
                    ],
                ],
            ],
            'loanCashPaymentTest' => [
                'type' => Segment::class,
                'options' => [
                    'route' => '/loantest/cash[/:action[/:id]]',
                    'constraints' => [
                        'action' => '[a-zA-Z][a-zA-Z0-9_-]*',
                        'id' => '[0-9]+',
                    ],
                    'defaults' => [
                        'controller' => LoanCashPaymentTest::class,
                        'action' => 'index'
                    ],
                ],
            ],

        ],
    ],
    'navigation' => [
        'loanStatus' => [
                [
                'label' => "Loan Request",
                'route' => "loanStatus"
            ],
                [
                'label' => "Loan Request",
                'route' => "loanStatus",
                'pages' => [
                        [
                        'label' => 'List',
                        'route' => 'loanStatus',
                        'action' => 'index',
                    ],
                        [
                        'label' => 'Add',
                        'route' => 'loanStatus',
                        'action' => 'add',
                    ],
                        [
                        'label' => 'Detail',
                        'route' => 'loanStatus',
                        'action' => 'view',
                    ],
                ],
            ],
        ],
        'loanApply' => [
                [
                'label' => "Loan Apply",
                'route' => "loanApply"
            ],
                [
                'label' => "Loan Apply",
                'route' => "loanApply",
                'pages' => [
                        [
                        'label' => 'List',
                        'route' => 'loanApply',
                        'action' => 'index',
                    ],
                        [
                        'label' => 'Add',
                        'route' => 'loanApply',
                        'action' => 'add',
                    ],
                        [
                        'label' => 'Edit',
                        'route' => 'loanApply',
                        'action' => 'edit',
                    ],
                ],
            ],
        ],
        'loanStatusTest' => [
                [
                'label' => "Loan Request",
                'route' => "loanStatusTest"
            ],
                [
                'label' => "Loan Request",
                'route' => "loanStatusTest",
                'pages' => [
                        [
                        'label' => 'List',
                        'route' => 'loanStatusTest',
                        'action' => 'index',
                    ],
                        [
                        'label' => 'Add',
                        'route' => 'loanStatusTest',
                        'action' => 'add',
                    ],
                        [
                        'label' => 'Detail',
                        'route' => 'loanStatusTest',
                        'action' => 'view',
                    ],
                ],
            ],
        ],
    ],
    'controllers' => [
        'factories' => [
            Controller\LoanStatus::class => ControllerFactory::class,
            Controller\LoanApply::class => ControllerFactory::class,
            Controller\LoanReport::class => ControllerFactory::class,
            Controller\LoanDocument::class => ControllerFactory::class,
            Controller\LoanCashPayment::class => ControllerFactory::class,
            Controller\LoanCashPaymentTest::class => ControllerFactory::class,
            Controller\LoanStatusTest::class => ControllerFactory::class,
        ],
    ],
    'view_manager' => [
        'template_path_stack' => [
            __DIR__ . '/../view',
        ],
    ],
];
