<?php

namespace Loan\Controller; 

use Application\Controller\HrisController;
use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Application\Model\FiscalYear;
use Application\Model\HrisQuery;
use SelfService\Form\LoanRequestForm;
use SelfService\Model\LoanRequest;
use Exception;
use SelfService\Model\LoanEmiDetail;
use Setup\Model\Loan;
use Loan\Repository\LoanReportRepository;
use Loan\Repository\LoanStatusRepository;
use ManagerService\Repository\LoanApproveRepository;
use SelfService\Repository\LoanRequestRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Form\Annotation\AnnotationBuilder;
use Zend\Db\Sql\Select as Select2;
use Zend\Form\Element\Select; 
use Zend\View\Model\JsonModel;
use Application\Helper\EntityHelper as ApplicationHelper;

class LoanReport extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(LoanReportRepository::class);
        $this->loanApproveRepository = new LoanApproveRepository($adapter);
        $this->loanRequestRepository = new LoanRequestRepository($adapter);
        $this->loanStatusRepository = new LoanStatusRepository($adapter);


    }

    public function initializeForm() {
        $builder = new AnnotationBuilder();
        $form = new LoanRequestForm();
        $this->form = $builder->createForm($form);
    }

    public function indexAction(){
        $request = $this->getRequest();
        if ($request->isPost()){
            try {
                $data = $request->getPost();
                $result = $this->repository->fetchEmployeeLoanDetails($data);
                $loanDetails = Helper::extractDbData($result);
                return new JsonModel(['success' => true, 'data' => $loanDetails, 'message' => null]);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
            }
        } 
        
        return $this->stickFlashMessagesTo([
                'searchValues' => ApplicationHelper::getSearchData($this->adapter),
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'preference' => $this->preference
        ]);
    }

    public function loanVoucherAction(){
        $request = $this->getRequest();
        if ($request->isPost()){
            try {
                $emp_id = !empty($_POST['emp_id']) ? $_POST['emp_id'] : null ;
                $fromDate = !empty($_POST['fromDate']) ? $_POST['fromDate'] : null ;
                $toDate = !empty($_POST['toDate']) ? $_POST['toDate'] : null ;
                $loanId = !empty($_POST['loanId']) ? $_POST['loanId'] : null ;
                //$loan_id = !empty($_POST['loan_id']) ? $_POST['loan_id'] : null ;
                $result = $this->repository->fetchLoanVoucher($emp_id, $fromDate, $toDate, $loanId);
                $loanVoucherDetails = Helper::extractDbData($result);
                
                $fromDate = '1-Jul-'.date('Y', strtotime($fromDate));
                $result = $this->repository->fetchOpeningBalance($emp_id, $fromDate, $loanId);
                $openingBalanceDetails = Helper::extractDbData($result);
                $openingBalanceDetails[0]["OPENING_BALANCE"] = $openingBalanceDetails[0]["OPENING_BALANCE"] == null ? "0.00" : $openingBalanceDetails[0]["OPENING_BALANCE"];
                array_unshift($loanVoucherDetails, ["DT" => $fromDate, 
                "PARTICULARS" => "Opening Balance", 
                "DEBIT_AMOUNT" =>  $openingBalanceDetails[0]["OPENING_BALANCE"],
                "CREDIT_AMOUNT" => "0",
                "BALANCE" => "0"]);
                return new JsonModel(['success' => true, 'data' => $loanVoucherDetails, 'balanceData' => $openingBalanceDetails, 'message' => null]);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
            }
        } 
        $loanList = $this->repository->getLoanlist();
        $loanList = Helper::extractDbData($loanList);
        
        return $this->stickFlashMessagesTo([
                'searchValues' => ApplicationHelper::getSearchData($this->adapter),
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'loanList' => $loanList,
                'preference' => $this->preference
        ]);
    }

    public function cashPaymentReportAction(){
        $loanFormElement = new Select();
        $loanFormElement->setName("loan");
        $loans = EntityHelper::getTableKVListWithSortOption($this->adapter, Loan::TABLE_NAME, Loan::LOAN_ID, [Loan::LOAN_NAME], [Loan::STATUS => 'E'], Loan::LOAN_NAME, "ASC", NULL, FALSE, TRUE);
        $loans1 = [-1 => "All Loans"] + $loans;
        $loanFormElement->setValueOptions($loans1);
        $loanFormElement->setAttributes(["id" => "loanId", "class" => "form-control reset-field"]);
        $loanFormElement->setLabel("Loan Type");

        $loanStatus = [
            '-1' => 'All Status',
            'RQ' => 'Pending',
            'RC' => 'Recommended',
            'AP' => 'Approved',
            'R' => 'Rejected',
            'C' => 'Cancelled'
        ];
        $loanStatusFormElement = new Select();
        $loanStatusFormElement->setName("loanStatus");
        $loanStatusFormElement->setValueOptions($loanStatus);
        $loanStatusFormElement->setAttributes(["id" => "loanRequestStatusId", "class" => "form-control reset-field"]);
        $loanStatusFormElement->setLabel("Status");

        return Helper::addFlashMessagesToArray($this, [
                    'loans' => $loanFormElement,
                    'loanStatus' => $loanStatusFormElement,
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'preference' => $this->preference
        ]);
    }

    public function pullCashPaymentListAction(){
        $request = $this->getRequest();
        $postedData = $getData = $request->getPost();
        $data = Helper::extractDbData($this->repository->getCashPaymentsList($postedData));
        return new JSONModel(['success' => true, 'data' => $data, 'message' => null]);
    }

    public function loanSummaryAction(){
        $request = $this->getRequest();
        if ($request->isPost()){
            try {
                $emp_id = !empty($_POST['emp_id']) ? $_POST['emp_id'] : null ;
                $fromDate = !empty($_POST['fromDate']) ? $_POST['fromDate'] : null ;
                $toDate = !empty($_POST['toDate']) ? $_POST['toDate'] : null ;
                $loanId = !empty($_POST['loanId']) ? $_POST['loanId'] : null ;
                $data = [];
                
                foreach($emp_id as $eid){
                    $result = $this->repository->fetchLoanSummary($eid, $fromDate, $toDate, $loanId);
                    $loanSummary = Helper::extractDbData($result);
                    array_push($data, $loanSummary[0]);
                }
                
                return new JsonModel(['success' => true, 'data' => $data, 'message' => null]);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
            }
        } 
        $loanList = $this->repository->getLoanlist();
        $loanList = Helper::extractDbData($loanList);
        
        $searchValues = ApplicationHelper::getSearchData($this->adapter);

        $allEmployees = Helper::extractDbData($this->repository->getAllEmployees());

        $searchValues['employee'] = $allEmployees;

        return $this->stickFlashMessagesTo([
                'searchValues' => $searchValues,
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'loanList' => $loanList,
                'preference' => $this->preference
        ]);
    }

    public function loanPaymentReportAction(){
        $request = $this->getRequest();
        if ($request->isPost()){
            try {
                $data = $request->getPost();
                $result = Helper::extractDbData($this->repository->getLoanPaymentReport($data));
                return new JsonModel(['success' => true, 'data' => $result, 'message' => null]);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
            }
        } 
        $loanList = Helper::extractDbData($this->repository->getLoanlist());
        $searchValues = ApplicationHelper::getSearchData($this->adapter);
        $$searchValues['employee'] = Helper::extractDbData($this->repository->getAllEmployees());

        return $this->stickFlashMessagesTo([
                'searchValues' => $searchValues,
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'loanList' => $loanList,
                'preference' => $this->preference
        ]);
    }

    public function detailsAction(){ 
        $this->initializeForm();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0) {
            return $this->redirect()->toRoute("loanStatus");
        }
        $loanRequest = new LoanRequest();
        $request = $this->getRequest();



        $detail     = $this->loanApproveRepository->fetchById($id);
        $detailSelf = $this->loanRequestRepository->fetchById($id);

        // echo "<pre>";
        // print_r($detailSelf);
        // die;

        $status = $detail['STATUS'];
        $employeeId = $detail['EMPLOYEE_ID'];
        $loanId = $detail['LOAN_REQUEST_ID'];


        $paidEmi  = $this->loanApproveRepository->getEmiPaidDetail($loanId, $employeeId);
        // echo "<pre>";
        // print_r($detail);
        // die;


        $recommApprove = $detail['RECOMMENDER_ID'] == $detail['APPROVER_ID'] ? 1 : 0;
        $employeeName  = $detail['FULL_NAME'];
        $authRecommender = $detail['RECOMMENDED_BY_NAME'] == null ? $detail['RECOMMENDER_NAME'] : $detail['RECOMMENDED_BY_NAME'];
        $authApprover = $detail['APPROVED_BY_NAME'] == null ? $detail['APPROVER_NAME'] : $detail['APPROVED_BY_NAME'];

        if (!$request->isPost()) {
            $loanRequest->exchangeArrayFromDB($detailSelf);
            if($detailSelf['INTEREST_RATE']==0){
                $loanRequest->interestRate = 0;
            }
            if($detailSelf['MONTHLY_INTEREST_RATE']==0){
                $loanRequest->monthlyInterestRate = 0;
            }
            $this->form->bind($loanRequest);
        } else {
            $getData = $request->getPost();
            $reason = $getData->approvedRemarks;
            $action = $getData->submit;
            $loanRequest->approvedDate = Helper::getcurrentExpressionDate();
            if ($action == "Reject") {
                $loanRequest->status = "R";
                $this->flashmessenger()->addMessage("Loan Request Rejected!!!");
            } else if ($action == "Approve") {
                $loanRequest->status = "AP";
                $this->flashmessenger()->addMessage("Loan Request Approved");
            }
            $loanRequest->approvedBy = $this->employeeId;
            $loanRequest->approvedRemarks = $reason;
            $this->loanApproveRepository->edit($loanRequest, $id);
            $this->loanApproveRepository->addToDetails($id);
            $loanRequest->loanRequestId = $id;
            $loanRequest->exchangeArrayFromDB($detail);
            // $this->financialImpact($loanRequest);
 
            return $this->redirect()->toRoute("loanStatus");
        }

        $loanDetailView  = $this->loanRequestRepository->fetchLoanDetailView($id);
        // echo "<pre>";print_r($loanDetailView);die;
        $months = EntityHelper::getTableKVList($this->adapter, "HRIS_MONTH_CODE", "MONTH_ID", ["MONTH_EDESC"], ["FISCAL_YEAR_ID = 8"],null,true,'MONTH_EDESC','desc');
        $years=EntityHelper::getTableKVList($this->adapter, "HRIS_FISCAL_YEARS", "FISCAL_YEAR_ID", ["FISCAL_YEAR_NAME"], null,null,false,'FISCAL_YEAR_ID','desc');

        return Helper::addFlashMessagesToArray($this, [
                    'form' => $this->form,
                    'id' => $id,
                    'detail'=> $detail,
                    'loanDetails'=>$detailSelf,
                    'employeeId' => $employeeId,
                    'employeeName' => $employeeName,
                    'requestedDt' => $detail['REQUESTED_DATE'],
                    'recommender' => $authRecommender,
                    'approver' => $authApprover,
                    'approvedDT' => $detail['APPROVED_DATE'],
                    'status' => $status,
                    'loans' => EntityHelper::getTableKVListWithSortOption($this->adapter, Loan::TABLE_NAME, Loan::LOAN_ID, [Loan::LOAN_NAME], [Loan::STATUS => "E"], Loan::LOAN_ID, "ASC", NULL, FALSE, TRUE),
                    'customRenderer' => Helper::renderCustomView(),
                    'recommApprove' => $recommApprove,
                    'loanDetailView' => $loanDetailView,
                    'month' => $months,
                    'year' => $years,
                    'paidEmi' => $paidEmi
        ]);
    }


    function groupAction() {
 
        $request = $this->getRequest();

        $loanDetails = Helper::extractDbData($this->loanRequestRepository->getLoanDetails());


        // $result      = $this->repository->fetchAllGroupLoanMonthExtra('');

        
        // $loanGroup = $this->repository->fetchAllGroupLoanMonth();


        $current_month    = $this->repository->getCurrentMonth();
        $months           = $this->repository->getCurrentFiscalYearMonth($current_month['FISCAL_YEAR_ID']);

        $fiscalYearDetail  = $this->loanRequestRepository->getCurrentFiscalYearsDetail();
        $fiscalYearDetails = Helper::extractDbData($this->loanRequestRepository->getCurrentFiscalYearsDetails());
        //FISCAL_YEAR_NAME    FISCAL_YEAR_ID
        // $current_month_map_id  = Helper::monthToIndex($current_month['MONTH_EDESC']);

        if ($request->isPost()){
        
            try {

                $data        = $request->getPost();
                $result      = $this->repository->fetchAllGroupLoanMonth($data);
                // $result      = 'test';
                 // $this->repository->fetchEmployeeLoanDetails($data);
                $loanDetails = Helper::extractDbData($result);
                return new JsonModel(['success' => true, 'data' => $loanDetails, 'message' => null]);
        
            } catch (Exception $e) {
        
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        
            }
        
        } 
        
        return $this->stickFlashMessagesTo([
                'searchValues' => ApplicationHelper::getSearchData($this->adapter),
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'preference' => $this->preference,
                'loanDetails' => $loanDetails,
                'fiscalYearDetail' => $fiscalYearDetail,
                'fiscalYearDetails'=> $fiscalYearDetails,
                'months'=> $months
        ]);

    }

    public function exploreAction()
    {
        $file = substr(realpath(dirname(__DIR__)),0,-3).'file\house_loan_77_78.json';


        // echo "here"; die;


        if (file_exists($file)) {

            echo('Start Time : ' . microtime());
            $getFile = file_get_contents($file);

            $allData = json_decode($getFile);

            for ($i=5; $i < count($allData); $i++) {
                    
                $data = Helper::getEmployeeID($this->adapter,$allData[$i]->D);

                $list .= ($data['EMPLOYEE_ID']) ? $data['EMPLOYEE_ID'].',' : '';
                
            }

            $list = substr($list,0,-1);
            // $month_id = '89,90,91,92,93';
            $month_id = '93';
            
            // GETTING SHEET DATA
            $salaryDetail = $this->repository->salaryDetail($month_id, 82, $list);

            // echo "<pre>";
            // print_r($salaryDetail);
            // die;

            for ($i=0; $i < count($salaryDetail) ; $i++) { 
                
                $data = [
                    'EMPLOYEE_ID' => $salaryDetail[$i]['EMPLOYEE_ID'],
                    'EMI' => $salaryDetail[$i]['VAL'],
                    'LOAN_REQUEST_ID' => $salaryDetail[$i]['LOAN_REQUEST_ID']
                ];

                /**
                 * GET UNPAID EMI ROW DETAIL
                 */
                $unpaidEmiDetail = $this->repository->unpaidEmiDetail($data);
                $paidEmiDetail   = $this->repository->paidEmiDetail($data);

                // echo "<pre>";
                // print_r($paidEmiDetail);
                // echo "<br>";
                // die;

                $previousPaidMonthNo = $unpaidEmiDetail['PREVIOUS_MONTH_NO'];
                // echo "<br>";
                $givenMonthId = $salaryDetail[$i]['MONTH_ID'];
                // echo "<br>";

                if ($givenMonthId > $previousPaidMonthNo) {

                    $emiID = $unpaidEmiDetail['PAID_EMI_ID'] + ($givenMonthId - $previousPaidMonthNo);

                    // if ($unpaidEmiDetail['PREVIOUS_MONTH_NO'] !== $salaryDetail[$i]['MONTH_ID']) {

                        $updateData = [
                            'PAID_FLAG' => 'Y',
                            'PAYMENT_MONTH' => $salaryDetail[$i]['MONTH_ID'],
                            'SHEET_NO'=> $salaryDetail[$i]['SHEET_NO'],
                            'EMI_ID'  => $emiID
                        ];
    
                        // echo "<pre>";
                        // print_r($updateData);
                        // die;
    
                        $this->repository->updateEMIDetail($updateData);
    
                        /**
                         * GET PAYMENT DETAIL
                         */
                        $data['EMI_ID'] = $updateData['EMI_ID'];
    
                        // echo "<pre>";
                        // print_r($data);
                        // die;
                        $paymentDetail = $this->repository->paymentDetail($data);
    
                        if (!$paymentDetail) {
    
                            // echo "inside here";
                            // die;
                            
                            $insertData = [
                                'LOAN_REQUEST_ID' => $data['LOAN_REQUEST_ID'],
                                'SNO' => $unpaidEmiDetail['SNO'],
                                'FROM_DATE'=> $salaryDetail[$i]['FROM_DATE'],
                                'TO_DATE'=> $salaryDetail[$i]['TO_DATE'],
                                'AMOUNT'=> $unpaidEmiDetail['INSTALLMENT'],
                                'PAID_FLAG' => 'Y',
                                'MONTH_NO' => $updateData['PAYMENT_MONTH'],
                                'INTEREST_AMOUNT' => $unpaidEmiDetail['INTEREST'],
                                'PRINCIPLE_AMOUNT' => $unpaidEmiDetail['PRINCIPAL_REPAID'],
                                'DAYS'=> $salaryDetail[$i]['DAYS'],
                                'SHEET_NO' => $updateData['SHEET_NO'],
                                'CREATED_DT' => $salaryDetail[$i]['CREATED_DT'],
                                'EMI_ID' => $data['EMI_ID'],
                                'EMPLOYEE_ID' => $data['EMPLOYEE_ID']
                            ];
    
                            $this->repository->insertEMIDetail($insertData);
    
    
                        }
    
                    // }

                } 
                // else {

                //     $emiID = $paidEmiDetail['EMI_ID'] - ($givenMonthId - $previousPaidMonthNo);
                //     $paidEmiDetail = $this->repository->paidEmiDetail($data);
                //     $updateData = [
                //         'SHEET_NO'=> $salaryDetail[$i]['SHEET_NO'],
                //         'EMI_ID'  => $unpaidEmiDetail
                //     ];

                //     // echo "<pre>";
                //     // print_r($updateData);
                //     // die;

                //     $this->repository->updateEMIDetail($updateData);

                //     /**
                //      * GET PAYMENT DETAIL
                //      */
                //     $data['EMI_ID'] = $updateData['EMI_ID'];

                //     // echo "<pre>";
                //     // print_r($data);
                //     // die;
                //     $paymentDetail = $this->repository->paymentDetail($data);

                //     if (!$paymentDetail) {

                //         // echo "inside here";
                //         // die;
                        
                //         $insertData = [
                //             'LOAN_REQUEST_ID' => $data['LOAN_REQUEST_ID'],
                //             'SNO' => $unpaidEmiDetail['SNO'],
                //             'FROM_DATE'=> $salaryDetail[$i]['FROM_DATE'],
                //             'TO_DATE'=> $salaryDetail[$i]['TO_DATE'],
                //             'AMOUNT'=> $unpaidEmiDetail['INSTALLMENT'],
                //             'PAID_FLAG' => 'Y',
                //             'MONTH_NO' => $updateData['PAYMENT_MONTH'],
                //             'INTEREST_AMOUNT' => $unpaidEmiDetail['INTEREST'],
                //             'PRINCIPLE_AMOUNT' => $unpaidEmiDetail['PRINCIPAL_REPAID'],
                //             'DAYS'=> $salaryDetail[$i]['DAYS'],
                //             'SHEET_NO' => $updateData['SHEET_NO'],
                //             'CREATED_DT' => $salaryDetail[$i]['CREATED_DT'],
                //             'EMI_ID' => $data['EMI_ID'],
                //             'EMPLOYEE_ID' => $data['EMPLOYEE_ID']
                //         ];

                //         $this->repository->insertEMIDetail($insertData);


                //     }

                // }

                // echo 'no smaller'; die;
                

            }




            echo "<pre>";
            echo('<br>End Time : ' . microtime());
            print_r('ends here');
            die;

            // echo count($allData); die;


            echo "<table style='border: 1px solid #000;'>";
            echo "<thead><tr>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>SN</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Employee Name</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Employee_ID</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Employee_code</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Employee_code_excel</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Exist in DB</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>loan Date</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>loan Amount</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Interest</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Interest Rate (Monthly)</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Monthly Installment</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Debit Balance 2078/79</th>";
            echo "<th style='border: 1px solid #000; padding: 4px;'>Credit Balance 2078/79</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Shrawan</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Bhadra</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Ashwin</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Kartik</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Mangsir</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Paush</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Magh</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Falgun</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Chaitra</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Bhaisakh</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Jestha</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Ashad</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Total Installment</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Debit Balance</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Credit Balance</th>";
            // echo "<th style='border: 1px solid #000; padding: 4px;'>Interest 2077/78</th>";
            echo "</tr></thead>";
            echo "<tbody>";
                


                for ($i=5; $i < count($allData); $i++) { 

                    $data = Helper::getEmployeeID($this->adapter,$allData[$i]->D);
                    $isLoanTaken = Helper::checkEmployeeLoanID($this->adapter, $data['EMPLOYEE_ID'], 1);

                    // if ($isLoanTaken) { echo 'taken';} else {echo "not taken";}

                    // die;

                    if ($allData[$i]->C !== 'भेटिएन') {
                // foreach ($allData as $data) {
                    echo "<tr>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->A."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->C."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$data['EMPLOYEE_ID']."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$data['EMPLOYEE_CODE']."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->D."</td>";
                        if ($isLoanTaken) {
                            echo "<td style='border: 1px solid #000; padding: 4px;'>Taken</td>";
                        } else {
                            echo "<td style='border: 1px solid #000; padding: 4px;'>Not Taken</td>";
                        }
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".Helper::getExcelDate($allData[$i]->G)."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->H."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>2%</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".((2/100)/12)."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->I."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->AC."</td>";
                        echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->AD."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->O."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->P."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->Q."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->R."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->S."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->T."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->U."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->V."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->W."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->X."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->Y."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->Z."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->AA."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->AB."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->AC."</td>";
                        // echo "<td style='border: 1px solid #000; padding: 4px;'>".$allData[$i]->K."</td>";

                    echo "</tr>";

                    }
                }
            echo "</tbody>";
            echo "</table>";

            die;

            // echo "<pre>"; print_r(json_decode($getFile)); die;

        } else {

            echo "no file exist"; die;

        }
       
       

        

        // return $this->stickFlashMessagesTo([
        //         'searchValues' => ApplicationHelper::getSearchData($this->adapter),
        //         'acl' => $this->acl,
        //         'employeeDetail' => $this->storageData['employee_detail'],
        //         'preference' => $this->preference
        // ]);
    }


    public function loanWiseReportAction()
    {
        $request = $this->getRequest();

        $loanDetails = Helper::extractDbData($this->loanRequestRepository->getLoanDetails());

        $current_month    = $this->repository->getCurrentMonth();
        $months           = $this->repository->getCurrentFiscalYearMonth($current_month['FISCAL_YEAR_ID']);

        $fiscalYearDetail  = $this->loanRequestRepository->getCurrentFiscalYearsDetail();
        $fiscalYearDetails = Helper::extractDbData($this->loanRequestRepository->getCurrentFiscalYearsDetails());
        //FISCAL_YEAR_NAME    FISCAL_YEAR_ID
        // $current_month_map_id  = Helper::monthToIndex($current_month['MONTH_EDESC']);

        if ($request->isPost()){

            try {

                $data        = $request->getPost();
                $result      = $this->repository->fetchAllLoanWise($data);

                $loanDetails = Helper::extractDbData($result);
                return new JsonModel(['success' => true, 'data' => $result, 'message' => null]);
        
            } catch (Exception $e) {
        
                return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        
            }
        
        } 
        
        return $this->stickFlashMessagesTo([
                'searchValues' => ApplicationHelper::getSearchData($this->adapter),
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'preference' => $this->preference,
                'loanDetails' => $loanDetails,
                'fiscalYearDetail' => $fiscalYearDetail,
                'fiscalYearDetails'=> $fiscalYearDetails,
                'months'=> $months
        ]);
    }

    public function pullLoanListIssuedByAction() {
        try {
            $request = $this->getRequest();
            $data    = $request->getPost();

            $issuedBy = $data['issuedBy'];

            $loanList = $this->repository->getLoanlistBy($issuedBy);

            return new JsonModel(['success' => true, 'data' => $loanList, 'message' => null]); 

        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function importAction() {  

        $request = $this->getRequest();
        
        if ($request->isPost()){
        
            try {

                $data = $request->getPost()['data'];

                $log = [];

                // Samajik loan
                $loanId = 2;
                $mthId = 39;

                // var_dump($data); die;

                for($i=0; $i<count($data); $i++){

                    $name = $data[$i]['A'];
                    $employeeCode = $data[$i]['B'];
                    $balance = $data[$i]['C'];

                    // $employeeCode = 931;
                    // $balance = 3808950.56;

                    if($employeeCode == null){
                        array_push($log, ['name'=>$name, 'message'=>'Employee code is empty']);
                        continue;
                    }

                    $employeeId = $this->repository->getEmployeeId($employeeCode); 

                    if($employeeId == null){
                        array_push($log, ['name'=>$name, 'message'=>'Employee id not found for ' . $name]);
                        continue;
                    }

                    $emi = $this->repository->getInstallment($employeeId, $mthId);
                    
                    if($emi==0 || $emi == null){
                        array_push($log,['name'=>$name,'message'=>'EMI either less than balance, 0 or not found EMI: '.$emi]);
                        continue;
                    }

                    $emiCount = ceil(($balance / $emi));

                    if(!((int)$balance < 1)){
                        
                        // Insert loan request
                        $loan = new LoanRequest();
                        $loan->loanRequestId = ((int) Helper::getMaxId($this->adapter, LoanRequest::TABLE_NAME, LoanRequest::LOAN_REQUEST_ID)) + 1;
                        $loan->employeeId = $employeeId;
                        $loan->loanId = $loanId;//social activity loan
                        $loan->requestedAmount = round($balance, 2);
                        $loan->reason = 'Manual Insertion';
                        $loan->status = 'AP';
                        $loan->deductOnSalary = 'Y';

                        $this->loanRequestRepository->add($loan);

                        // Insert EMI
                        

                        $emiDetail = new LoanEmiDetail();

                        $loanAmount = $balance;

                        for($j=0; $j<$emiCount; $j++){

                            $emiDetail->emiId = ((int) Helper::getMaxId($this->adapter, LoanEmiDetail::TABLE_NAME, LoanEmiDetail::EMI_ID)) + 1;

                            $emiDetail->loanRequestId = $loan->loanRequestId;
                            $emiDetail->employeeId = $employeeId;
                            $emiDetail->repaymentInstallments = $emiCount - $j;
                            $emiDetail->loanAmount = $loanAmount;
                            $emiDetail->installment = $j == ($emiCount-1) ? $loanAmount : $emi;
                            $loanAmount = $loanAmount - $emiDetail->installment;
                            $emiDetail->interest = 0;
                            $emiDetail->principalRepaid = $emiDetail->installment;
                            $emiDetail->remainingPrincipal = $loanAmount;
                            $emiDetail->status = 'AP';
                            $emiDetail->createdDt = Helper::getcurrentExpressionDate();
                            $emiDetail->paidFlag = 'N';
                            $emiDetail->sno = $j + 1;
                            $emiDetail->fiscalYearId = 9;

                            $this->loanRequestRepository->emiAdd($emiDetail);

                        }

                    }
                }

                // $result      = $this->repository->fetchEmployeeLoanDetails($data);
                // $loanDetails = Helper::extractDbData($result);
                
                return new JsonModel(['success' => true, 'log'=>$log]);
        
            } catch (Exception $e) {
        
                return new JsonModel(['success' => false, 'data' => $name, 'message' => $e->getMessage()]);
        
            }
        
        } 
        
        return $this->stickFlashMessagesTo([
                'searchValues' => ApplicationHelper::getSearchData($this->adapter),
                'acl' => $this->acl,
                'employeeDetail' => $this->storageData['employee_detail'],
                'preference' => $this->preference
        ]);
    }
}
