<?php

namespace ManagerService\Controller;

use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Application\Helper\LoanAdvanceHelper;
use Exception;
use Loan\Repository\LoanStatusRepository;
use ManagerService\Repository\LoanApproveRepository;
use Notification\Controller\HeadNotification;
use Notification\Model\NotificationEvents;
use SelfService\Form\LoanRequestForm;
use SelfService\Model\LoanRequest;
use SelfService\Model\LoanEmiDetail;
use SelfService\Repository\LoanRequestRepository;
use Setup\Model\Loan;
use Zend\Authentication\AuthenticationService;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Form\Annotation\AnnotationBuilder;
use Zend\Form\Element\Select;
use Zend\Mvc\Controller\AbstractActionController;
use Zend\View\Model\JsonModel;
use Payroll\Model\FinanceData;
use Payroll\Repository\FinanceDataRepository;

class LoanApproveController extends AbstractActionController {

    private $loanApproveRepository;
    private $employeeId;
    private $adapter;
    private $form; 

    public function __construct(AdapterInterface $adapter) {
        $this->adapter = $adapter;
        $this->loanApproveRepository = new LoanApproveRepository($adapter);
        $this->loanRequestRepository = new LoanRequestRepository($adapter);
        $auth = new AuthenticationService();
        $this->employeeId = $auth->getStorage()->read()['employee_id'];
    }

    public function initializeForm() {
        $builder = new AnnotationBuilder();
        $form = new LoanRequestForm();
        $this->form = $builder->createForm($form);
    }

    public function indexAction() {

        $loanApprove = $this->getAllList();
        // echo "<pre>";
        // print_r($loanApprove);
        // die;
        return Helper::addFlashMessagesToArray($this, ['loanApprove' => $loanApprove, 'id' => $this->employeeId]);

    } 

    public function viewAction() {
       
        $this->initializeForm();

        /**
         * LOAN REQUEST ID FOR LOAN REQUESTER
         */
        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0) {
            return $this->redirect()->toRoute("loanApprove");
        }
        /**
         * LOAN REQUESTER DETAILS
         */
        $detail          = $this->loanApproveRepository->fetchById($id);

        // ASSIGN DETAIL TO VARIABLES
        $status          = $detail['STATUS'];
        $approvedDT      = $detail['APPROVED_DATE'];
        $period          = $detail ['REPAYMENT_INSTALLMENTS'] / 12;
        $requestedEmpID  = $detail['EMPLOYEE_ID'];
        $employeeName    = $detail['FULL_NAME'];
        $authRecommender = ( $detail['RECOMMENDED_BY_NAME'] == NULL ) ? $detail['RECOMMENDER_NAME'] : $detail['RECOMMENDED_BY_NAME'];
        $authApprover    = ( $detail['APPROVED_BY_NAME'] == NULL ) ? $detail['APPROVER_NAME'] : $detail['APPROVED_BY_NAME'];
        $recommenderId   = ( $detail['RECOMMENDED_BY'] == null ) ? $detail['RECOMMENDER_ID'] : $detail['RECOMMENDED_BY'];
        
        // LOAN REQUESTER DOCUMENTS
        $detailFile     = $this->loanApproveRepository->fetchFileById($id);
        
        // LOAN REQUESTER EMI DETAILS
        $loanDetailView = $this->loanApproveRepository->fetchLoanDetailView($id);

        /**
         * ROLE FOR LOAN APPROVER
         */
        $role = $this->params()->fromRoute('role');
        
        $loanRequestModel = new LoanRequest();

        // echo "<pre>";
        // print_r($loanRequestModel);
        // die;
        
        $request = $this->getRequest();

        if ( !$request->isPost() ) {

            $loanRequestModel->exchangeArrayFromDB($detail);
            $this->form->bind($loanRequestModel);
        
        } else {
            
            $getData = $request->getPost(); 
            $action  = $getData->submit;

            $loanRequestId         = $getData['loanRequestId'];
            $loanId                = $getData['loanId'];
            $employeeId            = $getData['employeeId'];
            $appliedLoan           = $getData['appliedLoan'];
            $originAppliedLoan     = $getData['requestedLoanAmtOrigin'];
            $repaymentInstallments = $getData['repaymentInstallments'];
            $repaymentOrigin       = $getData['repaymentOrigin'];
            $monthlyInstallmentAmt = $getData['monthlyInstallmentAmt'];
            $monthlyInterestRate   = $getData['monthlyInterestRate'];
            $approverId            = $getData['approverId'];
            $approvedRemarks       = $getData['approvedRemarks'];
            $submit                = $getData['submit'];


            switch ($submit) {
                case 'Approve':

                    if (($originAppliedLoan == $appliedLoan) AND ($repaymentOrigin == $repaymentInstallments)) {

                        $loanAmount = $monthlyInstallmentAmt;


                    } else {


                        // REMOVING PREVIOUS EMI DETAIL
                        // $this->loanApproveRepository->removeEMIDetail($loanRequestId, $employeeId);
                        
                        $number                = $getData['number'];
                        $amountEMI             = $getData['amount'];
                        $repaymentEMI          = $getData['repayment'];
                        $installmentAmtEMI     = $getData['installment'];
                        $interestEMI           = $getData['interest'];
                        $principalEMI          = $getData['principal'];
                        $principalRemainingEMI = $getData['principalRemaining'];
                        

                        /**
                         * FIRST REQUIRED TO REMOVE PREVIOUS EMI
                         * */

                        // $this->loanRequestRepository->emiDelete($loanRequestId);

                        for ($i=0; $i < count($number); $i++) { 
                            
                            $emiModel = new LoanEmiDetail();
                            $emiModel->emiId = ((int) Helper::getMaxId($this->adapter, "HRIS_EMPLOYEE_EMI_DETAIL", "EMI_ID")) + 1;
                            $emiModel->loanRequestId = $loanRequestId;
                            $emiModel->employeeId = $employeeId;
                            $emiModel->repaymentInstallments = $repaymentEMI[$i];
                            $emiModel->loanAmount = $amountEMI[$i];
                            $emiModel->installment = $installmentAmtEMI[$i];
                            $emiModel->interest = $interestEMI[$i] ;
                            $emiModel->principalRepaid = $principalEMI[$i];
                            $emiModel->remainingPrincipal = $principalRemainingEMI[$i];
                            $emiModel->status = 'AP';
                            $emiModel->createdDt = date('Y-m-d');
                            $emiModel->createdBy= $this->employeeId;
                            $emiModel->modifiedDt = date('Y-m-d');
                            $emiModel->paidFlag= 'N';
                            $emiModel->sno = $i + 1;
                            // $this->loanRequestRepository->emiAdd($emiModel);

                        }


                        $loanAmount = $installmentAmtEMI[0];

                    }

                    $requestedData = $this->loanApproveRepository->fetchRowById($loanRequestId);

                    // FOR MONTHLY VALUE
                    $monthlyData = [];
                    // FOR MONTHLY VALUE

                    $loanUpdateModel = new LoanRequest();
                    $loanUpdateModel->exchangeArrayFromDB($requestedData[0]);
                    $loanUpdateModel->loanDate    = date('Y-m-d');
                    $loanUpdateModel->appliedLoan = $appliedLoan;
                    $loanUpdateModel->status      = "AP";
                    $loanUpdateModel->approvedBy  = $approverId;
                    $loanUpdateModel->approvedDate= date('Y-m-d');
                    $loanUpdateModel->approvedRemarks = $approvedRemarks;
                    $loanUpdateModel->repaymentInstallments = $repaymentInstallments;
                    $loanUpdateModel->monthlyInstallmentAmount = $monthlyInstallmentAmt;
                    $loanUpdateModel->monthlyInterestRate = $monthlyInterestRate;

                    

                    if ($loanId == 1 || $loanId == 12) {

                        $loanUpdateModel->landLoan = $loanAmount;

                        $monthlyData['MTH_ID'] = 38;
                        

                    } else if ($loanId == 3 || $loanId == 13) {
                        
                        $loanUpdateModel->socialLoan = $loanAmount;

                        $monthlyData['MTH_ID'] = 41;

                    } else if ($loanId == 4) {

                        $loanUpdateModel->motorCycleLoan = $loanAmount;
                        $monthlyData['MTH_ID'] = 40;
                        
                   
                    } else if ($loanId == 5 || $loanId == 6) {

                        $loanUpdateModel->marriageLoan = $loanAmount;
                        $monthlyData['MTH_ID'] = 46;

                    } else if ($loanId == 7) {

                        $loanUpdateModel->educationLoan = $loanAmount;
                        $monthlyData['MTH_ID'] = 42;
                        
                    } else if ($loanId == 8) { 

                        $loanUpdateModel->medicalLoan = $loanAmount;
                        // $monthlyData['MTH_ID'] = 41;
                        
                    } else if ($loanId == 2 || $loanId == 9) {
                        
                        $loanUpdateModel->hml = $loanAmount;

                        if ($loanId == 2) {

                            $monthlyData['MTH_ID'] = 39;
                            
                        } else {
                            
                            $monthlyData['MTH_ID'] = 21;

                        }
                        
                    } else if ($loanId == 10) {
                        
                        $loanUpdateModel->familyInsuranceLoan = $loanAmount;
                        $monthlyData['MTH_ID'] = 43;


                    } else if ($loanId == 11) {
                        
                        $loanUpdateModel->vehiclePurchaseLoan = $loanAmount;
                        $monthlyData['MTH_ID'] = 48;


                    }  else if ($loanId == 14) {
                        
                        $loanUpdateModel->modernTechnology = $loanAmount;
                        $monthlyData['MTH_ID'] = 47;


                    } 
                    

                    // $this->loanApproveRepository->edit($loanUpdateModel, $loanRequestId);

                    // UPDATING CIT 

                    $citFlatValue           = $requestedData[0]['CIT'];
                    $getCurrentFiscalYearId = $this->loanRequestRepository->getCurrentFiscalYears();
                    $getCurrentMonthId      = $this->loanRequestRepository->getCurrentMonth($getCurrentFiscalYearId);

                

                    $updateData = [
                        'FLAT_VALUE' => $citFlatValue,
                        'FISCAL_YEAR_ID' => $getCurrentFiscalYearId,
                        'EMPLOYEE_ID' => $employeeId,
                        'MODIFIED_DT' => date('Y-m-d')
                    ];

                    // $this->loanApproveRepository->updateFlatValue($updateData);

                    /**
                     * FOR INSERT MONTHLY VALUE
                     */

                    $monthlyData['EMPLOYEE_ID']    =  $employeeId;
                    $monthlyData['MTH_VALUE']      =  $loanAmount;
                    $monthlyData['CREATED_DT']     =  date('Y-m-d');
                    $monthlyData['MODIFIED_DT']    =  NULL;
                    $monthlyData['FISCAL_YEAR_ID'] = $getCurrentFiscalYearId;
                    $monthlyData['MONTH_ID']       =  $getCurrentMonthId + 1;

                    // $this->loanApproveRepository->insertMonthlyValue($monthlyData);


                    // HeadNotification::pushNotification(($loanUpdateModel->status == 'AP') ? NotificationEvents::LOAN_APPROVE_ACCEPTED : NotificationEvents::LOAN_APPROVE_REJECTED, $loanUpdateModel, $this->adapter, $this);

                    $this->flashmessenger()->addMessage("Loan Approved !!!");
                    break;
                
                default:

                    $loanUpdateModel = new LoanRequest();
                    $loanUpdateModel->status      = "R";
                    $loanUpdateModel->approvedBy  = $approverId;
                    $loanUpdateModel->approvedDate= date('Y-m-d');
                    $loanUpdateModel->approvedRemarks = $approvedRemarks;

                    // $this->loanApproveRepository->edit($loanUpdateModel, $loanRequestId);
                    // $this->loanApproveRepository->updateEMIStatus($loanRequestId, $loanUpdateModel->status);

                    $this->flashmessenger()->addMessage("Loan Request Rejected !!!");
                    break;
            }
            
            return $this->redirect()->toRoute("loanApprove");

        }

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'id' => $d,
            'year' => $period,
            'detail' => $detail,
            'employeeName' => $employeeName,
            'requestedDate' => $detail['REQUESTED_DATE'],
            'role' => $role,
            'recommender' => $authRecommender,
            'approver' => $authApprover,
            'status' => $status,
            'recommendedBy' => $recommenderId,
            'approvedDT' => $approvedDT,
            'employeeId' => $this->employeeId,
            'requestedEmployeeId' => $requestedEmployeeID,
            'loans' => EntityHelper::getTableKVListWithSortOption($this->adapter, Loan::TABLE_NAME, Loan::LOAN_ID, [Loan::LOAN_NAME], [Loan::STATUS => "E"], Loan::LOAN_ID, "ASC", null, false, true),
            'loanDetailView' => $loanDetailView,
            'detailFile' => $detailFile
        ]);
    }

    public function statusAction() {
        $loanFormElement = new Select();
        $loanFormElement->setName("loan");
        $loans = EntityHelper::getTableKVListWithSortOption($this->adapter, Loan::TABLE_NAME, Loan::LOAN_ID, [Loan::LOAN_NAME], [Loan::STATUS => 'E'], null, null, null, false, true);
        $loans1 = [-1 => "All"] + $loans;
        $loanFormElement->setValueOptions($loans1);
        $loanFormElement->setAttributes(["id" => "loanId", "class" => "form-control reset-field"]);
        $loanFormElement->setLabel("Loan Type");

        $loanStatus = [
            '-1' => 'All Status',
            'RQ' => 'Pending',
            'RC' => 'Recommended',
            'AP' => 'Approved',
            'R' => 'Rejected'
        ];
        $loanStatusFormElement = new Select();
        $loanStatusFormElement->setName("loanStatus");
        $loanStatusFormElement->setValueOptions($loanStatus);
        $loanStatusFormElement->setAttributes(["id" => "loanRequestStatusId", "class" => "form-control reset-field"]);
        $loanStatusFormElement->setLabel("Status");

        return Helper::addFlashMessagesToArray($this, [
                    'loans' => $loanFormElement,
                    'loanStatus' => $loanStatusFormElement,
                    'recomApproveId' => $this->employeeId,
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
        ]);
    }

    public function batchApproveRejectAction() {

        $request = $this->getRequest();
        
        try {
            
            if (!$request->ispost()) {

                throw new Exception('the request is not post');
            
            }

            $action;
            $postData      = $request->getPost()['data'];
            $postBtnAction = $request->getPost()['btnAction'];

            // return new CustomViewModel(['success' => true, 'data' => $postData]);


            if ($postBtnAction == 'btnApprove') {

                $action = 'Approve';
            
            } elseif ($postBtnAction == 'btnReject') {
                
                $action = 'Reject';
            
            } else {
                
                throw new Exception('no action defined');
            
            }
            // return new CustomViewModel(['success' => true, 'data' => $postData]);

            if ($postData == null) {

                throw new Exception('no selected rows');

            } else {

                
                try {
                    $loanRequestRepository = new LoanRequestRepository($this->adapter);

                    foreach ($postData as $data) {

                        /**
                         * GET LOAN REQUEST DETAIL
                         * */
                        $loanDetail = $this->loanRequestRepository->fetchById($data['id']);

                        $loanRequestModel = new LoanRequest();

                        // return new CustomViewModel(['success' => true, 'data' => $loanDetail ]);

                        $id   = $data['id'];
                        $role = $data['role'];

                        $mapping = LoanAdvanceHelper::getLoanMapData($loanDetail['LOAN_ID'], $loanDetail['MONTHLY_INSTALLMENT_AMOUNT']);
                        
                        $field   = $mapping['LOAN_ITEM'];
                        

                        if ($role == 2) {

                            $loanRequestModel->recommendedDate = Helper::getcurrentExpressionDate();
                            $loanRequestModel->recommendedBy   = $this->employeeId;

                            if ($action == "Reject") {

                                $loanRequestModel->status = "R";
                            
                            } else if ($action == "Approve") {

                                $loanRequestModel->status   = "AP";
                                $loanRequestModel->loanDate = date('Y-m-d');
                                $loanRequestModel->$field   = $mapping['LOAN_AMOUNT'];
                            
                            }



                            // $this->loanApproveRepository->edit($loanRequestModel, $id);
                                
                            $data['CIT']         = $loanDetail['CIT'];
                            $data['EMPLOYEE_ID'] = $loanDetail['EMPLOYEE_ID'];
                            $data['MTH_ID']      = $mapping['MTH_ID'];
                            $data['LOAN_AMOUNT'] = $mapping['LOAN_AMOUNT'];

                            if ($action == 'Approve') {

                                // $this->updateCITAndMonthlyValue($data)

                            }

                            $loanRequestModel->loanRequestId = $id;
                            try {

                                HeadNotification::pushNotification(($loanRequestModel->status == 'AP') ? NotificationEvents::LOAN_RECOMMEND_ACCEPTED : NotificationEvents::LOAN_RECOMMEND_REJECTED, $loanRequestModel, $this->adapter, $this);

                            } catch (Exception $e) {
                                
                            }

                        } else if ($role == 3 || $role == 4) {

                            $loanRequestModel->approvedDate = Helper::getcurrentExpressionDate();
                            $loanRequestModel->approvedBy   = $this->employeeId;

                            if ( $action == "Reject" ) {

                                $loanRequestModel->status = "R";
                            
                            } else if ($action == "Approve") {
                                
                                $loanRequestModel->status   = "AP";
                                $loanRequestModel->loanDate = date('Y-m-d');
                                $loanRequestModel->$field   = $mapping['LOAN_AMOUNT'];
                            
                            }
                            
                            if ($role == 4) {
                            
                                $loanRequestModel->recommendedBy   = $this->employeeId;
                                $loanRequestModel->recommendedDate = Helper::getcurrentExpressionDate();

                            }
                            
                            /**
                             * UPDATE 
                             * HRIS_EMPLOYEE_LOAN_REQUEST TABLE
                             * HRIS_EMPLOYEE_EMI_DETAIL TABLE
                             * */

                            $this->loanApproveRepository->edit($loanRequestModel, $id);
                            $this->loanApproveRepository->updateEMIStatus($id, $loanRequestModel->status);


                            /**
                             * FOR CIT AND MONTHLY VALUE IF LOAN APPROVED
                             * */
                            
                            $data['CIT']         = $loanDetail['CIT'];
                            $data['EMPLOYEE_ID'] = $loanDetail['EMPLOYEE_ID'];
                            $data['MTH_ID']      = $mapping['MTH_ID'];
                            $data['LOAN_AMOUNT'] = $mapping['LOAN_AMOUNT'];

                            if ($action == 'Approve') {

                                $this->updateCITAndMonthlyValue($data);

                            }

                            // return new CustomViewModel(['success' => true, 'data' =>  $this->updateCITAndMonthlyValue($data)]);
                            $loanRequestModel->loanRequestId = $id;

                            try {
                                
                                HeadNotification::pushNotification(($loanRequestModel->status == 'AP') ? NotificationEvents::LOAN_APPROVE_ACCEPTED : NotificationEvents::LOAN_APPROVE_REJECTED, $loanRequestModel, $this->adapter, $this);
                            
                            } catch (Exception $e) {
                                
                            
                            }
                            return new CustomViewModel(['success' => true, 'data' => 'success']);
                        }
                    }
                } catch (Exception $ex) {
                   
                }
            }
            $listData = $this->getAllList();
            return new CustomViewModel(['success' => true, 'data' => $listData]);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'error' => $e->getMessage()]);
        }
    }

    public function getAllList() {
        $list = $this->loanApproveRepository->getAllRequest($this->employeeId);
        return Helper::extractDbData($list);
    }

    public function pullLoanRequestStatusListAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();

            $loanStatusRepository = new LoanStatusRepository($this->adapter);
            $result = $loanStatusRepository->getFilteredRecord($data, $data['recomApproveId']);

            $recordList = Helper::extractDbData($result);

            return new JsonModel([
                "success" => "true",
                "data" => $recordList,
            ]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function financialImpact($loanModel){
        $model = new FinanceData();
        $financeDataRepo = new FinanceDataRepository($this->adapter);
        $model->financeDataId = (int) Helper::getMaxId($this->adapter, FinanceData::TABLE_NAME, FinanceData::FINANCE_DATA_ID) + 1;
        $model->moduleCode = 'LN';
        $model->masterId = $loanModel->loanId;
        $model->requestId = $loanModel->loanRequestId;
        $model->amount = $loanModel->requestedAmount;
        $financeDataRepo->add($model);
        // $financeDataRepo->financialDataGlEntry($model->financeDataId);
    }


    public function updateCITAndMonthlyValue($data)
    {
        // UPDATING CIT 

        $citFlatValue           = $data['CIT'];
        $employeeId             = $data['EMPLOYEE_ID'];
        $getCurrentFiscalYearId = $this->loanRequestRepository->getCurrentFiscalYears();
        $getCurrentMonthId      = $this->loanRequestRepository->getCurrentMonth($getCurrentFiscalYearId);


        $updateData = [
            'FLAT_VALUE'     => $citFlatValue,
            'FISCAL_YEAR_ID' => $getCurrentFiscalYearId,
            'EMPLOYEE_ID'    => $employeeId,
            'MODIFIED_DT'    => date('Y-m-d')
        ];

        $this->loanApproveRepository->updateFlatValue($updateData);

        /**
         * FOR INSERT MONTHLY VALUE
         */
        $monthlyData['MTH_ID']         =  $data['MTH_ID'];
        $monthlyData['EMPLOYEE_ID']    =  $employeeId;
        $monthlyData['MTH_VALUE']      =  $data['LOAN_AMOUNT'];
        $monthlyData['CREATED_DT']     =  date('Y-m-d');
        $monthlyData['MODIFIED_DT']    =  NULL;
        $monthlyData['FISCAL_YEAR_ID'] =  $getCurrentFiscalYearId;
        $monthlyData['MONTH_ID']       =  $getCurrentMonthId + 1;

        $this->loanApproveRepository->insertMonthlyValue($monthlyData);

        return true;
       
    }


}
