<?php

namespace Overtime\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use Notification\Controller\HeadNotification;
use Notification\Model\NotificationEvents;
use SelfService\Form\OvertimeRequestForm;
use SelfService\Model\Overtime;
use SelfService\Model\OvertimeDetail;
use SelfService\Repository\OvertimeDetailRepository;
use SelfService\Repository\OvertimeRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class OvertimeRequestHRController extends HrisController {

    private $detailRepository;

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeForm(OvertimeRequestForm::class);
        $this->repository = new OvertimeRepository($adapter);
        $this->detailRepository = new OvertimeDetailRepository($adapter);
    }

    public function indexAction(){

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["EMPLOYEE_CODE","FULL_NAME"], ["STATUS" => 'E', 'RETIRED_FLAG' => 'N'], "FIRST_NAME", "ASC", NULL, FALSE, TRUE)
        ]);

    }

    public function automationAction(){

        $mapping = [

             'sun' => 'WEEKDAY1',
             'mon' => 'WEEKDAY2',
             'tue' => 'WEEKDAY3',
             'wed' => 'WEEKDAY4',
             'thu' => 'WEEKDAY5',
             'fri' => 'WEEKDAY6',
             'sat' => 'WEEKDAY7'

        ];

        $request = $this->getRequest();

        


        if ($request->isPost()) {

            
            $data = $request->getPost();

            $employeeId  = $data['employeeId'];
            $fromDate    = date('Y-m-d', strtotime($data['fromOvertimeDate']));
            $toDate      = date('Y-m-d', strtotime($data['toOvertimeDate']));
            $fromNepDate = $data['fromNepaliDate'];
            $toNepDate   = $data['toNepaliDate'];
            $desc        = $data['description'];

            if ($toDate >= $fromDate) {

                $attendances = $this->repository->getAttendance($fromDate, $toDate, $employeeId);
                

                // REQUEST DOES NOT WORK FOR NIGHT SHIFT EMPLOYEE AT THE MOMENT (FOR FUTURE WORK)
                

                    if ($attendances) {

                        
                    
                        for ($i=0; $i < count($attendances) ; $i++) { 

                            $shift       = $this->repository->getShiftIdByDate($employeeId, $attendances[$i]['ATTENDANCE_DT']);




                            $dayoff = [
                                'WEEKDAY1' => $shift['WEEKDAY1'],
                                'WEEKDAY2' => $shift['WEEKDAY2'],
                                'WEEKDAY3' => $shift['WEEKDAY4'],
                                'WEEKDAY4' => $shift['WEEKDAY4'],
                                'WEEKDAY5' => $shift['WEEKDAY5'],
                                'WEEKDAY6' => $shift['WEEKDAY6'],
                                'WEEKDAY7' => $shift['WEEKDAY7']

                            ];

                        if ($shift['SHIFT_ENAME'] !== 'night_shift') {

                            $inTime          = ($attendances[$i]['IN_TIME']) ? substr($attendances[$i]['IN_TIME'],0,19) : null;
                            $outTime         = ($attendances[$i]['OUT_TIME']) ? substr($attendances[$i]['OUT_TIME'],0,19) : null;
                            $exludeINTime    = ($attendances[$i]['IN_TIME']) ? substr($attendances[$i]['IN_TIME'], 0, 10) : null;
                            $exludeOutTime   = ($attendances[$i]['OUT_TIME']) ? substr($attendances[$i]['OUT_TIME'], 0, 10) : null;
                            $isHoliday       = ($attendances[$i]['HOLIDAY_ID']) ? 1 : 0; 


                            // FIND WEEK DAY AND WEEKEND DAY
                            $weekly    = ($inTime) ? date("D", strtotime($inTime)) : ''; 

                            if ($weekly !== '') {

                                $mappingValue = $mapping[strtolower($weekly)];
                                $dayoffValue  = $dayoff[$mappingValue];



                                /**
                                 * ARRANGING WITH SHIFT
                                 * */

                                // FOR NORMAL DAY SHIFT
                                if ($dayoffValue == 'H') {

                                    $earlyOTTime = $exludeINTime.' '.substr($shift['START_TIME'],11,8);
                                    $afterOTTime = $exludeOutTime.' '.substr($shift['HALF_TIME'],11,8);

                                } else {

                                    $earlyOTTime = $exludeINTime.' '.substr($shift['START_TIME'],11,8);
                                    $afterOTTime = $exludeOutTime.' '.substr($shift['END_TIME'],11,8);

                                }


                                switch ($dayoffValue) {
                                    case 'DAY_OFF':
                                            
                                            if ($inTime) {

                                                $earlyDayOff = new \DateTime($inTime);

                                                $availableOT = $earlyDayOff->diff(new \DateTime($outTime));

                                                $total_hour = ($availableOT->h * 60) + $availableOT->i;

                                            }

                                        break;

                                    case 'H':
                                            
                                            if ($inTime) {

                                                $earlyDay = new \DateTime($inTime);
                                                $availableEarlyOT = $earlyDay->diff(new \DateTime($earlyOTTime));
                                                $earlyOTHour = ($availableEarlyOT->h * 60) + $availableEarlyOT->i;


                                                $afterDay = new \DateTime($afterOTTime);
                                                $availableAfterOT = $afterDay->diff(new \DateTime($outTime));
                                                $afterOTHour = ($availableAfterOT->h * 60) + $availableAfterOT->i;

                                                $total_hour = (($availableEarlyOT->h * 60) + $availableEarlyOT->i) +
                                                            (($availableAfterOT->h * 60) + $availableAfterOT->i);

                                            }

                                        break;
                                    
                                    default:

                                        if ($inTime) {
                
                                            if ($isHoliday > 0) {

                                                $earlyDayOff = new \DateTime($inTime);
                                                $availableOT = $earlyDayOff->diff(new \DateTime($outTime));

                                                $total_hour = ($availableOT->h * 60) + $availableOT->i;

                                            } else {

                                                // $earlyOTTimeValue = new \DateTime($earlyOTTime);
                                                $earlyDay = new \DateTime($inTime);
                                                $availableEarlyOT = $earlyDay->diff(new \DateTime($earlyOTTime));
                                                $earlyOTHour = ($availableEarlyOT->h * 60) + $availableEarlyOT->i;

                                                $afterDay = new \DateTime($afterOTTime);
                                                $availableAfterOT = $afterDay->diff(new \DateTime($outTime));
                                                $afterOTHour = ($availableAfterOT->h * 60) + $availableAfterOT->i;

                                                $total_hour = (($availableEarlyOT->h * 60) + $availableEarlyOT->i) +
                                                            (($availableAfterOT->h * 60) + $availableAfterOT->i);


                                            }

                                        }

                                        break;
                                }


                                $overtime = new Overtime();

                                $overtime->overtimeId = ((int) Helper::getMaxId($this->adapter, Overtime::TABLE_NAME, Overtime::OVERTIME_ID)) + 1;
                                $overtime->employeeId = $employeeId;
                                $overtime->overtimeDate = $attendances[$i]['ATTENDANCE_DT'];
                                $overtime->requestedDate = date('Y-m-d');
                                $overtime->description = $desc;
                                $overtime->status = 'RQ';
                                $overtime->allTotalHour = $total_hour;

                                /* CHECK BEFORE INSERT */

                                $checkOTRequest = $this->repository->checkOTRequest($attendances[$i]['ATTENDANCE_DT'], $employeeId);


                                if (!$checkOTRequest) {

                                    $this->repository->add($overtime);


                                    if ($dayoffValue == 'DAY_OFF' || $isHoliday > 0) {

                                        $totalHour = [
                                            $total_hour
                                        ];
                                        $startTime = [$inTime];
                                        $endTime   = [$outTime];
                                        $count = 1;

                                    } else {

                                        $totalHour = [
                                            $earlyOTHour,
                                            $afterOTHour
                                        ];
                                        $startTime = [$inTime, $afterOTTime];
                                        $endTime   = [$earlyOTTime, $outTime];
                                        $count = 2;

                                    }

                                    $overtimeDetail = new OvertimeDetail();

                                    $overtimeRequest = $this->repository->checkOTRequest($attendances[$i]['ATTENDANCE_DT'], $employeeId);
                                    $overtimeRequestDetail = $this->repository->checkOTDetail($overtimeRequest['OVERTIME_ID']);

                                    if (!$overtimeRequestDetail) {


                                        for ($j=0; $j < $count; $j++) { 

                                            $overtimeDetail->detailId = ((int) Helper::getMaxId($this->adapter, OvertimeDetail::TABLE_NAME, OvertimeDetail::DETAIL_ID));
                                            $overtimeDetail->overtimeId = $overtimeRequest['OVERTIME_ID'];
                                            $overtimeDetail->startTime  = $startTime[$j];
                                            $overtimeDetail->endTime    = $endTime[$j];
                                            $overtimeDetail->status     = 'E';
                                            $overtimeDetail->createdBy  = $employeeId;
                                            $overtimeDetail->createdDate= date('Y-m-d');
                                            $overtimeDetail->totalHour  = $totalHour[$j];

                                            $this->detailRepository->add($overtimeDetail);

                                        }

                                    }

                                }

                            }

                        }
                    
                    }

                }

            }

            
            $this->flashmessenger()->addMessage("Overtime Successfully Requested");
            return $this->redirect()->toRoute("overtimeRequestHr", ['action'=>'automation']);

        }


        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["EMPLOYEE_CODE","FULL_NAME"], ["STATUS" => 'E', 'RETIRED_FLAG' => 'N'], "FIRST_NAME", "ASC", NULL, FALSE, TRUE)
        ]);

    }

    public function automationTestAction(){

        $mapping = [

             'sun' => 'WEEKDAY1',
             'mon' => 'WEEKDAY2',
             'tue' => 'WEEKDAY3',
             'wed' => 'WEEKDAY4',
             'thu' => 'WEEKDAY5',
             'fri' => 'WEEKDAY6',
             'sat' => 'WEEKDAY7'

        ];
        $request = $this->getRequest();

        // echo 'here';
        // die;


        if (true) {

            // Zend\Stdlib\Parameters Object
            // (
            //     [storage:ArrayObject:private] => Array
            //         (
            //             [employeeId] => 456
            //             [fromOvertimeDate] => 11-Jun-2023
            //             [fromNepaliDate] => 2080-02-28
            //             [toOvertimeDate] => 13-Jun-2023
            //             [toNepaliDate] => 2080-02-30
            //             [description] => erp
            //         )

            // )

            // $data = $request->getPost();
            $data = [
                'employeeId' => 456,
                'fromOvertimeDate' => '11-Jun-2023',
                'fromNepaliDate' => '2080-02-28',
                'toOvertimeDate' => '13-Jun-2023',
                'toNepaliDate' => '2080-02-30',
                'description' => 'erp'
            ];

            $employeeId  = $data['employeeId'];
            $fromDate    = date('Y-m-d', strtotime($data['fromOvertimeDate']));
            $toDate      = date('Y-m-d', strtotime($data['toOvertimeDate']));
            $fromNepDate = $data['fromNepaliDate'];
            $toNepDate   = $data['toNepaliDate'];
            $desc        = $data['description'];

            if ($toDate >= $fromDate) {

                $attendances = $this->repository->getAttendance($fromDate, $toDate, $employeeId);
                $total_count = count($attendances);

                $shift       = $this->repository->getShift($employeeId);

                echo "<pre>";
                echo "attendances :";
                echo "<pre>"; 
                print_r($attendances);
                echo "<pre>";
                echo "<pre>"; 
                echo count($attendances);
                
                echo "<hr>";

               


                $dayoff = [
                    'WEEKDAY1' => $shift['WEEKDAY1'],
                    'WEEKDAY2' => $shift['WEEKDAY2'],
                    'WEEKDAY3' => $shift['WEEKDAY4'],
                    'WEEKDAY4' => $shift['WEEKDAY4'],
                    'WEEKDAY5' => $shift['WEEKDAY5'],
                    'WEEKDAY6' => $shift['WEEKDAY6'],
                    'WEEKDAY7' => $shift['WEEKDAY7']

                ];

                // REQUEST DOES NOT WORK FOR NIGHT SHIFT EMPLOYEE AT THE MOMENT (FOR FUTURE WORK)
                if ($shift['SHIFT_ENAME'] !== 'night_shift') {

                    

                    if ($attendances) {

                        // for ($i=0; $i < $total_count; $i++) { 
                        //     echo $i;
                        //     echo "<br>";
                        // }

                        // die;

                        for ($i=0; $i < $total_count ; $i++) { 
                            

                            $inTime          = substr($attendances[$i]['IN_TIME'],0,19);
                            $outTime         = substr($attendances[$i]['OUT_TIME'],0,19);
                            $exludeINTime    = substr($attendances[$i]['IN_TIME'], 0, 10);
                            $exludeOutTime   = substr($attendances[$i]['OUT_TIME'], 0, 10);
                            $isHoliday       = ($attendances[$i]['HOLIDAY_ID']) ? 1 : 0; 


                            echo "<pre>";
                            echo "intime :";
                            print_r($inTime);
                            echo "<pre>";
                            echo "outime :";
                            print_r($outTime);
                            echo "<pre>";
                            echo "exludeINTime :";
                            print_r($exludeINTime);
                            echo "<pre>";
                            echo "exludeOutTime :";
                            print_r($exludeOutTime);
                            echo "<pre>";
                            echo "isHoliday :";
                            print_r($isHoliday);
                            echo "<hr>";


                            // FIND WEEK DAY AND WEEKEND DAY
                            $weekly    = date("D", strtotime($inTime)); 


                            $mappingValue = $mapping[strtolower($weekly)];
                            $dayoffValue  = $dayoff[$mappingValue];

                            echo "<pre>";
                            echo "weekly :";
                            print_r($weekly);
                            echo "<pre>";
                            echo "mappingValue :";
                            print_r($mappingValue);
                            echo "<pre>";
                            echo "mappingValue :";
                            print_r($dayoffValue);
                            echo "<hr>";





                            /**
                             * ARRANGING WITH SHIFT
                             * */

                            // FOR NORMAL DAY SHIFT
                            if ($dayoffValue == 'H') {

                                $earlyOTTime = $exludeINTime.' '.substr($shift['START_TIME'],11,8);
                                $afterOTTime = $exludeOutTime.' '.substr($shift['HALF_TIME'],11,8);

                            } else {

                                $earlyOTTime = $exludeINTime.' '.substr($shift['START_TIME'],11,8);
                                $afterOTTime = $exludeOutTime.' '.substr($shift['END_TIME'],11,8);

                            }

                            echo "<pre>";
                            echo "earlyOTTime :";
                            print_r($earlyOTTime);
                            echo "<pre>";
                            echo "afterOTTime :";
                            print_r($afterOTTime);
                            echo "<hr>";


                            switch ($dayoffValue) {
                                case 'DAY_OFF':
                                        
                                        if ($inTime) {

                                            $earlyDayOff = new \DateTime($inTime);

                                            $availableOT = $earlyDayOff->diff(new \DateTime($outTime));

                                            $total_hour = ($availableOT->h * 60) + $availableOT->i;

                                        }

                                        echo "<pre>";
                                        echo "DAY_OFF--- :";
                                        echo "<pre>";
                                        echo "earlyDayOff :";
                                        print_r($earlyDayOff);
                                        echo "<pre>";
                                        echo "availableOT :";
                                        print_r($availableOT);
                                        echo "<pre>";
                                        echo "total_hour :";
                                        print_r($total_hour);
                                        echo "<hr>";

                                    break;

                                case 'H':
                                        
                                        if ($inTime) {

                                            $earlyDay = new \DateTime($inTime);
                                            $availableEarlyOT = $earlyDay->diff(new \DateTime($earlyOTTime));
                                            $earlyOTHour = ($availableEarlyOT->h * 60) + $availableEarlyOT->i;


                                            $afterDay = new \DateTime($afterOTTime);
                                            $availableAfterOT = $afterDay->diff(new \DateTime($outTime));
                                            $afterOTHour = ($availableAfterOT->h * 60) + $availableAfterOT->i;

                                            $total_hour = (($availableEarlyOT->h * 60) + $availableEarlyOT->i) +
                                                        (($availableAfterOT->h * 60) + $availableAfterOT->i);

                                        }

                                        echo "<pre>";
                                        echo "Half Day--- :";
                                        echo "<pre>";
                                        echo "earlyOTHour :";
                                        print_r($earlyOTHour);
                                        echo "<pre>";
                                        echo "afterOTHour :";
                                        print_r($afterOTHour);
                                        echo "<pre>";
                                        echo "total_hour :";
                                        print_r($total_hour);
                                        echo "<hr>";

                                    break;
                                
                                default:

                                    if ($inTime) {
            
                                        if ($isHoliday > 0) {

                                            $earlyDayOff = new \DateTime($inTime);
                                            $availableOT = $earlyDayOff->diff(new \DateTime($outTime));

                                            $total_hour = ($availableOT->h * 60) + $availableOT->i;

                                            echo "<pre>";
                                            echo "Holiday--- :";
                                            echo "<pre>";
                                            echo "earlyDayOff :";
                                            print_r($earlyDayOff);
                                            echo "<pre>";
                                            echo "availableOT :";
                                            print_r($availableOT);
                                            echo "<pre>";
                                            echo "total_hour :";
                                            print_r($total_hour);
                                            echo "<hr>";

                                        } else {

                                            // $earlyOTTimeValue = new \DateTime($earlyOTTime);
                                            $earlyDay = new \DateTime($inTime);
                                            $availableEarlyOT = $earlyDay->diff(new \DateTime($earlyOTTime));
                                            $earlyOTHour = ($availableEarlyOT->h * 60) + $availableEarlyOT->i;

                                            $afterDay = new \DateTime($afterOTTime);
                                            $availableAfterOT = $afterDay->diff(new \DateTime($outTime));
                                            $afterOTHour = ($availableAfterOT->h * 60) + $availableAfterOT->i;

                                            $total_hour = (($availableEarlyOT->h * 60) + $availableEarlyOT->i) +
                                                        (($availableAfterOT->h * 60) + $availableAfterOT->i);
                                            

                                            echo "<pre>";
                                            echo "Normal--- :";
                                            echo "<pre>";
                                            echo "earlyOTHour :";
                                            print_r($earlyOTHour);
                                            echo "<pre>";
                                            echo "afterOTHour :";
                                            print_r($afterOTHour);
                                            echo "<pre>";
                                            echo "total_hour :";
                                            print_r($total_hour);
                                            echo "<hr>";

                                        }

                                    }

                                    break;
                            }


                            $overtime = new Overtime();

                            $overtime->overtimeId = ((int) Helper::getMaxId($this->adapter, Overtime::TABLE_NAME, Overtime::OVERTIME_ID)) + 1;
                            $overtime->employeeId = $employeeId;
                            $overtime->overtimeDate = $attendances[$i]['ATTENDANCE_DT'];
                            $overtime->requestedDate = date('Y-m-d');
                            $overtime->description = $desc;
                            $overtime->status   = 'RQ';
                            $overtime->allTotalHour = $total_hour;

                            echo "<pre>";
                            echo "overtime Model --- :";
                            echo "<pre>";
                            print_r($overtime);
                            
                            echo "<hr>";



                            /* CHECK BEFORE INSERT */

                            $checkOTRequest = $this->repository->checkOTRequestTest($attendances[$i]['ATTENDANCE_DT'], $employeeId);

                            echo "<pre>";
                            echo "request OT request --- :";
                            echo "<pre>";
                            print_r($checkOTRequest);
                            
                            echo "<hr>";


                            if (!$checkOTRequest) {

                                $this->repository->addOTTest($overtime);

                                echo "<pre>";
                                echo "overtime if not empty after inserting --- :";
                                echo "<pre>";
                                echo "<hr>";

                                
                                if ($dayoffValue == 'DAY_OFF' || $isHoliday > 0) {

                                    $totalHour = [
                                        $total_hour
                                    ];
                                    $startTime = [$inTime];
                                    $endTime   = [$outTime];
                                    $count = 1;

                                    echo "<pre>";
                                    echo "DAY_OFF or Holiday --- :";
                                    echo "<pre>";
                                    echo "total hour --- :";
                                    print_r($totalHour);
                                    echo "<pre>";
                                    echo "startTime --- :";
                                    print_r($startTime);
                                    echo "<pre>";
                                    echo "endTime --- :";
                                    print_r($endTime);
                                    echo "<pre>";
                                    echo "count --- :";
                                    print_r($count);
                                    echo "<hr>";

                                    

                                } else {

                                    $totalHour = [
                                        $earlyOTHour,
                                        $afterOTHour
                                    ];
                                    $startTime = [$inTime, $afterOTTime];
                                    $endTime   = [$earlyOTTime, $outTime];
                                    $count = 2;

                                    echo "<pre>";
                                    echo "Normal --- :";
                                    echo "<pre>";
                                    echo "total hour --- :";
                                    print_r($totalHour);
                                    echo "<pre>";
                                    echo "startTime --- :";
                                    print_r($startTime);
                                    echo "<pre>";
                                    echo "endTime --- :";
                                    print_r($endTime);
                                    echo "<pre>";
                                    echo "count --- :";
                                    print_r($count);
                                    echo "<hr>";

                                }

                                $overtimeDetail = new OvertimeDetail();
                                
                                
                                $overtimeRequest = $this->repository->checkOTRequest($attendances[$i]['ATTENDANCE_DT'], $employeeId);
                                
                                if (!$overtimeRequest) {

                                    echo "<pre>";
                                    echo "check overtimeRequest --- :";
                                    echo "<pre>";
                                    print_r($overtimeRequest);
                                    echo "<pre>";
                                    echo "<hr>";

                                    for ($j=0; $j < $count; $j++) { 

                                        $overtimeDetail->detailId = ((int) Helper::getMaxId($this->adapter, OvertimeDetail::TABLE_NAME, OvertimeDetail::DETAIL_ID));
                                        $overtimeDetail->overtimeId = $overtimeRequest['OVERTIME_ID'];
                                        $overtimeDetail->startTime  = $startTime[$i];
                                        $overtimeDetail->endTime    = $endTime[$i];
                                        $overtimeDetail->status     = 'E';
                                        $overtimeDetail->createdBy  = $employeeId;
                                        $overtimeDetail->createdDate= date('Y-m-d');
                                        $overtimeDetail->totalHour  = $totalHour[$i];

                                        echo "<pre>";
                                        echo "overtimeDetail model --- :";
                                        echo "<pre>";
                                        print_r($overtimeDetail);
                                        
                                        echo "<hr>";

                                        // $this->detailRepository->add($overtimeDetail);

                                    }
                                }

                                echo "here";
                                // die;
                                
                                

                            }

                        }

                    }
                }

                
            }
            echo "================================================================";
            die;

            
            $this->flashmessenger()->addMessage("Overtime Successfully Requested");
            return $this->redirect()->toRoute("overtimeRequestHr", ['action'=>'automation']);

        }


        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employees' => EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["EMPLOYEE_CODE","FULL_NAME"], ["STATUS" => 'E', 'RETIRED_FLAG' => 'N'], "FIRST_NAME", "ASC", NULL, FALSE, TRUE)
        ]);

    }

}