<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Payroll\Repository\BasicSalaryRepository;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Mvc\Console\View\ViewModel;
use Application\Helper\Helper;
use Payroll\Form\BasicSalaryForm;
use Payroll\Model\BasicSalaryModel;
use Payroll\Repository\PayrollRepository;
use Payroll\Repository\SalarySheetRepo;
use Recruitment\Repository\UserApplicationRepository;
use Setup\Model\FunctionalLevels;
use Setup\Model\Position;
use Zend\View\Model\JsonModel;

class BasicSalaryController extends HrisController {

    public $adapter;

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(BasicSalaryRepository::class);
        $this->initializeForm(BasicSalaryForm::class);
    }

    public function indexAction() {

        $request = $this->getRequest();

        if ($request->isPost()) {

            $searchData = $request->getPost();

            // Get data from database
            $list = iterator_to_array($this->repository->all($searchData), false);

            return new JsonModel(['success' => true, 'data' => $list, 'error' => '']);

        }

        $functionalList = EntityHelper::getTableKVListWithSortOption($this->adapter, FunctionalLevels::TABLE_NAME, FunctionalLevels::FUNCTIONAL_LEVEL_ID, [FunctionalLevels::FUNCTIONAL_LEVEL_EDESC], ["STATUS" => "E"], null, null, null, [null => '---'], true);
        $positionList = EntityHelper::getTableKVListWithSortOption($this->adapter, Position::TABLE_NAME, Position::POSITION_ID, [Position::POSITION_NAME], ["STATUS" => "E"], Position::POSITION_NAME, "ASC", null, [null => '---'], true);

        return $this->stickFlashMessagesTo([
            'functionalList' => $functionalList,
            'positionList' => $positionList,
        ]);

    }

    public function addAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $this->form->setData($request->getPost());

            if ($this->form->isValid()) {

                $basicSalary = new BasicSalaryModel();
                $basicSalary->exchangeArrayFromForm($this->form->getData());
                $basicSalary->egcId = ((int) Helper::getMaxId($this->adapter, BasicSalaryModel::TABLE_NAME, BasicSalaryModel::EGC_ID)) + 1;
                $basicSalary->createdDate = Helper::getcurrentExpressionDate();
                $basicSalary->fromDate = date('Y-m-d', strtotime($basicSalary->fromDate)); 
                $basicSalary->toDate = $basicSalary->toDate == null ? null : date('Y-m-d', strtotime($basicSalary->toDate));
                $basicSalary->status = 'E';
                $basicSalary->isNew = 'Y';

                // Update to_date of current grade policy
                $this->repository->updateToDate($basicSalary->fromDate, 
                                                $basicSalary->functionalLevelId, 
                                                $basicSalary->positionId, 
                                                $basicSalary->salaryType);

                $this->repository->add($basicSalary);

                // // Recalculate grade if it is salaray increment
                if(filter_var($request->getPost()['modifyGrade'], FILTER_VALIDATE_BOOLEAN)){

                    // Update each and every grade
                    $this->repository->updateGrade($basicSalary->functionalLevelId, 
                                                    $basicSalary->positionId, 
                                                    $basicSalary->fromDate, 
                                                    $basicSalary->salaryType, 
                                                    $basicSalary->oneDaySalary,
                                                    $basicSalary->egcId);
                }

                return new JsonModel(["success"=>true]);
            }

        }
        
        $functionalList = EntityHelper::getTableKVListWithSortOption($this->adapter, FunctionalLevels::TABLE_NAME, FunctionalLevels::FUNCTIONAL_LEVEL_ID, [FunctionalLevels::FUNCTIONAL_LEVEL_EDESC], ["STATUS" => "E"], null, null, null, [null => '---'], true);
        $positionList = EntityHelper::getTableKVListWithSortOption($this->adapter, Position::TABLE_NAME, Position::POSITION_ID, [Position::POSITION_NAME], ["STATUS" => "E"], Position::POSITION_NAME, "ASC", null, [null => '---'], true);

        return $this->stickFlashMessagesTo([
            'form' => $this->form,
            'functionalList' => $functionalList,
            'positionList' => $positionList,
        ]);

    }

    public function editAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("basicSalary", array("action"=>"index"));
        }

        if($request->isPost()){
            $id = $request->getPost()['egcId'];
        }

        $data = iterator_to_array($this->repository->fetchById($id));

        $model = new BasicSalaryModel();
        $model->exchangeArrayFromDB($data);

        if ($request->isPost())
        {

            $basicSalary = new BasicSalaryModel();
            $postData = $request->getPost();

            $this->form->setData($postData);

            if ($this->form->isValid())
            {

                $basicSalary->exchangeArrayFromForm($this->form->getData());
                $basicSalary->modifiedDate = Helper::getcurrentExpressionDate();
                $basicSalary->fromDate = date('Y-m-d', strtotime($basicSalary->fromDate));
                $basicSalary->toDate = $basicSalary->toDate == null ? null : date('Y-m-d', strtotime($basicSalary->toDate));
                $basicSalary->status = 'E';

                // If there is change in one day salary or functional level or position
                if($basicSalary->oneDaySalary != $model->oneDaySalary || 
                    $basicSalary->functionalLevelId != $model->functionalLevelId
                    || $basicSalary->positionId != $model->positionId){


                    // Revert grade to previous
                    $this->repository->revertGrade($id);

                    // Delete all related grade history
                    $this->repository->deleteGradeChangeHistory($id);

                    // Update to_date of current basic salary
                    $this->repository->updateToDate($basicSalary->fromDate, 
                                                    $basicSalary->functionalLevelId, 
                                                    $basicSalary->positionId, 
                                                    $basicSalary->salaryType);

                    // Update new grade
                    $this->repository->updateGrade($basicSalary->functionalLevelId, 
                                                    $basicSalary->positionId, 
                                                    $basicSalary->fromDate, 
                                                    $basicSalary->salaryType, 
                                                    $basicSalary->oneDaySalary,
                                                    $basicSalary->egcId);
                }

                $this->repository->edit($basicSalary, $basicSalary->egcId);

                // Set to_date of previous data as null
                $this->repository->updateToDate($model->fromDate, 
                                                $model->functionalLevelId, 
                                                $model->positionId, 
                                                $model->salaryType,
                                                true);

                return new JsonModel(["success"=>true]);
            }
        }

        $model->fromDate = date('d-M-Y', strtotime($model->fromDate)); 
        $model->toDate = $basicSalary->toDate == null ? null : date('d-M-Y', strtotime($model->toDate));
        $this->form->bind($model);

        $functionalList = EntityHelper::getTableKVListWithSortOption($this->adapter, FunctionalLevels::TABLE_NAME, FunctionalLevels::FUNCTIONAL_LEVEL_ID, [FunctionalLevels::FUNCTIONAL_LEVEL_EDESC], ["STATUS" => "E"], null, null, null, [null => '---'], true);
        $positionList = EntityHelper::getTableKVListWithSortOption($this->adapter, Position::TABLE_NAME, Position::POSITION_ID, [Position::POSITION_NAME], ["STATUS" => "E"], Position::POSITION_NAME, "ASC", null, [null => '---'], true);

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'id' => $id,
            'functionalList' => $functionalList,
            'positionList' => $positionList,
        ]);

    }

    /**
     * Function to check if the given basic salary master data already exists
     */
    public function validateOverlapAction(){

        $request = $this->getRequest();

        $postData = $request->getPost();

        $functionalId = $postData['functionalId'];
        $positionId = $postData['positionId'];
        $salaryType = $postData['salaryType'];
        $egcId = $postData['egcId'];
        $fromDate = date('Y-m-d', strtotime($postData['fromDate']));

        $result = iterator_to_array(
                    $this->repository->validateOverlap($functionalId, $positionId, $fromDate, $salaryType, $egcId), 
                false);

        $valid = count($result) == 0 ? true : false;

        return new JsonModel(["data"=>$valid]);
    }

    public function excelUploadAction(){

        $request = $this->getRequest();

        $postData = $request->getPost();

        $data = $postData['data'];
        
        $fromDate = date('Y-m-d', strtotime($postData['fromDate'])); 
        $toDate = $postData['toDate'] == null ? null : date('Y-m-d', strtotime($postData['toDate']));

        $calculateGrade = filter_var($request->getPost()['modifyGrade'], FILTER_VALIDATE_BOOLEAN);
        $createdDate = Helper::getcurrentExpressionDate();

        for($i=0; $i < count($data); $i++){

            $functional = $data[$i]["A"];
            $position = $data[$i]["B"];
            $salaryType = $data[$i]["E"];
            $basicSalary = $data[$i]["F"];
            $oneDaySalary = $data[$i]["G"];
            $gradeCeiling = $data[$i]["H"];

            // If the given data already exists skip the process
            $result = iterator_to_array(
                $this->repository->validateOverlap($functional, $position, $fromDate, $salaryType, null), 
            false);

            $valid = count($result) == 0 ? true : false;

            if($valid){

                $model = new BasicSalaryModel();
                $model->egcId = ((int) Helper::getMaxId($this->adapter, BasicSalaryModel::TABLE_NAME, BasicSalaryModel::EGC_ID)) + 1;
                $model->functionalLevelId = $functional;
                $model->positionId = $position;
                $model->basicSalary = $basicSalary;
                $model->oneDaySalary = $oneDaySalary;
                $model->gradeCeilingNo = $gradeCeiling;
                $model->fromDate = $fromDate;
                $model->toDate = $toDate;
                $model->createdDate = $createdDate;
                $model->salaryType = $salaryType;
                $model->status = 'E';
                $model->isNew = 'Y';

                // Update to_date of current basic salary
                $this->repository->updateToDate($model->fromDate, 
                                                $model->functionalLevelId, 
                                                $model->positionId, 
                                                $model->salaryType);

                // // Recalculate grade if it is salaray increment
                if($calculateGrade && $model->salaryType == 'MAIN'){

                    // Update each and every grade
                    $this->repository->updateGrade($model->functionalLevelId, 
                                                    $model->positionId, 
                                                    $model->fromDate, 
                                                    $model->salaryType, 
                                                    $model->oneDaySalary,
                                                    $model->egcId);
                }

                $this->repository->add($model);
            }
        }

        return new JsonModel(["success"=>true]);

    }

    public function deleteAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("basicSalary", array("action"=>"index"));
        }

        // Update the status
        $this->repository->delete($id);

        // If it is latest and has effected grade, revert the grade
        $this->repository->deleteGradeHistory($id);

        $this->flashmessenger()->addMessage("Salary deleted successfully");
        return $this->redirect()->toRoute("basicSalary", array("action"=>"index"));

    }

    public function testAction(){
        var_dump("tesy");
        var_dump(Helper::sparrowMessage($this->adapter, '9840601527', 'hello'));

        die;

    }
}