<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Model\FiscalYear;
use Payroll\Repository\BonusRepository;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Setup\Model\HrEmployees;
use Zend\View\Model\JsonModel;

class BonusController extends HrisController {

    public $adapter;

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(BonusRepository::class);
    }

    public function generateAction(){

        $fiscalYears = EntityHelper::getTableKVListWithSortOption($this->adapter, FiscalYear::TABLE_NAME, FiscalYear::FISCAL_YEAR_ID, [FiscalYear::FISCAL_YEAR_NAME], ["STATUS" => "E"], null, null, null, [null => '---'], true);

        return $this->stickFlashMessagesTo([
            'fiscalYears' => $fiscalYears
        ]);

    }

    public function reportAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $data = $request->getPost();

            $data = iterator_to_array($this->repository->fetchDetail($data['fiscalId'], $data['employeeId']), false);

            return new JsonModel(['success'=>true, 'data'=>$data]);

        }

        $fiscalYears = EntityHelper::getTableKVListWithSortOption($this->adapter, FiscalYear::TABLE_NAME, FiscalYear::FISCAL_YEAR_ID, [FiscalYear::FISCAL_YEAR_NAME], ["STATUS" => "E"], null, null, null, [null => '---'], true);
        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, HrEmployees::TABLE_NAME, HrEmployees::EMPLOYEE_ID, [HrEmployees::FULL_NAME], ["STATUS" => "E"], null, null, null, [null => '---'], true);

        return $this->stickFlashMessagesTo([
            'fiscalYears' => $fiscalYears,
            'employees' => $employees
        ]);

    }

    public function validateExistAction(){
        return new JsonModel(['exist'=>$this->repository->validateExist($this->getRequest()->getPost()['fiscalId'])]);
    }

    public function generateBonusAction(){
        // Create provision sheet
        $fiscalId = $this->getRequest()->getPost()['fiscalId'];
        $bonusAmount = $this->getRequest()->getPost()['bonusAmount'];
        
        $this->repository->generateBonus($fiscalId, $bonusAmount);

        return new  JsonModel([
            'success'=>true
        ]);
    }

    public function fetchBonusDataAction(){

        $fiscalId = $this->getRequest()->getPost()['fiscalId'];
        $data = iterator_to_array($this->repository->fetchDetail($fiscalId), false);
        $bonusAmount = $this->repository->fetchBonusAmount($fiscalId);

        return new JsonModel(['data'=> $data, 'success' => true, 'bonusAmount'=>$bonusAmount]);
    }

    public function wipeBonusAction(){

        $fiscalId = $this->getRequest()->getPost()['fiscalId'];
        $this->repository->wipeOut($fiscalId);

        return new JsonModel(['success' => true]);

    }

    

}