<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use MobileApi\Controller\Employee;
use Payroll\Repository\GradeRepository;
use Setup\Repository\JobHistoryRepository;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Zend\View\Model\JsonModel;
use Payroll\Model\GradePuraskarModel;

class GradeController extends HrisController {

    protected $adapter;
    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(GradeRepository::class);
    }

    public function indexAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $employeeId = $request->getPost()['employeeId'];
            $data = Helper::decodeBase64List(iterator_to_array($this->repository->fetchById($employeeId), false), ['REMARKS'], 0);

            return new JsonModel(['data'=>$data, 'success'=>true]);
        }

        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FIRST_NAME", "MIDDLE_NAME", "LAST_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FIRST_NAME", "ASC", " ", FALSE, TRUE);

        return $this->stickFlashMessagesTo([
            'employees' => $employees
        ]);
    }

    public function gradeChangeAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $searchData = $request->getPost();
            $data = iterator_to_array($this->repository->fetchByMonth($searchData), false);

            return new JsonModel(['data'=>$data, 'success'=>true]);
        }

        $employees = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FULL_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FULL_NAME", "ASC", " ", FALSE, TRUE);
        return $this->stickFlashMessagesTo([
            'employees' => $employees
        ]);
    }

    public function bulkGradeAction(){
        $postData = $this->getRequest()->getPost();
        $this->repository->updateGradeInfo($postData);

        return new JsonModel(['success'=>true]);
    }

    public function bulkIncrementAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {
            $data = iterator_to_array($this->repository->getPuraskarDate(), false);

            return new JsonModel(['data'=>$data, 'success'=>true]);
        }

    }

    public function bulkGradeAssignAction(){

        $request = $this->getRequest();
        $postedData = $request->getPost();

        $monthVal = $postedData['month'];
        $gradePoint = $postedData['gradePoint'];
        $remark = base64_encode($postedData['remark']);
        $fiscalYearId = $postedData['fiscalYearId'];
        $effectDate = date('Y-m-d', strtotime($postedData['effectDate']));

        // Store puraskar data
        $puraskarModel = new GradePuraskarModel();
        $puraskarModel->puraskarId = ((int) Helper::getMaxId($this->adapter, GradePuraskarModel::TABLE_NAME, GradePuraskarModel::PURASKAR_ID)) + 1;
        $puraskarModel->puraskarPoint = $gradePoint;
        $puraskarModel->remarks = $remark;
        $puraskarModel->effectDate = $effectDate;
        $puraskarModel->monthCondition = $monthVal;

        $this->repository->createPuraskar($puraskarModel);

        $this->repository->bulkGradeAssign($monthVal, $gradePoint, $postedData['remark'], $effectDate, $puraskarModel->puraskarId);

        return new JsonModel(['success' => true, 'error' => '']);
    }

    public function individualIncrementAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $requestData = $this->getRequest()->getPost(); 
            $data = $this->repository->pullEmployeeData($requestData);

            return new JsonModel(['data'=>$data, 'success'=>true]);
        }

        return Helper::addFlashMessagesToArray($this, [
            'searchValues' => EntityHelper::getSearchData($this->adapter),
        ]);
    }

    public function updateIndividualGradeAction(){

        $postData = $this->getRequest()->getPost();

        $this->repository->individualGradeInsertion($postData);

        return new JsonModel(['success'=>true]);
    }

    public function pullGradeAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {
            $fiscalYearId = $request->getPost()['fiscalYearId'];
            $monthNo = $request->getPost()['monthId'];

            $this->repository->callGradeMonthly($fiscalYearId, $monthNo);

            return new JsonModel(['success'=>true]);
        }

        return Helper::addFlashMessagesToArray($this, [
            'fiscalYears' => iterator_to_array($this->repository->getFiscalYear(), false),
            'months' => iterator_to_array($this->repository->getMonths(), false),
        ]);

    }

}