<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Application\Model\FiscalYear;
use Application\Model\Months;
use Exception;
use Payroll\Repository\PayrollReportRepo;
use Payroll\Repository\RulesRepository;
use Payroll\Repository\SalarySheetRepo;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class PayrollReportController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(PayrollReportRepo::class);

    }

    public function indexAction() {
        die();
        echo 'NO Action';
    }

    public function varianceAction() {
        $fiscalYears = EntityHelper::getTableList($this->adapter, FiscalYear::TABLE_NAME, [FiscalYear::FISCAL_YEAR_ID, FiscalYear::FISCAL_YEAR_NAME]);
        $months = EntityHelper::getTableList($this->adapter, Months::TABLE_NAME, [Months::MONTH_ID, Months::MONTH_EDESC, Months::FISCAL_YEAR_ID]);

        $columnsList = $this->repository->getVarianceColumns();
        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'fiscalYears' => $fiscalYears,
                    'months' => $months,
                    'columnsList' => $columnsList,
                    'preference' => $this->preference
        ]);
    }

    public function pullVarianceListAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();

            $results = $this->repository->getVarianceReprot($data);


            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($results);
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function gradeBasicAction() {
        $datas['otVariables'] = $this->repository->getGbVariables();
        $datas['monthList'] = $this->repository->getMonthList();

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'datas' => $datas,
                    'preference' => $this->preference
        ]);
    }

    public function pullGradeBasicAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $defaultColumnsList = $this->repository->getOtDefaultColumns();
            $reportType = $data['reportType'];
            if ($reportType == 'S') {
                $results = $this->repository->getGradeBasicSummary($data);
            } elseif ($reportType == 'D') {
                $results = $this->repository->getGradeBasicReport($data);
            } else {
                $results = $this->repository->getGradeBasicReport($data);
            }
            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($results);
            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function basicMonthlyReportAction() {
        $otVariables = $this->repository->getGbVariables();

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'otVariables' => $otVariables,
                    'preference' => $this->preference
        ]);
    }

    public function basicMonthlyAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $defaultColumnsList = $this->repository->getOtMonthlyDefaultColumns($data['fiscalId']);
            $results = $this->repository->getBasicMonthly($data, $defaultColumnsList);
            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($results);
            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function specialMonthlyReportAction() {
        $request = $this->getRequest();
        if ($request->isPost()) {
            $data = $request->getPost();
            $result = Helper::extractDbData($this->repository->getSpecialMonthly($data));
            return new JsonModel(['success' => true, 'data' => $result, 'message' => null]);
        }

        $otVariables = $this->repository->getGbVariables();

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'otVariables' => $otVariables,
                    'preference' => $this->preference
        ]);
    }

    // menu for this action not inserted
    public function groupSheetAction() {
        $nonDefaultList = $this->repository->getSalaryGroupColumns('S', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('S');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $data['salarySheetList'] = iterator_to_array($salarySheetRepo->fetchAll(), false);
        $links['getGroupListLink'] = $this->url()->fromRoute('payrollReport', ['action' => 'getGroupList']);
        $data['links'] = $links;

        return Helper::addFlashMessagesToArray($this, [
            'searchValues' => EntityHelper::getSearchData($this->adapter),
            'salaryType' => $salaryType,
            'nonDefaultList' => $nonDefaultList,
            'groupVariables' => $groupVariables,
            'preference' => $this->preference,
            'data' => json_encode($data)
        ]);
    }

    public function pullGroupSheetAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $resultData = [];
            $reportType = $data['reportType'];
            $groupVariable = $data['groupVariable'];

            if ($reportType == "GS") {
                $defaultColumnsList = $this->repository->getDefaultColumns('S');
                $resultData = $this->repository->getGroupReport('S', $data);
            } elseif ($reportType == "GD") {
                $defaultColumnsList = $this->repository->getVarianceDetailColumns($groupVariable);
                $resultData = $this->repository->getGroupDetailReport($data);
            }
            
            $monthDetails=EntityHelper::rawQueryResult($this->adapter, "SELECT MONTH_EDESC,YEAR FROM HRIS_MONTH_CODE WHERE MONTH_ID=?",['monthId'=>$data['monthId']])->current();
           
            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($resultData);
            $result['columns'] = $defaultColumnsList;
            $result['monthDetails'] = $monthDetails;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function employeeWiseGroupSheetAction() {
        $nonDefaultList = $this->repository->getSalaryGroupColumns('S', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('S');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $data['salarySheetList'] = iterator_to_array($salarySheetRepo->fetchAll(), false);
        $links['getGroupListLink'] = $this->url()->fromRoute('payrollReport', ['action' => 'getGroupList']);
        $data['links'] = $links;

        $fiscalYears = EntityHelper::getTableKVListWithSortOption($this->adapter, FiscalYear::TABLE_NAME,FiscalYear::FISCAL_YEAR_ID, [FiscalYear::FISCAL_YEAR_NAME], [FiscalYear::STATUS => 'E'], FiscalYear::FISCAL_YEAR_ID,  "DESC");
        return Helper::addFlashMessagesToArray($this, [
            'searchValues' => EntityHelper::getSearchData($this->adapter),
            'salaryType' => $salaryType,
            'fiscalYears' => $fiscalYears,
            'nonDefaultList' => $nonDefaultList,
            'groupVariables' => $groupVariables,
            'preference' => $this->preference,
            'data' => json_encode($data)
        ]);
    }

    public function pullemployeeWiseGroupSheetAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $resultData = [];
            $groupVariable = $data['groupVariable'];

            $defaultColumnsList = $this->repository->getDefaultColumns('S');
            $resultData = $this->repository->getEmployeeWiseGroupReport('S', $data);

            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($resultData);
            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function groupTaxReportAction() {
        $nonDefaultList = $this->repository->getSalaryGroupColumns('T', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('T');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'salaryType' => $salaryType,
//                    'fiscalYears' => $fiscalYears,
//                    'months' => $months,
                    'nonDefaultList' => $nonDefaultList,
                    'groupVariables' => $groupVariables,
                    'preference' => $this->preference,
                    'acl' => $this->acl,
        ]);
    }

    public function pullGroupTaxReportAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $resultData = [];
            $reportType = $data['reportType'];
            $groupVariable = $data['groupVariable'];

            if ($reportType == "GS") {
                $defaultColumnsList = $this->repository->getDefaultColumns('T');
                $resultData = $this->repository->getGroupReport('T', $data);
            } elseif ($reportType == "GD") {
                $defaultColumnsList = $this->repository->getVarianceDetailColumns($groupVariable);
                $resultData = $this->repository->getGroupDetailReport($data);
            }
            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($resultData);
            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function monthlySummaryAction() {
        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);
        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'salaryType' => $salaryType,
                    'preference' => $this->preference
        ]);
    }

    public function pullMonthlySummaryAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $resultData = [];
            $resultData['additionDetail'] = $this->repository->fetchMonthlySummary('A', $data);
            $resultData['deductionDetail'] = $this->repository->fetchMonthlySummary('D', $data);

            $result = [];
            $result['success'] = true;
            $result['data'] = $resultData;
//            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function departmentWiseAction() {
        $ruleRepo = new RulesRepository($this->adapter);
        $ruleList = iterator_to_array($ruleRepo->fetchAll(), false);

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'preference' => $this->preference,
                    'ruleList' => $ruleList,
                    'salaryType' => $salaryType
        ]);
    }

    public function pulldepartmentWiseAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
//            $resultData = [];
            $resultData = $this->repository->pulldepartmentWise($data);
//            $resultData['deductionDetail'] = $this->repository->fetchMonthlySummary('D', $data);

            $result = [];
            $result['success'] = true;
            $result['data'] = $resultData;
//           $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function jvReportAction() {
        $ruleRepo = new RulesRepository($this->adapter);
        $ruleList = iterator_to_array($ruleRepo->fetchAll(), false);

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $data = $request->getPost();
                $resultData = $this->repository->getJvReport($data);
                return new JSONModel(['success' => true, 'data' => $resultData, 'error' => '']);
            } catch (Exception $e) {
                return new JSONModel(['success' => false, 'data' => [], 'error' => '']);
            }
        }
        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'preference' => $this->preference,
                    'ruleList' => $ruleList,
                    'salaryType' => $salaryType
        ]);
    }

    public function taxYearlyAction() {

        $incomes = $this->repository->gettaxYearlyByHeads('IN');
        $taxExcemptions = $this->repository->gettaxYearlyByHeads('TE');
        $otherTax = $this->repository->gettaxYearlyByHeads('OT');
        $miscellaneous = $this->repository->gettaxYearlyByHeads('MI');
        $bMiscellaneou = $this->repository->gettaxYearlyByHeads('BM');
        $cMiscellaneou = $this->repository->gettaxYearlyByHeads('CM');
        $sumOfExemption = $this->repository->gettaxYearlyByHeads('SE', 'sin');
        $sumOfOtherTax = $this->repository->gettaxYearlyByHeads('ST', 'sin');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'salaryType' => $salaryType,
                    'preference' => $this->preference,
                    'incomes' => $incomes,
                    'taxExcemptions' => $taxExcemptions,
                    'otherTax' => $otherTax,
                    'miscellaneous' => $miscellaneous,
                    'bMiscellaneou' => $bMiscellaneou,
                    'cMiscellaneou' => $cMiscellaneou,
                    'sumOfExemption' => $sumOfExemption,
                    'sumOfOtherTax' => $sumOfOtherTax
        ]);
    }

    public function pulltaxYearlyAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();

            $resultData = $this->repository->getTaxYearly($data);
            $result = [];
            $result['success'] = true;
            $result['data']['employees'] = Helper::extractDbData($resultData);
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function getGroupListAction() {
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $data = EntityHelper::getTableList($this->adapter, "HRIS_SALARY_SHEET_GROUP", ["GROUP_ID", "GROUP_NAME"]);
                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }
    }

    public function pullGroupAction() {
        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $group=$data['group'];
            $monthId=$data['monthId'];
            $salaryTypeId=$data['salaryTypeId'];

            $sheetList= $salarySheetRepo->fetchGeneratedSheetByGroup($monthId,$group,$salaryTypeId);

            return new JsonModel(['success' => true, 'sheetData' => $sheetList, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function annualSalarySheetAction() {
        $nonDefaultList = $this->repository->getSalaryGroupColumns('N', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('N');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $data['salarySheetList'] = iterator_to_array($salarySheetRepo->fetchAll(), false);
        $links['getGroupListLink'] = $this->url()->fromRoute('payrollReport', ['action' => 'getGroupList']);
        $data['links'] = $links;

        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'salaryType' => $salaryType,
//                    'fiscalYears' => $fiscalYears,
//                    'months' => $months,
                    'nonDefaultList' => $nonDefaultList,
                    'groupVariables' => $groupVariables,
                    'preference' => $this->preference,
                    'data' => json_encode($data),
					'acl' => $this->acl,
                    'employeeDetail' => $this->storageData['employee_detail'],
        ]);
    }

    public function pullAnnualSalarySheetAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            // print_r($data);die;
            $resultData = [];
            $reportType = $data['reportType'];
            $groupVariable = $data['groupVariable'];
            $returnList = [];
            if ($reportType == "GS") {
                $defaultColumnsList = [];
                $resultData = $this->repository->getAnnualSheetReport('N', $data);
                $final = [];
                $employeeIdList = [];
                foreach ($resultData as $data){
                    if (!in_array($data['EMPLOYEE_ID'], $employeeIdList)){
                        array_push($employeeIdList,$data['EMPLOYEE_ID']);
                    }
                }
                
                foreach($employeeIdList as $eId){
                    $tempList = [];
                    $tempList['EMPLOYEE_ID'] = $eId;
                    foreach ($resultData as $data){
                        if($data['EMPLOYEE_ID'] == $eId){
                            $tempList['ID_ACCOUNT_NO'] = $data['ID_ACCOUNT_NO'];
                            $tempList['EMPLOYEE_CODE'] = $data['EMPLOYEE_CODE'];
                            $tempList['COMPANY_NAME'] = $data['COMPANY_NAME'];
                            $tempList['BANK_NAME'] = $data['BANK_NAME'];
                            $tempList['FULL_NAME'] = $data['FULL_NAME'];
                            $tempList[$data['PAY_EDESC']] = $data['AMOUNT'];
                        }
                    }
                    array_push($final,$tempList);
                }

                foreach ($final as $data){
                    $keys = array_keys($data);
                    foreach($keys as $k){
                        if ($k != 'EMPLOYEE_ID' && $k != 'ID_ACCOUNT_NO' && $k != 'EMPLOYEE_CODE' && $k != 'COMPANY_NAME' && $k != 'BANK_NAME' && $k != 'FULL_NAME')
                        if (!in_array(str_replace('/','_',str_replace(' ', '', $k)), $defaultColumnsList)){
                            $defaultColumnsList[str_replace('/','_',str_replace(' ', '', $k))]=$k;
                        }
                    }
                }
                foreach ($final as $data){
                    $keys = array_keys($data);
                    foreach($defaultColumnsList as $defaultColumn){
                        if (!in_array($defaultColumn, $keys)){
                            $data[$defaultColumn] = 0;
                        }
                    }
                    array_push($returnList,$data);
                }
                $tempFinal = [];
                foreach($returnList as $f){
                    $tempFinalRow = [];
                    $tempFinalRow['EMPLOYEE_ID'] = $f['EMPLOYEE_ID'];
                    $tempFinalRow['ID_ACCOUNT_NO'] = $f['ID_ACCOUNT_NO'];
                    $tempFinalRow['EMPLOYEE_CODE'] = $f['EMPLOYEE_CODE'];
                    $tempFinalRow['COMPANY_NAME'] = $f['COMPANY_NAME'];
                    $tempFinalRow['BANK_NAME'] = $f['BANK_NAME'];
                    $tempFinalRow['FULL_NAME'] = $f['FULL_NAME'];
                    foreach($defaultColumnsList as $k => $v){
                        $tempFinalRow[$k] = $f[$v];
                    }
                    array_push($tempFinal,$tempFinalRow);
                }
                

                $returnList = $tempFinal;
                unset($tempFinal);
            } elseif ($reportType == "GD") {
                $defaultColumnsList = $this->repository->getVarianceDetailColumns($groupVariable);
                $resultData = $this->repository->getGroupDetailReport($data);
            }
            
            // $monthDetails=EntityHelper::rawQueryResult($this->adapter, "SELECT MONTH_EDESC,YEAR FROM HRIS_MONTH_CODE WHERE MONTH_ID=:monthId",['monthId'=>$data['monthId']])->current();
            
            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($returnList);
            $result['columns'] = $defaultColumnsList;
            // $result['monthDetails'] = $monthDetails;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function tdsReportAction(){
        $nonDefaultList = $this->repository->getSalaryGroupColumns('N', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('N');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $payHeads = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_PAY_SETUP", "PAY_ID", ["PAY_EDESC"], ["PAY_CODE in ('11211', '11112')"], "PAY_ID", "ASC", "-");
        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'preference' => $this->preference,
					'acl' => $this->acl,
                    'employeeDetail' => $this->storageData['employee_detail'],
                    'payHeads' => $payHeads,
        ]);
    }

    public function pullTdsReportAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            
            $data = $this->repository->getTdsReport($data);
           
            return new CustomViewModel(['success' => true, 'data' => $data, 'error' => '']);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    public function gradeSankhyaReportAction(){
        $nonDefaultList = $this->repository->getSalaryGroupColumns('N', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('N');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $payHeads = EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_PAY_SETUP", "PAY_ID", ["PAY_EDESC"], ["PAY_CODE in ('11211', '11112')"], "PAY_ID", "ASC", "-");
        return Helper::addFlashMessagesToArray($this, [
                    'searchValues' => EntityHelper::getSearchData($this->adapter),
                    'preference' => $this->preference,
					'acl' => $this->acl,
                    'employeeDetail' => $this->storageData['employee_detail'],
                    'payHeads' => $payHeads,
        ]);
    }

    public function pullGradeSankhyaAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $data = $this->repository->getGradeSankhyaReport($data);
           
            return new CustomViewModel(['success' => true, 'data' => $data, 'error' => '']);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }
	
	public function finalReconcilationSheetAction() {
        $nonDefaultList = $this->repository->getSalaryGroupColumns('S', 'N');
        $groupVariables = $this->repository->getSalaryGroupColumns('S');

        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);

        $data['salarySheetList'] = iterator_to_array($salarySheetRepo->fetchAll(), false);
        $links['getGroupListLink'] = $this->url()->fromRoute('payrollReport', ['action' => 'getGroupList']);
        $data['links'] = $links;

        $fiscalYears = EntityHelper::getTableKVListWithSortOption($this->adapter, FiscalYear::TABLE_NAME,FiscalYear::FISCAL_YEAR_ID, [FiscalYear::FISCAL_YEAR_NAME], [FiscalYear::STATUS => 'E'], FiscalYear::FISCAL_YEAR_ID,  "DESC");
        return Helper::addFlashMessagesToArray($this, [
            'searchValues' => EntityHelper::getSearchData($this->adapter),
            'salaryType' => $salaryType,
            'fiscalYears' => $fiscalYears,
            'nonDefaultList' => $nonDefaultList,
            'groupVariables' => $groupVariables,
            'preference' => $this->preference,
            'data' => json_encode($data)
        ]);
    }

    public function pullFinalReconcilationSheetAction() {
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $resultData = [];
            $groupVariable = $data['groupVariable'];
            $defaultColumnsList = $this->repository->getDefaultColumns('S');
            $resultData = $this->repository->getFinalReconcilationSheetReport($data);

            $result = [];
            $result['success'] = true;
            $result['data'] = Helper::extractDbData($resultData);
            $result['columns'] = $defaultColumnsList;
            $result['error'] = "";
            return new CustomViewModel($result);
        } catch (Exception $e) {
            return new CustomViewModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }
	
	
    public function getEmployeeAdditionBreakDownAction(){
        try {
           $request = $this->getRequest();
           if (!$request->isPost()) {
               throw new Exception("The request should be of type post");
           }
           $postData = $request->getPost();
        //    print_r($postData);die;
           $data = $this->repository->getEmployeeAdditionBreakDown($postData['fiscalYear'],$postData['employeeId']);
           return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
       } catch (Exception $e) {
           return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
       }
   }

    public function getEmployeeSubDetailAction(){
        try {
            $request = $this->getRequest();
            if (!$request->isPost()) {
                throw new Exception("The request should be of type post");
            }
            $postData = $request->getPost();
            $data = $this->repository->getEmployeeSubDetail($postData['payId'],$postData['employeeId'], $postData['fiscalYear']);
            return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
        }
    }

    private $month = [
        1 => "Baisakh",
        2 => "Jestha",
        3 => "Asar",
        4 => "Shrawan",
        5 => "Bhadra",
        6 => "Ashwin",
        7 => "Kartik",
        8 => "Mangsir",
        9 => "Poush",
        10 => "Magh",
        11 => "Falgun",
        12 => "Chaitra"
    ];

    public function postSapAction(){

        $years = $this->repository->getAllYears();

        return Helper::addFlashMessagesToArray($this, [
            'months'=> $this->month,
            'years' => $years,
            'locations' => EntityHelper::getTableList($this->adapter,'HRIS_LOCATIONS', ['LOCATION_ID','LOCATION_EDESC'], ["STATUS" => "E"])
        ]);
    }


// TODO: 1. Select current date and month on ui automatically.
    // 2. Implement curl function for api post
    // 3. Validation and log maintainence for sap posting

    public function postSapSubmitActionNew(){

        $request = $this->getRequest();

        $yearId = $request->getPost()['yearId'];
        $monthNo = $request->getPost()['monthNo'];
        $salaryType = $request->getPost()['salaryType'];
        $employeeType = $request->getPost()['employeeType'];
        $locationId = $request->getPost()['location_id'];


        $monthId = $this->repository->getMonthId($yearId, $monthNo);

        $sheetIds = iterator_to_array($this->repository->getSheetIds($monthId, $salaryType, $employeeType,$locationId), false);
        $monthInfo = iterator_to_array($this->repository->getMonthInfo($monthId), false)[0]['MONTH_NO'];

        $singleData = $sheetIds[0];
        $payType = $singleData['SALARY_TYPE_ID'] == '1' ? 'NormalSalary' : 'OvertimeSalary';
        $year = $this->repository->getYear($monthId);

        $returnData = array(
            "U_DocDate"=> date("Y-m-d"),
            "U_Month"=>$this->month[$monthInfo],
            "U_PayType"=> $payType,
            "U_EmpType"=> $singleData['EMPLOYEE_TYPE'],
            "U_Remarks"=> null,
            "U_Year" => $year,
            "MST1Collection"=>[]
        );



        // Login information


        $content = array(
                "CompanyDB"=> "FINAL_20231221",
                "Password"=> "NoAs@2021",
                "UserName"=> "manager"
            );
        
        $url = "https://172.16.100.1:50000/b1s/v1/Login";

        $cookie_file = 'cookies.txt'; 
    
        # Make the call using API.
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);

        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        
        curl_setopt($ch, CURLOPT_COOKIEJAR, $cookie_file);
        curl_setopt($ch, CURLOPT_COOKIEFILE, $cookie_file);
        
        // Response
        $response = curl_exec($ch);
        // var_dump($response); die;
        curl_close($ch);
        // die;
        $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        $url = "https://172.16.100.1:50000/b1s/v1/OMST";

        $ch1 = curl_init(); // Create a new cURL handle

        // print_r(json_encode($returnData)); die;


        curl_setopt($ch1, CURLOPT_URL, $url);
        curl_setopt($ch1, CURLOPT_POST, 1);
        curl_setopt($ch1, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch1, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch1, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch1, CURLOPT_VERBOSE, true);
        curl_setopt($ch1, CURLOPT_COOKIEFILE, $cookie_file);
        curl_setopt($ch1, CURLOPT_POSTFIELDS, json_encode($returnData));
        curl_setopt($ch1, CURLOPT_TIMEOUT, 100000);

        curl_setopt($ch1, CURLOPT_CONNECTTIMEOUT, 100000); 
        // $length = strlen(json_encode($returnData));
        // curl_setopt($ch1, CURLOPT_HTTPHEADER, ["Content-Length: $length"]);

        // print_r($length);die;

        // Response
        $response = curl_exec($ch1);
        $status_code = curl_getinfo($ch1, CURLINFO_HTTP_CODE);
        curl_close($ch1);
        $docNumber = json_decode($response)->DocEntry;



        foreach($sheetIds as $idCol){
            $id = $idCol['SHEET_NO'];
            $sheetData = iterator_to_array($this->repository->getFilterData($id), false)[0];
            $data = array("fiscalId"=>$sheetData['FISCAL_ID'], "reportType"=>"S", "monthId"=>$sheetData["MONTH_ID"], 
                        "salaryTypeId"=>$sheetData["SALARY_TYPE_ID"], "groupId"=>$sheetData["GROUP_ID"]);


            $columnList = $this->repository->getDefaultColumnsSap('S');
            $resultData = $this->repository->getGroupReport('S', $data);

            foreach($resultData as $row){

                $singleData = array(
                    "U_EmpCode"=> $row["SAP_CODE"],
                    "U_EmpName"=> $row["FULL_NAME"],
                    "U_cBranch"=> $row["SAP_BRANCH_ID"],
                    "U_nBranch"=> $row["LOCATION_CODE"],

                );

                foreach($columnList as $column){
                    $singleData[$column["SAP_HEADING"]] = floatval($row[$column["VARIANCE"]]); 
                }

                if($employeeType == 'CONTRACT'){
                    $singleData['U_Head2'] = $singleData['U_Head1'];
                    $singleData['U_Head1'] = 0;
                    $singleData['U_Head48'] = $singleData['U_Head47'];
                    $singleData['U_Head47'] = 0;
                }else{
                    $singleData['U_Head2'] = 0;
                    $singleData['U_Head48'] = 0;
                }

                $singleData['U_Head45'] = 0;
                $singleData['U_Head46'] = 0;

                array_push($returnData["MST1Collection"], $singleData);

                $url = "https://172.16.100.1:50000/b1s/v1/MST1";

                $ch2 = curl_init(); // Create a new cURL handle

                // print_r(json_encode($singleData)); die;


                curl_setopt($ch2, CURLOPT_URL, $url);
                curl_setopt($ch2, CURLOPT_POST, 1);
                curl_setopt($ch2, CURLOPT_SSL_VERIFYHOST, 0);
                curl_setopt($ch2, CURLOPT_SSL_VERIFYPEER, 0);
                curl_setopt($ch2, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($ch2, CURLOPT_VERBOSE, true);
                curl_setopt($ch2, CURLOPT_COOKIEFILE, $cookie_file);
                curl_setopt($ch2, CURLOPT_POSTFIELDS, json_encode($singleData));
                curl_setopt($ch2, CURLOPT_TIMEOUT, 100000);

                curl_setopt($ch2, CURLOPT_CONNECTTIMEOUT, 100000); 
                // $length = strlen(json_encode($returnData));
                // curl_setopt($ch1, CURLOPT_HTTPHEADER, ["Content-Length: $length"]);

                // print_r($length);die;

                // Response
                $response = curl_exec($ch2);
                $status_code = curl_getinfo($ch2, CURLINFO_HTTP_CODE);
                curl_close($ch2);
                // var_dump($response); die;

            }
            // break;
        }
        // echo '<pre>'; print_r($args); die;

        // Finally post to sap
        


        $this->repository->updateLog($monthId, $salaryType, $employeeType, $this->employeeId);

        return new JsonModel(['success'=>true]);

    }


    public function postSapSubmitAction(){

        $request = $this->getRequest();

        $yearId = $request->getPost()['yearId'];
        $monthNo = $request->getPost()['monthNo'];
        $salaryType = $request->getPost()['salaryType'];
        $employeeType = $request->getPost()['employeeType'];
        $locationId = $request->getPost()['location_id'];


        $monthId = $this->repository->getMonthId($yearId, $monthNo);

        $sheetIds = iterator_to_array($this->repository->getSheetIds($monthId, $salaryType, $employeeType,$locationId), false);
        $monthInfo = iterator_to_array($this->repository->getMonthInfo($monthId), false)[0]['MONTH_NO'];

        $singleData = $sheetIds[0];
        $payType = $singleData['SALARY_TYPE_ID'] == '1' ? 'NormalSalary' : 'OvertimeSalary';
        $year = $this->repository->getYear($monthId);

        $returnData = array(
            "U_DocDate"=> date("Y-m-d"),
            "U_Month"=>$this->month[$monthInfo],
            "U_PayType"=> $payType,
            "U_EmpType"=> $singleData['EMPLOYEE_TYPE'],
            "U_Remarks"=> null,
            "U_Year" => $year,
            "MST1Collection"=>array()
        );



        foreach($sheetIds as $idCol){
            $id = $idCol['SHEET_NO'];
            $sheetData = iterator_to_array($this->repository->getFilterData($id), false)[0];
            $data = array("fiscalId"=>$sheetData['FISCAL_ID'], "reportType"=>"S", "monthId"=>$sheetData["MONTH_ID"], 
                        "salaryTypeId"=>$sheetData["SALARY_TYPE_ID"], "groupId"=>$sheetData["GROUP_ID"]);


            $columnList = $this->repository->getDefaultColumnsSap('S');
            $resultData = $this->repository->getGroupReport('S', $data);

            $count = 0;
            // $lineId = 1;
            foreach($resultData as $row){
                $count = $count + 1;
                // $lineId = $lineId+1;

                $singleData = array(
                    // "LineId" => $lineId,
                    "U_EmpCode"=> $row["SAP_CODE"],
                    "U_EmpName"=> $row["FULL_NAME"],
                    "U_cBranch"=> $row["SAP_BRANCH_ID"],
                    "U_nBranch"=> $row["LOCATION_CODE"],

                );

                foreach($columnList as $column){
                    $value = floatval($row[$column["VARIANCE"]]);
                    if($value == 0){
                        continue;
                    }
                    $singleData[$column["SAP_HEADING"]] = $value; 
                }

                // $singleData['U_Head1'] = floatval($row[$columnList[3]["VARIANCE"]]); 

                // if($employeeType == 'CONTRACT'){
                //     $singleData['U_Head2'] = $singleData['U_Head1'];
                //     $singleData['U_Head1'] = 0;
                //     $singleData['U_Head48'] = $singleData['U_Head47'];
                //     $singleData['U_Head47'] = 0;
                // }else{
                //     $singleData['U_Head2'] = 0;
                //     $singleData['U_Head48'] = 0;
                // }

                // $singleData['U_Head45'] = 0;
                // $singleData['U_Head46'] = 0;

                array_push($returnData["MST1Collection"], $singleData);
                // if($count == 2){
                //     break;
                // }
            }
        //    break;
        }

        // print_r(json_encode($returnData)); die;

        $useAPI ='N';
        if ($useAPI=='Y'){
            // Login information


                $content = array(
                    "CompanyDB"=> "FINAL_20231221",
                    "Password"=> "NoAs@2021",
                    "UserName"=> "manager"
                );
            
            $url = "https://172.16.100.1:50000/b1s/v1/Login";

            $cookie_file = 'cookies.txt'; 

            # Make the call using API.
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_POST, 1);

            curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($content));
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
            
            curl_setopt($ch, CURLOPT_COOKIEJAR, $cookie_file);
            curl_setopt($ch, CURLOPT_COOKIEFILE, $cookie_file);
            
            // Response
            $response = curl_exec($ch);
            // var_dump($response); die;
            curl_close($ch);
            // die;
            $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

            // $returnData['MST1Collection'][0]['LineId']=9;
            // $returnData['MST1Collection'][2]=[
            //             "U_EmpCode" => '962',
            //             "U_EmpName" => 'Bharat Kumar Bista',
            //             "U_cBranch" => 5,
            //             "U_nBranch" => 'PRO003',
            //             "U_Head3" => 4500,
            //             "U_Head1" => 33822,
            //             "U_Head11" => 2000,
            //             "U_Head18" => 3382.2,
            //             "U_Head13" => 2300,
            //             "U_Head19" => 10655.03,
            //             "U_Head29" => 6764.4,
            //             "U_Head30" => 2500,
            //             "U_Head33" => 511.36,
            //             "U_Head34" => 704.09,
            //             "U_Head28" => 6094.64,
            //             "U_Head47" => 23099.68,
            //             "U_Head7" => 300,
            //             "U_Head16" => 4025,
            // ];
            $returnData['MST1Collection'][2]= $returnData['MST1Collection'][1];
            // $returnData['MST1Collection'][2]['LineId']=4;
            // echo '<pre>';print_r($returnData); die;
            $args = json_encode($returnData);
            // echo '<pre>'; print_r($args); die;

            // Finally post to sap
            $url = "https://172.16.100.1:50000/b1s/v1/OMST";

            $ch1 = curl_init(); // Create a new cURL handle
            curl_setopt($ch1, CURLOPT_TIMEOUT, 100000);

            curl_setopt($ch1, CURLOPT_CONNECTTIMEOUT, 100000); 


            curl_setopt($ch1, CURLOPT_URL, $url);
            curl_setopt($ch1, CURLOPT_POST, 1);
            curl_setopt($ch1, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($ch1, CURLOPT_SSL_VERIFYPEER, 0);
            curl_setopt($ch1, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch1, CURLOPT_VERBOSE, true);
            curl_setopt($ch1, CURLOPT_COOKIEFILE, $cookie_file);
            curl_setopt($ch1, CURLOPT_POSTFIELDS, json_encode($returnData, JSON_UNESCAPED_UNICODE));
            // curl_setopt($ch1, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
            // curl_setopt($ch1, CURLOPT_RETURNTRANSFER, true);
            // $length = strlen(json_encode($returnData));
            // curl_setopt($ch1, CURLOPT_HTTPHEADER, ["Content-Length: $length"]);

            // print_r($length);die;

            // Response
            $response = curl_exec($ch1);
            $status_code = curl_getinfo($ch1, CURLINFO_HTTP_CODE);
            // if ($status_code == 201) {
            //     echo "Data inserted successfully!";
            // } else {
            //     echo "Insertion failed with HTTP code: " . $status_code . "\n";
            //     echo "Error message: " . $response;
            // }
            
            curl_close($ch1);
        }else{
            $this->repository->manualInsert($returnData);
        }
        


        $this->repository->updateLog($monthId, $salaryType, $employeeType, $this->employeeId);

        return new JsonModel(['success'=>true]);

    }



    public function validateSapPostAction(){
        $request = $this->getRequest();
        $yearId = $request->getPost()['yearId'];
        $monthNo = $request->getPost()['monthNo'];
        $salaryType = $request->getPost()['salaryType'];
        $employeeType = $request->getPost()['employeeType'];
        $locationId = $request->getPost()['location_id'];


        $monthId = $this->repository->getMonthId($yearId, $monthNo);


        $exists = $this->repository->validateLog($monthId, $salaryType, $employeeType,$locationId) > 0 ? true : false;

        return new JsonModel(["exists"=>$exists]);


    }

    public function postSapActionBkp(){

        $id = (int) $this->params()->fromRoute('id');

        $sheetData = iterator_to_array($this->repository->getFilterData($id), false)[0];
        $data = array("fiscalId"=>$sheetData['FISCAL_ID'], "reportType"=>"S", "monthId"=>$sheetData["MONTH_ID"], 
                    "salaryTypeId"=>$sheetData["SALARY_TYPE_ID"], "groupId"=>$sheetData["GROUP_ID"]);


        $payType = $data['salaryTypeId'] == '1' ? 'NormalSalary' : 'OvertimeSalary';

        $columnList = $this->repository->getDefaultColumnsSap('S');
        $resultData = $this->repository->getGroupReport('S', $data);




        // var_dump($data); die;

        // dd()
        $year = $this->repository->getYear($data['monthId']);

        $employeeType = $resultData[0]['EMPLOYEE_TYPE'];

        $returnData = [
            "U_DocDate"=> $sheetData["CREATED_DT"],
            "U_Month"=>$this->month[$sheetData["MONTH_NO"]],
            "U_PayType"=> $payType,
            "U_EmpType"=> $employeeType,
            "U_Remarks"=> null,
            "U_Year" => $year,
            "MST1Collection"=>[]
        ];

        foreach($resultData as $row){

            $singleData = [
                "U_EmpCode"=> $row["SAP_CODE"],
                "U_EmpName"=> $row["FULL_NAME"],
                "U_cBranch"=> $row["SAP_BRANCH_ID"],
                "U_nBranch"=> $row["LOCATION_CODE"],

            ];

            foreach($columnList as $column){
                $singleData[$column["SAP_HEADING"]] = $row[$column["VARIANCE"]]; 
            }

            if($employeeType == 'CONTRACT'){
                $singleData['U_Head2'] = $singleData['U_Head1'];
                $singleData['U_Head1'] = "0";
                $singleData['U_Head48'] = $singleData['U_Head47'];
                $singleData['U_Head47'] = "0";
            }else{
                $singleData['U_Head2'] = "0";
                $singleData['U_Head48'] = "0";
            }

            $singleData['U_Head45'] = "0";
            $singleData['U_Head46'] = "0";

            array_push($returnData["MST1Collection"], $singleData);

        }

        return new JsonModel($returnData);

    }

    private function dbConn(){
        $dsn = '172.16.100.1';
        $database = 'NOCHR';
        $username = 'NOCHR';
        $password = 'Noc$%^@123';

        // Create a mysqli connection
$mysqli = new mysqli($dsn, $username, $password, $database);

// Check the connection
if ($mysqli->connect_error) {
    die("Connection failed: " . $mysqli->connect_error);
}

var_dump("check"); die;

        $odbc_connection= odbc_connect($dsn, $username, $password, SQL_CUR_USE_ODBC);

        return $odbc_connection;
    }

    private function close_dbconn($odbc_connection){
        odbc_close($odbc_connection);
    }

}
