<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use MobileApi\Controller\Employee;
use Payroll\Form\PayrollSuspendForm;
use Payroll\Model\PayrollSuspendModel;
use Payroll\Repository\GradeRepository;
use Payroll\Repository\PayrollSuspendRepository;
use Setup\Repository\JobHistoryRepository;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Zend\View\Model\JsonModel;

class PayrollSuspendController extends HrisController {

    protected $adapter;
    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(PayrollSuspendRepository::class);
        $this->initializeForm(PayrollSuspendForm::class);
    }

    public function indexAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {

            $searchData = $request->getPost();

            $data = iterator_to_array($this->repository->getSuspendData($searchData), false);

            return new JsonModel(['success'=>true, 'data'=>$data]);

        }

        return Helper::addFlashMessagesToArray($this, [
            'employeeList'=>EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FULL_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FIRST_NAME", "ASC", " ", FALSE, TRUE),
        ]);

    }

    public function addAction(){


        $request = $this->getRequest();

        if ($request->isPost()) {

            $this->form->setData($request->getPost());

            if ($this->form->isValid()) {

                $model = new PayrollSuspendModel();
                $model->exchangeArrayFromForm($this->form->getData());
                $model->modificationId = ((int) Helper::getMaxId($this->adapter, PayrollSuspendModel::TABLE_NAME, PayrollSuspendModel::MODIFICATION_ID)) + 1;
                $model->createdDate = Helper::getcurrentExpressionDate();
                $model->status = 'E';

                $this->repository->add($model);

                $this->flashmessenger()->addMessage("Payroll Suspend successfully added.");
                return $this->redirect()->toRoute("payrollSuspend");

            }

        }

        $monthData = iterator_to_array($this->repository->monthData(), false);

        $monthList = [];

        foreach($monthData as $month){

            $monthList[$month['MONTH_ID']] = $month['MONTH_TEXT'];

        }

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employeeList'=>EntityHelper::getTableKVListWithSortOption($this->adapter, "HRIS_EMPLOYEES", "EMPLOYEE_ID", ["FULL_NAME"], ['STATUS'=>'E', 'EMPLOYEE_TYPE' => 'R', 'RETIRED_FLAG'=>'N'], "FIRST_NAME", "ASC", " ", FALSE, TRUE),
            'monthData' => $monthList,
        ]);

    }

    public function viewAction(){

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("payrollSuspend", array("action"=>"index"));
        }

        $data = $this->repository->fetchById($id);
        $documents = iterator_to_array($this->repository->fetchDocument($id), false);

        return Helper::addFlashMessagesToArray($this, [
            'data' => $data,
            'documents'=> $documents
        ]);

    }

    public function deleteAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("payrollSuspend", array("action"=>"index"));
        }

        // Update the status
        $this->repository->delete($id);

        $this->flashmessenger()->addMessage("Payroll Suspend deleted successfully");
        return $this->redirect()->toRoute("payrollSuspend", array("action"=>"index"));

    }

    public function fileUploadAction()
    {
        $request = $this->getRequest();
        $responseData = [];
        $files = $request->getFiles()->toArray();
        try {
            if (sizeof($files) > 0) {
                $ext = pathinfo($files['file']['name'], PATHINFO_EXTENSION);
                if ($ext == 'txt' || $ext == 'pdf' || $ext == 'jpg' || $ext == 'jpeg' || $ext == 'png' || $ext=='docx' || $ext=='odt' || $ext=='doc' ) {
                    $fileName = pathinfo($files['file']['name'], PATHINFO_FILENAME);
                    $unique = Helper::generateUniqueName();
                    $newFileName = $unique . "." . $ext;
                    if (!file_exists(Helper::UPLOAD_DIR . "/suspend_document/")) {
                        mkdir(Helper::UPLOAD_DIR . "/suspend_document/", 0777, true);
                    }
                    $success = move_uploaded_file($files['file']['tmp_name'], Helper::UPLOAD_DIR . "/suspend_document/" . $newFileName);
                    if (!$success) {
                        throw new Exception("Upload unsuccessful.");
                    }
                    $responseData = ["success" => true, "data" => ["fileName" => $newFileName, "oldFileName" => $fileName . "." . $ext]];
                } else { 
                    throw new Exception("Upload unsuccessful.");
                }
            }
        } catch (Exception $e) {
            $responseData = [
                "success" => false,
                "message" => $e->getMessage(),
                "traceAsString" => $e->getTraceAsString(),
                "line" => $e->getLine()
            ];
        }
        return new JsonModel($responseData);
    }

    public function pushFileLinkAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $this->repository->pushFileLink($data);
            return new JsonModel(['success' => true, 'data' => $data, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function pullFilesByIdAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $files = $this->repository->pullFilesbyModificationId($data['id']);
            return new JsonModel(['success' => true, 'data' => $files, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function deleteFileByNameAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $this->repository->deleteFileByName($data['name']);
            return new JsonModel(['success' => true, 'data' => null, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

}