<?php

namespace Payroll\Controller;

use Application\Controller\HrisController;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use Payroll\Repository\RulesRepository;
use Payroll\Repository\SalarySheetRepo;
use Payroll\Repository\TaxSheetRepo;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class TaxSheetController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(TaxSheetRepo::class);
    }

    public function indexAction() {
        
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $postedData = $request->getPost();

                // echo "<pre>";
                // print_r($postedData);die;

                $companyId = isset($postedData['companyId']) ? $postedData['companyId'] : -1;
                $branchId = isset($postedData['branchId']) ? $postedData['branchId'] : -1;
                $locationId = isset($postedData['locationId']) ? $postedData['locationId'] : -1;
                $departmentId = isset($postedData['departmentId']) ? $postedData['departmentId'] : -1;
                $designationId = isset($postedData['designationId']) ? $postedData['designationId'] : -1;
                $positionId = isset($postedData['positionId']) ? $postedData['positionId'] : -1;
                $serviceTypeId = isset($postedData['serviceTypeId']) ? $postedData['serviceTypeId'] : -1;
                $serviceEventTypeId = isset($postedData['serviceEventTypeId']) ? $postedData['serviceEventTypeId'] : -1;
                $employeeTypeId = isset($postedData['employeeTypeId']) ? $postedData['employeeTypeId'] : -1;
                $genderId = isset($postedData['genderId']) ? $postedData['genderId'] : -1;
                $employeeId = isset($postedData['employeeId']) ? $postedData['employeeId'] : -1;
                $monthId=$postedData['monthId'];
                $fiscalYearId=$postedData['fiscalYearId'];
                $salaryType=isset($postedData['salaryType'])? $postedData['salaryType'] : 1;

                $data = $this->repository->fetchTaxSheetPivoted($companyId, $branchId, $departmentId, $designationId, $positionId, $serviceTypeId, $serviceEventTypeId, $employeeTypeId, $genderId, $employeeId, $locationId, $fiscalYearId,$monthId,$salaryType);

                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }
        $ruleRepo = new RulesRepository($this->adapter);
        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        
        //extracted manually 
        $taxPayIds=[6,38,40,42,43,44,45,46,47,54,69,87,88,89,90,94,95,96,97,98,107,115,149];
        
        $taxPayIdData = iterator_to_array($ruleRepo->fetchAll(), false);
        $i=0;
        foreach($taxPayIdData as $key){
            if(in_array($key['PAY_ID'],$taxPayIds)){
                $taxdata[$i]['PAY_ID']=$key['PAY_ID'];
                $taxdata[$i]['PAY_EDESC']=$key['PAY_EDESC'];
                $taxdata[$i]['PAY_TYPE_FLAG']=$key['PAY_TYPE_FLAG'];
                $i++;
            }

        }

        $data['ruleList']=$taxdata;

        $data['acl'] = $this->acl;
        
        $salaryTypeList =EntityHelper::getTableList($this->adapter, hris_salary_type, ['SALARY_TYPE_ID' => "SALARY_TYPE_ID", 'SALARY_TYPE_NAME'=>"SALARY_TYPE_NAME"]);
        
        $data['salaryTypeList']=$salaryTypeList;

        // echo "<pre>";
        //         print_r($data);die;


        return $data;
    }

    public function taxslipAction() {
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $postedData = $request->getPost();
                $data = $this->repository->fetchEmployeeTaxSlip($postedData['monthId'], $postedData['employeeId']);
                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }
    }

}
