<?php

namespace Payroll\Repository;

use Application\Model\Model;
use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;
use Payroll\Model\BasicSalaryModel;

class BasicSalaryRepository extends HrisRepository{

    public function __construct(AdapterInterface $adapter, $tablename = null) {
      parent::__construct($adapter, BasicSalaryModel::TABLE_NAME);
    }

    public function add($data){
        $array = $data->getArrayCopyForDb();
        $this->tableGateway->insert($array);
    }

    public function all($searchData){

      $sql = "
        select 
        gcms.egc_id as egc_id,
        hfl.functional_level_no as functional_level,
        hp.position_name as position,
        gcms.basic_salary as basic_salary,
        gcms.grade_ceiling_no as grade_ceiling_no,
        gcms.from_date as from_date,
        gcms.to_date as to_date,
        gcms.salary_type as salary_type,
        gcms.one_day_salary as one_day_salary
        from hris_employees_grade_ceiling_master_setup gcms
        left join hris_functional_levels hfl on hfl.functional_level_id = gcms.functional_level_id
        left join hris_positions hp on hp.position_id = gcms.position_id
        where gcms.status = 'E' and gcms.is_new = 'Y'
      ";

      if($searchData['functionalId'] != ''){
        $sql = $sql . " and gcms.functional_level_id = {$searchData['functionalId']}";
      }

      if($searchData['positionId'] != ''){
        $sql = $sql . " and gcms.position_id = {$searchData['positionId']}";
      }

      if($searchData['salaryType'] != ''){
        $salaryType = $searchData['salaryType'];
        $sql = $sql . " and gcms.salary_type = '{$salaryType}'";
      }

      $orderByString = " order by gcms.from_date desc, hp.order_id, gcms.salary_type desc";
      $sql = $sql . $orderByString;

      return $this->adapter->query($sql)->execute();

    }

    public function fetchById($id){
        return $this->tableGateway->select([BasicSalaryModel::EGC_ID=>$id])->current();
    }

    public function edit(Model $model, $id) {
      $array = $model->getArrayCopyForDB();
      $this->tableGateway->update($array, [BasicSalaryModel::EGC_ID => $id]);
  }

   public function validateOverlap($functionalId, $positionId, $fromDate, $salaryType, $egcId){

      $functionalId = $functionalId == null ? "is null" : " = {$functionalId}";

      $sql = "
          select
          from_date,
          to_date 
          from hris_employees_grade_ceiling_master_setup 
          where position_id = {$positionId} 
          and status = 'E' 
          and from_date = '{$fromDate}'
          and salary_type = '{$salaryType}' 
          and functional_level_id {$functionalId} 
        ";

      if($egcId != null){
        $sql = $sql . " and egc_id <> {$egcId}";
      }

      return $this->adapter->query($sql)->execute();

   }

   public function updateToDate($newFromDate, $functionalLevelId, $positionId, $salaryType, $setNull = false){

    $functionalId = $functionalLevelId == null ? "null" : "{$functionalLevelId}";
    $setSql = $setNull == true ? "null" : "(select add_days('{$newFromDate}', -1) from dummy)";

    $sql = "
        update hris_employees_grade_ceiling_master_setup
        set to_date = {$setSql}
        where EGC_ID = GET_LATEST_BASIC_SALARY_ID ( {$positionId}, '{$salaryType}', {$functionalId})
    ";

    $this->adapter->query($sql)->execute();

   }

   public function updateGrade($functionalId, $positionId, $fromDate, $salaryType, $oneDaySalary, $egcId){

    $functionalId = $functionalId == null ? 'null' : $functionalId;
    $oneDaySalary = $oneDaySalary == null ? 'null' : $oneDaySalary;

    $sql = "
      CALL HRIS_UPDATE_GRADE_ON_SALARY_INCREMENT({$functionalId}, {$positionId}, '{$fromDate}', '{$salaryType}', {$oneDaySalary}, {$egcId});
    ";

    $this->adapter->query($sql)->execute();
   }

   public function revertGrade($egcId){
      $sql = "
      CALL HRIS_REVERT_OLD_GRADE($egcId);
      ";

      $this->adapter->query($sql)->execute();
   }

   public function deleteGradeChangeHistory($egcId){
    $sql = "
      delete from hris_grade_history where egc_id = {$egcId}
    ";

    $this->adapter->query($sql)->execute();
   }

   public function updateGradeHistory($employeeId, $currentGrade, $newGrade, $fiscalYearId, $remarks, $effectDate, $historyId){

     $sql = "select GET_SN_NO({$employeeId}, '{$effectDate}') as sn_no from dummy";

     $snNo = $this->adapter->query($sql)->execute()->current()['SN_NO'];

     // Update up all the required data
      $sql = "UPDATE HRIS_GRADE_HISTORY SET SN_NO = SN_NO + 1 WHERE EMPLOYEE_ID = {$employeeId} AND SN_NO >= {$snNo}";
      $this->adapter->query($sql)->execute();

      $sql = "INSERT INTO hris_grade_history (EMPLOYEE_ID, PREVIOUS_GRADE, NEW_GRADE, FISCAL_YEAR_ID, REMARKS, EFFECT_DATE, SN_NO, PROMOTION_ID)
              VALUES ({$employeeId}, {$currentGrade}, {$newGrade}, {$fiscalYearId}, '{$remarks}', '{$effectDate}', {$snNo}, {$historyId} )";

      $this->adapter->query($sql)->execute();

      $this->recalculateGrade($employeeId, $snNo-1);
   }

   public function delete($id){

    $sql = "
      UPDATE HRIS_EMPLOYEES_GRADE_CEILING_MASTER_SETUP SET STATUS = 'D' WHERE EGC_ID = {$id}
    ";

    $this->adapter->query($sql)->execute();

   }

   public function deleteGradeHistory($id){

    $sql = "CALL HRIS_DELETE_GRADE_HISTORY({$id})";

    $this->adapter->query($sql)->execute();

   }

   public function deleteExistingUnAppliedGrade($employeeId, $positionId, $functionalId, $effectDate){

    $sql = "select sn_no from hris_grade_history where employee_id = {$employeeId} 
            and is_implemented = 'N' and egc_id = GET_LATEST_BASIC_SALARY_ID({$positionId}, 'MAIN', {$functionalId})";

    $snNo = $this->adapter->query($sql)->execute()->current()['SN_NO'];

    if($snNo != null){
      $sql = "delete 
              from hris_grade_history where employee_id = {$employeeId} and sn_no = {$snNo} and effect_date > '{$effectDate}'";

      $this->adapter->query($sql)->execute();

      // Update down all the required data
      $sql = "UPDATE HRIS_GRADE_HISTORY SET SN_NO = SN_NO - 1 WHERE EMPLOYEE_ID = {$employeeId} AND SN_NO > {$snNo}";
      $this->adapter->query($sql)->execute();

      $this->recalculateGrade($employeeId, $snNo-1);
    }

   }

   public function recalculateGrade($employeeId, $snNo){

    $sql = "CALL HRIS_RECALCULATE_GRADE({$employeeId}, {$snNo})";
    $this->adapter->query($sql)->execute();
   }

}