<?php

namespace Payroll\Repository;

use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;

class BonusRepository extends HrisRepository{

    public function __construct(AdapterInterface $adapter, $tablename = null) {
      parent::__construct($adapter, 'HRIS_BONUS');
    }

    public function validateExist($fiscalYearId){
        $sql = "
            SELECT COUNT(*) AS COUNT FROM HRIS_BONUS WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E';
        ";

        return $this->adapter->query($sql)->execute()->current()['COUNT'] > 0;
    }

    public function generateBonus($fiscalYearId, $bonusAmount){
        
        $asarMonth = $this->adapter->query("SELECT MONTH_ID FROM HRIS_MONTH_CODE WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND MONTH_NO = 3")
                    ->execute()->current()['MONTH_ID'];

        $sql = "
            CALL HRIS_CALCULATE_BONUS(
                {$fiscalYearId},
                {$asarMonth},
                {$bonusAmount}
            )
        ";
        $this->adapter->query($sql)->execute();
    
    }

    public function getEmployeeIds(){

        $sql = "
                SELECT EMPLOYEE_ID FROM HRIS_EMPLOYEES 
                WHERE STATUS = 'E' AND RETIRED_FLAG = 'N' AND RESIGNED_FLAG = 'N'
            ";

        return $this->adapter->query($sql)->execute();

    }

    public function fetchDetail($fiscalYearId=null, $employeeId=null){

        $sql = "
            SELECT D.*, E.FULL_NAME AS EMPLOYEE_NAME, Y.FISCAL_YEAR_NAME
            FROM HRIS_BONUS_DETAIL D
            LEFT JOIN HRIS_EMPLOYEES E ON D.EMPLOYEE_ID = E.EMPLOYEE_ID
            LEFT JOIN HRIS_BONUS S ON D.BONUS_ID = S.BONUS_ID
            LEFT JOIN HRIS_FISCAL_YEARS Y ON Y.FISCAL_YEAR_ID = S.FISCAL_YEAR_ID
            WHERE 1=1
        ";

        if($fiscalYearId){

            $sql = $sql . " AND D.BONUS_ID = (
                SELECT BONUS_ID FROM HRIS_BONUS WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E')";

        }

        if($employeeId){

            $ids = implode(',', $employeeId);
            $sql = $sql . " AND D.EMPLOYEE_ID IN ($ids)";

        }

        $sql = $sql . " ORDER BY E.FULL_NAME, Y.START_DATE";

        return $this->adapter->query($sql)->execute();

    }

    public function wipeOut($fiscalYearId){

        $sql = "
            SELECT BONUS_ID FROM HRIS_BONUS WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E'
        ";

        $bonusId = $this->adapter->query($sql)->execute()->current()['BONUS_ID'];

        $sql = "
            DELETE FROM HRIS_BONUS WHERE BONUS_ID = {$bonusId}; 
        ";

        $this->adapter->query($sql)->execute();

        $sql = "
            DELETE FROM HRIS_BONUS_DETAIL WHERE BONUS_ID = {$bonusId};
        ";

        $this->adapter->query($sql)->execute();

    }

    public function fetchBonusAmount($fiscalYearId){

        return $this->adapter->query("SELECT BONUS_AMOUNT FROM HRIS_BONUS WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E'")
                ->execute()->current()['BONUS_AMOUNT'];

    }

}