<?php

namespace Payroll\Repository;

use Application\Model\Model;
use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;
use Payroll\Model\GradePuraskarModel;
use Zend\Db\TableGateway\TableGateway;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;

class GradeRepository extends HrisRepository{

    private $puraskarGateway;

    public function __construct(AdapterInterface $adapter, $tablename = null) {
      parent::__construct($adapter, 'HRIS_GRADE_HISTORY');
      $this->puraskarGateway = new TableGateway(GradePuraskarModel::TABLE_NAME, $adapter);
    }

    public function fetchById($employeeId){

        $sql = "
            SELECT 
            EFFECT_DATE AS EFFECT_DATE,
            BS_DATE(EFFECT_DATE) AS EFFECT_DATE_BS,
            CASE WHEN EGC_ID IS NOT NULL THEN 'Salary Change'
            WHEN PURASKAR_ID IS NOT NULL THEN 'Grade Award'
            WHEN PROMOTION_ID IS NOT NULL AND IS_APPOINTMENT = 'N'
            THEN 'Promotion'
            WHEN PROMOTION_ID IS NOT NULL AND IS_APPOINTMENT = 'Y'
            THEN 'Appointment' 
            WHEN IS_AUTO_INCREMENT = 'Y' THEN 'Auto Increment'
            WHEN IS_INDIVIDUAL_PURASKAR = 'Y'
            THEN 'Individual Puraskar'
            ELSE 'Opening Grade'
            END
            AS SERVICE_EVENT,
            PREVIOUS_GRADE AS INITIAL_GRADE,
            GRADE_CHANGE AS CHANGE_IN_GRADE,
            NEW_GRADE AS FINAL_GRADE,
            BASE64_DECODE(REMARKS) AS REMARKS,
            CASE WHEN IS_IMPLEMENTED = 'Y' THEN 'Yes' ELSE 'No' END AS IS_IMPLEMENTED,
            CASE WHEN IS_OVERIDE = 'Y' THEN 'Yes' ELSE 'No' END AS IS_OVERIDE
            FROM HRIS_GRADE_HISTORY 
            WHERE EMPLOYEE_ID = {$employeeId} ORDER BY EFFECT_DATE ASC;
        ";

        return $this->adapter->query($sql)->execute();

    }

    public function fetchByMonth($searchData){

        $sql = "
            SELECT MONTH_ID FROM HRIS_MONTH_CODE WHERE CURRENT_DATE BETWEEN FROM_DATE AND TO_DATE
        ";

        $monthId = $this->adapter->query($sql)->execute()->current()['MONTH_ID'];

        $sql = "
            SELECT
            H.GRADE_HISTORY_ID AS GRADE_HISTORY_ID,
            H.EFFECT_DATE AS EFFECT_DATE,
            BS_DATE(H.EFFECT_DATE) AS EFFECT_DATE_BS,
            CASE WHEN H.EGC_ID IS NOT NULL 
            THEN 'Salary Change' 
            WHEN H.PURASKAR_ID IS NOT NULL 
            THEN 'Grade Award' 
            WHEN H.PROMOTION_ID IS NOT NULL AND H.IS_APPOINTMENT = 'N'
            THEN 'Promotion'
            WHEN H.PROMOTION_ID IS NOT NULL AND H.IS_APPOINTMENT = 'Y'
            THEN 'Appointment' 
            WHEN H.IS_AUTO_INCREMENT = 'Y'
            THEN 'Auto Increment' 
            WHEN H.IS_INDIVIDUAL_PURASKAR = 'Y'
            THEN 'Individual Puraskar'
            ELSE 'Opening Grade' 
            END AS SERVICE_EVENT,
            H.PREVIOUS_GRADE AS INITIAL_GRADE,
            H.GRADE_CHANGE AS CHANGE_IN_GRADE,
            H.NEW_GRADE AS FINAL_GRADE,
            BASE64_DECODE(H.REMARKS) AS REMARKS,
            H.IS_IMPLEMENTED AS IS_IMPLEMENTED,
            CASE WHEN IS_IMPLEMENTED = 'Y' THEN 'Yes' ELSE 'No' END AS IS_IMPLEMENTED_TEXT,
            E.FIRST_NAME || ' ' || ifnull(E.MIDDLE_NAME,
        '') || ' ' || E.LAST_NAME AS EMPLOYEE_NAME,
            E.EMPLOYEE_CODE AS EMPLOYEE_CODE 
            FROM HRIS_GRADE_HISTORY H 
            LEFT JOIN HRIS_EMPLOYEES E ON E.EMPLOYEE_ID = H.EMPLOYEE_ID 
            WHERE 1=1
        ";

        if($searchData['fromDate']){
            $fromDate = date('Y-m-d', strtotime($searchData['fromDate']));
            $sql = $sql . " AND H.EFFECT_DATE >= '{$fromDate}'";
        }

        if($searchData['toDate']){
            $toDate = date('Y-m-d', strtotime($searchData['toDate']));
            $sql = $sql . " AND H.EFFECT_DATE <= '{$toDate}'";
        }

        if($searchData['employeeId']){
            $sql = $sql . " AND H.EMPLOYEE_ID = {$searchData['employeeId']}";
        }

        if($searchData['implemented']){
            $value = $searchData['implemented'];
            $sql = $sql . " AND H.IS_IMPLEMENTED = '{$value}'";
        }

        $sql = $sql . " ORDER BY H.EFFECT_DATE ASC";

        return $this->adapter->query($sql)->execute();

    }

    public function updateGradeInfo($postData){

        $isOverride = $postData['isOverride'];

        $sql = "UPDATE HRIS_GRADE_HISTORY
                SET NEW_GRADE = {$postData['grade']}, GRADE_CHANGE = {$postData['gradeChange']},
                IS_OVERIDE = '{$isOverride}', IS_IMPLEMENTED = 'Y'
                WHERE GRADE_HISTORY_ID = {$postData['id']}";

        $this->adapter->query($sql)->execute();

        $sql = "CALL HRIS_RECALCULATE_GRADE(
            (SELECT EMPLOYEE_ID FROM HRIS_GRADE_HISTORY WHERE GRADE_HISTORY_ID = {$postData['id']})
            , 1)";

        $this->adapter->query($sql)->execute();
    }

    public function getPuraskarDate(){

        $sql = "
            SELECT 
            EFFECT_DATE,
            BS_DATE(EFFECT_DATE) AS EFFECT_DATE_BS,
            BASE64_DECODE(REMARKS) AS REMARKS,
            PURASKAR_POINT,
            MONTH_CONDITION,
            CREATED_DATE
            FROM HRIS_GRADE_PURASKAR ORDER BY EFFECT_DATE DESC;
        ";

        return $this->adapter->query($sql)->execute();
    }

    public function bulkGradeAssign($month, $gradePoint, $remark, $effectDate, $puraskarId){

        $sql = "CALL HRIS_BULK_GRADE_ASSIGN({$month}, {$gradePoint}, '{$remark}', '{$effectDate}', {$puraskarId})";

        $statement = $this->adapter->query($sql);

        $statement->execute();
    }

    public function createPuraskar(Model $model){
        return $this->puraskarGateway->insert($model->getArrayCopyForDB());
    }

    public function pullEmployeeData($data){

        $companyId = isset($data['companyId']) ? $data['companyId'] : -1;
        $branchId = isset($data['branchId']) ? $data['branchId'] : -1;
        $departmentId = isset($data['departmentId']) ? $data['departmentId'] : -1;
        $designationId = isset($data['designationId']) ? $data['designationId'] : -1;
        $positionId = isset($data['positionId']) ? $data['positionId'] : -1;
        $serviceTypeId = isset($data['serviceTypeId']) ? $data['serviceTypeId'] : -1;
        $serviceEventTypeId = isset($data['serviceEventTypeId']) ? $data['serviceEventTypeId'] : -1;
        $genderId = isset($data['genderId']) ? $data['genderId'] : -1;
        $functionalTypeId = isset($data['functionalTypeId']) ? $data['functionalTypeId'] : -1;
        $employeeId = isset($data['employeeId']) ? $data['employeeId'] : -1;
        $effectDate = date('Y-m-d', strtotime($data['effectDate']));

        $boundedParameter = [];
        $searchCondition = $this->getSearchConditonBoundedPayroll($companyId, $branchId, $departmentId, $positionId, $designationId, $serviceTypeId, $serviceEventTypeId, $employeeId, $genderId, null, $functionalTypeId);
        $searchCondition = $this->getSearchConditonBoundedPayroll($companyId, $branchId, $departmentId, $positionId, $designationId, $serviceTypeId, $serviceEventTypeId, null, $employeeId, $genderId, null, $functionalTypeId);
        $boundedParameter = array_merge($boundedParameter, $searchCondition['parameter']);

        $sql = "
                    SELECT 
                    EMPLOYEE_ID AS EMPLOYEE_ID,
                    EMPLOYEE_CODE AS EMPLOYEE_CODE,
                    FULL_NAME AS EMPLOYEE_NAME,
                    HRIS_GET_LATEST_GRADE(EMPLOYEE_ID, '{$effectDate}', 'GRADE') AS CURRENT_GRADE
                    FROM HRIS_EMPLOYEES SSED
                    WHERE EMPLOYEE_TYPE = 'R'
                    {$searchCondition['sql']} 
            ";

        $result = EntityHelper::rawQueryResult($this->adapter, $sql, $boundedParameter);
        return Helper::extractDbData($result);

    }

    public function individualGradeInsertion($data){

        $employeeId = $data['id'];
        $effectDate = date('Y-m-d', strtotime($data['effectDate']));
        $gradeChange = $data['gradeChange'];
        $remarks = $data['remarks'];

        $sql = "
            CALL HRIS_GRADE_INSERT({$employeeId}, '{$effectDate}', $gradeChange, '{$remarks}', 'N', NULL, NULL, 'N', 'Y')
        ";

        $this->adapter->query($sql)->execute();

    }

    public function getFiscalYear(){
        
        $sql = "
            SELECT FISCAL_YEAR_ID, FISCAL_YEAR_NAME FROM HRIS_FISCAL_YEARS WHERE STATUS = 'E' ORDER BY START_DATE
        ";

        return $this->adapter->query($sql)->execute();
    }

    public function getMonths(){
        
        $sql = "
            select month_edesc, fiscal_year_month_no from hris_month_code group by month_edesc, fiscal_year_month_no order by fiscal_year_month_no
        ";

        return $this->adapter->query($sql)->execute();
    }

    public function callGradeMonthly($fiscalYearId, $monthNo){

        $sql = "
            SELECT MONTH_ID FROM HRIS_MONTH_CODE WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND FISCAL_YEAR_MONTH_NO = {$monthNo}
        ";

        $monthId = $this->adapter->query($sql)->execute()->current()['MONTH_ID'];

        $sql = "
            CALL HRIS_GRADE_INCREMENT_BASED_ON_DATE($monthId)
        ";

        $this->adapter->query($sql)->execute();
    }


}