<?php

namespace Payroll\Repository;

use Application\Model\Model;
use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;
use Payroll\Model\PayrollSuspendModel;

class PayrollSuspendRepository extends HrisRepository{

    public function __construct(AdapterInterface $adapter, $tablename = null) {
      parent::__construct($adapter, PayrollSuspendModel::TABLE_NAME);
    }

    public function add($data){
        $array = $data->getArrayCopyForDb();
        $this->tableGateway->insert($array);
        $this->linkWithFiles();
    }

    public function monthData(){

        $sql = "
            select 
            m.month_id,
            m.year || ' ' || m.month_edesc as month_text
            from hris_month_code m
            where m.from_date > add_days(current_date, -40) order by month_id;
        ";

        return $this->adapter->query($sql)->execute();

    }

    public function getSuspendData($searchData){

        $sql = "
            SELECT 
            MODIFICATION_ID AS MODIFICATION_ID,
            E.FIRST_NAME || ' ' || E.LAST_NAME AS EMPLOYEE_NAME,
            FM.YEAR || ' ' || FM.MONTH_EDESC AS FROM_MONTH,
            TM.YEAR || ' ' || TM.MONTH_EDESC AS TO_MONTH,
            CASE WHEN VALUE_TYPE = 'S' THEN 'Salary'
            ELSE 'Grade' END AS VALUE_TYPE,
            VALUE AS VALUE,
            CASE WHEN IS_HALTED = 'Y' THEN 'Yes'
            ELSE 'No' END AS IS_HALTED,
            CREATED_DATE AS CREATED_DATE
            FROM HRIS_GRADE_SALARY_MODIFICATION M
            LEFT JOIN HRIS_EMPLOYEES E ON M.EMPLOYEE_ID = E.EMPLOYEE_ID
            LEFT JOIN HRIS_MONTH_CODE FM ON M.FROM_MONTH_ID = FM.MONTH_ID
            LEFT JOIN HRIS_MONTH_CODE TM ON M.TO_MONTH_ID = TM.MONTH_ID
            WHERE M.STATUS = 'E' 
        ";

        if($searchData['employeeId']){
            $sql = $sql . " AND M.EMPLOYEE_ID = {$searchData['employeeId']}";
        }

        if($searchData['valueType']){
            $data = $searchData['valueType'];
            $sql = $sql . " AND M.VALUE_TYPE = '{$data}'";
        }

        if($searchData['halted']){
            $data = $searchData['halted'];
            $sql = $sql . " AND M.IS_HALTED = '{$data}'";
        }

        $sql = $sql . " ORDER BY CREATED_DATE DESC";

        return $this->adapter->query($sql)->execute();

    }

    public function delete($id){

        $sql = "
          UPDATE HRIS_GRADE_SALARY_MODIFICATION SET STATUS = 'D' WHERE MODIFICATION_ID = {$id}
        ";
    
        $this->adapter->query($sql)->execute();
    
    }

    public function pushFileLink($data){
        $fileName = $data['file'];
        $id = $data['id'] == null ? 0 : $data['id'] ;
        $oldfile = $data['fileName'];
        $sql = "INSERT INTO HRIS_GRADE_MODIFICATION_DOCUMENT(FILE_ID, FILE_NAME, FILE_IN_DIR_NAME, UPLOADED_DATE, MODIFICATION_ID)
        VALUES( (select ifnull(max(FILE_ID)+1, 1) from HRIS_GRADE_MODIFICATION_DOCUMENT), '{$fileName}', '{$oldfile}', current_date, {$id})";
        return $this->rawQuery($sql);
    }

    public function pullFilesbyModificationId($id){
        $sql = "select * from HRIS_GRADE_MODIFICATION_DOCUMENT where MODIFICATION_ID = $id and status = 'E'";
        return $this->rawQuery($sql);
    }

    public function deleteFileByName($name){
        $sql = "update HRIS_GRADE_MODIFICATION_DOCUMENT set status = 'D' where FILE_NAME = '{$name}'";
        return $this->rawQuery($sql);
    }

    public function linkWithFiles(){
        if(!empty($_POST['fileUploadList'])){
            $filesList = $_POST['fileUploadList'];
            $filesList = "'".implode("','" , $filesList)."'";
            $sql = "UPDATE HRIS_GRADE_MODIFICATION_DOCUMENT SET MODIFICATION_ID = (SELECT MAX(MODIFICATION_ID) FROM HRIS_GRADE_SALARY_MODIFICATION) 
                    WHERE FILE_NAME IN($filesList)";

            $statement = $this->adapter->query($sql);
            $statement->execute();
        }
    }

    public function fetchById($id){

        $sql = "
            SELECT 
            MODIFICATION_ID AS MODIFICATION_ID,
            E.FIRST_NAME || ' ' || E.LAST_NAME AS EMPLOYEE_NAME,
            FM.YEAR || ' ' || FM.MONTH_EDESC AS FROM_MONTH,
            TM.YEAR || ' ' || TM.MONTH_EDESC AS TO_MONTH,
            CASE WHEN VALUE_TYPE = 'S' THEN 'Salary'
            ELSE 'Grade' END AS VALUE_TYPE,
            VALUE AS VALUE,
            CASE WHEN IS_HALTED = 'Y' THEN 'Yes'
            ELSE 'No' END AS IS_HALTED,
            CREATED_DATE AS CREATED_DATE
            FROM HRIS_GRADE_SALARY_MODIFICATION M
            LEFT JOIN HRIS_EMPLOYEES E ON M.EMPLOYEE_ID = E.EMPLOYEE_ID
            LEFT JOIN HRIS_MONTH_CODE FM ON M.FROM_MONTH_ID = FM.MONTH_ID
            LEFT JOIN HRIS_MONTH_CODE TM ON M.TO_MONTH_ID = TM.MONTH_ID
            WHERE M.STATUS = 'E' AND MODIFICATION_ID = {$id}
        ";

        return $this->adapter->query($sql)->execute()->current();

    }

    public function fetchDocument($id){

        $sql = "
            SELECT FILE_NAME FROM HRIS_GRADE_MODIFICATION_DOCUMENT WHERE MODIFICATION_ID = {$id} AND STATUS = 'E'
        ";

        return $this->adapter->query($sql)->execute();

    }

}