<?php

namespace Payroll\Repository;

use Zend\Db\Adapter\AdapterInterface;
use Application\Repository\HrisRepository;

class ProvisionRepository extends HrisRepository{

    public function __construct(AdapterInterface $adapter, $tablename = null) {
      parent::__construct($adapter, 'HRIS_PROVISION_SHEET');
    }

    public function validateExist($fiscalYearId){
        $sql = "
            SELECT COUNT(*) AS COUNT FROM HRIS_PROVISION_SHEET WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E';
        ";

        return $this->adapter->query($sql)->execute()->current()['COUNT'] > 0;
    }

    public function generateProvision($fiscalYearId){
        
        $sql = "
            CALL HRIS_GENERATE_PROVISION({$fiscalYearId})
        ";
        $this->adapter->query($sql)->execute();

        $sql = "
                SELECT COUNT(*) AS COUNT FROM HRIS_PROVISION_DETAIL_ERROR_LOG
                WHERE PROVISION_ID = (SELECT MAX(PROVISION_ID) FROM HRIS_PROVISION_SHEET) GROUP BY ERROR_TYPE;
            ";

        return $this->adapter->query($sql)->execute();
    
    }

    public function getEmployeeIds(){

        $sql = "
                SELECT EMPLOYEE_ID FROM HRIS_EMPLOYEES 
                WHERE STATUS = 'E' AND RETIRED_FLAG = 'N' AND RESIGNED_FLAG = 'N'
            ";

        return $this->adapter->query($sql)->execute();

    }

    public function fetchDetail($fiscalYearId=null, $employeeId=null){

        $sql = "
            SELECT D.*, E.FULL_NAME AS EMPLOYEE_NAME, Y.FISCAL_YEAR_NAME
            FROM HRIS_PROVISION_DETAIL D
            LEFT JOIN HRIS_EMPLOYEES E ON D.EMPLOYEE_ID = E.EMPLOYEE_ID
            LEFT JOIN HRIS_PROVISION_SHEET S ON D.PROVISION_ID = S.PROVISION_ID
            LEFT JOIN HRIS_FISCAL_YEARS Y ON Y.FISCAL_YEAR_ID = S.FISCAL_YEAR_ID
            WHERE 1=1
        ";

        if($fiscalYearId){

            $sql = $sql . " AND D.PROVISION_ID = (
                SELECT PROVISION_ID FROM HRIS_PROVISION_SHEET WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E')";

        }

        if($employeeId){

            $ids = implode(',', $employeeId);
            $sql = $sql . " AND D.EMPLOYEE_ID IN ($ids)";

        }

        $sql = $sql . " ORDER BY E.FULL_NAME, Y.START_DATE";

        return $this->adapter->query($sql)->execute();

    }

    public function fetchLog($fiscalYearId){
        $sql = "
            SELECT 
            E.FULL_NAME AS EMPLOYEE_NAME,
            CASE WHEN L.ERROR_TYPE = 'S' THEN 'Error: Salary not found, please ensure proper assign of functional level and position. And salary has been updated'
            ELSE 'Warning: Grade is zero, make sure it is not zero and calculation is valid' END AS LOG_MESSAGE,
            ERROR_TYPE
            FROM HRIS_PROVISION_DETAIL_ERROR_LOG L
            LEFT JOIN HRIS_EMPLOYEES E ON L.EMPLOYEE_ID = E.EMPLOYEE_ID
            WHERE PROVISION_ID = (
            SELECT PROVISION_ID FROM HRIS_PROVISION_SHEET WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E') ORDER BY E.FULL_NAME
        ";

        return $this->adapter->query($sql)->execute();
    }

    public function wipeOut($fiscalYearId){

        $sql = "
            SELECT PROVISION_ID FROM HRIS_PROVISION_SHEET WHERE FISCAL_YEAR_ID = {$fiscalYearId} AND STATUS = 'E'
        ";

        $provisionId = $this->adapter->query($sql)->execute()->current()['PROVISION_ID'];

        $sql = "
            DELETE FROM HRIS_PROVISION_SHEET WHERE PROVISION_ID = {$provisionId}; 
        ";

        $this->adapter->query($sql)->execute();

        $sql = "
            DELETE FROM HRIS_PROVISION_DETAIL WHERE PROVISION_ID = {$provisionId};
        ";

        $this->adapter->query($sql)->execute();

        $sql = "
            DELETE FROM HRIS_PROVISION_DETAIL_ERROR_LOG WHERE PROVISION_ID = {$provisionId};
        ";

        $this->adapter->query($sql)->execute();
    }

}