<?php
namespace Payroll\Repository;

use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Application\Model\Model;
use Application\Repository\HrisRepository;
use Payroll\Model\TaxSheet;
use Zend\Db\Adapter\AdapterInterface;

class TaxSheetRepo extends HrisRepository {

    public function __construct(AdapterInterface $adapter, $tableName = null) {
        if ($tableName == null) {
            $tableName = TaxSheet::TABLE_NAME;
        }
        parent::__construct($adapter, $tableName);
    }

    public function add(Model $model) {
        return $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function fetchById($id) {
        return $this->tableGateway->select($id);
    }

    private function fetchPayIdsAsArray() {
        $rawList = $this->rawQuery("SELECT PAY_ID FROM HRIS_PAY_SETUP WHERE STATUS ='E'");
        $dbArray = "";
        foreach ($rawList as $key => $row) {
            if ($key == sizeof($rawList) - 1) {
                $dbArray .= "{$row['PAY_ID']} AS P_{$row['PAY_ID']}";
            } else {
                $dbArray .= "{$row['PAY_ID']} AS P_{$row['PAY_ID']},";
            }
        }
        return $dbArray;
    }

    //FOR TAXSLIP IN HR
    public function fetchTaxSheetPivoted($companyId, $branchId, $departmentId, $designationId, $positionId, $serviceTypeId, $serviceEventTypeId, $employeeTypeId, $genderId, $employeeId, $locationId, $fiscalYearId,$monthId,$salaryType) {

        $boundedParams = [];
        $searchConditon = EntityHelper::getSearchConditonBounded($companyId, $branchId, $departmentId, $positionId, $designationId, $serviceTypeId, $serviceEventTypeId, $employeeTypeId, $employeeId, $genderId,$locationId);
        $boundedParams = array_merge($boundedParams, $searchConditon['parameter']);

        if($monthId== null || $monthId==''){
            $temp="select string_agg(month_id,',') as month_id from hris_month_code where fiscal_year_id=$fiscalYearId and month_id < (select month_id from hris_month_code where current_date between from_date and to_date);";
            $tempre=$this->rawQuery($temp);
            $monthCondn=$tempre[0]['MONTH_ID'];

        }
        else{
            $monthCondn=$monthId;
        }



        //manually inserted pay_id of tax and it's related fields
        $taxPayIds=[6,38,40,42,43,44,45,46,47,54,69,87,88,89,90,94,95,96,97,98,107,115,149];
        for ($i=0; $i < count($taxPayIds) ; $i++) { 
            $test[$i] = 'SUM(CASE WHEN SSD.PAY_ID ='.$taxPayIds[$i].' THEN SSD.VAL END) AS P_'. $taxPayIds[$i];
        }

        $select = implode(',', $test);

        // print_r($select); die;

        $sql="select e.full_name,hw.*,hmc.month_edesc,hmc.year,
        CASE
        WHEN L.ORDER_ID IS NULL THEN 99
        ELSE L.ORDER_ID
    END AS ORDER_LEVEL from
        (SELECT
            SS.MONTH_ID,
            SS.SHEET_NO,
            E.employee_id,
            --SSD.PAY_ID,
            --UPPER(REPLACE(REPLACE(P.PAY_EDESC,'%','PERCENT'), ' ', '' )),
            --SSD.VAL,
            $select
        
        FROM HRIS_SALARY_SHEET SS
        LEFT JOIN HRIS_SALARY_SHEET_DETAIL SSD ON (SS.SHEET_NO = SSD.SHEET_NO)
        LEFT JOIN HRIS_EMPLOYEES E ON (E.EMPLOYEE_ID = SSD.EMPLOYEE_ID)
        LEFT JOIN HRIS_PAY_SETUP P ON P.PAY_ID = SSD.PAY_ID
        
        WHERE SS.MONTH_ID IN ({$monthCondn})
            AND SS.SALARY_TYPE_ID = {$salaryType}
            AND SSD.PAY_ID IN (6,38,40,42,43,44,45,46,47,54,69,87,88,89,90,94,95,96,97,98,107,115,149) 
            --AND E.EMPLOYEE_ID = 170
        GROUP BY SS.SHEET_NO, SS.MONTH_ID, E.employee_id
        ORDER BY SS.SHEET_NO, SS.MONTH_ID ASC ) hw
        left join hris_employees e on hw.employee_id=e.employee_id
        left join hris_month_code hmc on hmc.month_id=hw.month_id
        LEFT JOIN HRIS_FUNCTIONAL_LEVELS L ON L.FUNCTIONAL_LEVEL_ID = E.FUNCTIONAL_LEVEL_ID
              LEFT JOIN HRIS_FUNCTIONAL_LEVELS LL ON LL.FUNCTIONAL_LEVEL_ID = E.ACTING_FUNCTIONAL_LEVEL_ID
              LEFT JOIN HRIS_JOB_HISTORY HEIRJH ON (HEIRJH.EMPLOYEE_ID = E.EMPLOYEE_ID AND HEIRJH.START_DATE =
              (SELECT MAX(START_DATE) FROM HRIS_JOB_HISTORY WHERE EMPLOYEE_ID = E.EMPLOYEE_ID AND SERVICE_EVENT_TYPE_ID IN (1, 2)))
        where 1=1 {$searchConditon['sql']}
        ORDER BY
                  ORDER_LEVEL, LL.ORDER_ID,HEIRJH.START_DATE,hmc.month_id,e.employee_id
        ";

        // print_r($sql);die;

        $statement = $this->adapter->query($sql);
        $result = $statement->execute($boundedParams);
        return Helper::extractDbData($result);
    }

    // FOR TAXSLIP IN SELFSERVICE
    public function fetchTaxSheetPivotedEmp( $employeeId, $fiscalYearId,$monthId,$salaryType) {

        $boundedParams = [];

        if($monthId== null || $monthId==''){
            $temp="select string_agg(month_id,',') as month_id from hris_month_code where fiscal_year_id=$fiscalYearId and month_id < (select month_id from hris_month_code where current_date between from_date and to_date);";
            $tempre=$this->rawQuery($temp);
            $monthCondn=$tempre[0]['MONTH_ID'];

        }
        else{
            $monthCondn=$monthId;
        }



        //manually inserted pay_id of tax and it's related fields
        $taxPayIds=[6,38,40,42,43,44,45,46,47,54,69,87,88,89,90,94,95,96,97,98,107,115,149];
        for ($i=0; $i < count($taxPayIds) ; $i++) { 
            $test[$i] = 'SUM(CASE WHEN SSD.PAY_ID ='.$taxPayIds[$i].' THEN SSD.VAL END) AS P_'. $taxPayIds[$i];
        }

        $select = implode(',', $test);

        // print_r($select); die;

        $sql="select e.full_name,hw.*,hmc.month_edesc,hmc.year from
        (SELECT
            SS.MONTH_ID,
            SS.SHEET_NO,
            E.employee_id,
            --SSD.PAY_ID,
            --UPPER(REPLACE(REPLACE(P.PAY_EDESC,'%','PERCENT'), ' ', '' )),
            --SSD.VAL,
            $select
        
        FROM HRIS_SALARY_SHEET SS
        LEFT JOIN HRIS_SALARY_SHEET_DETAIL SSD ON (SS.SHEET_NO = SSD.SHEET_NO)
        LEFT JOIN HRIS_EMPLOYEES E ON (E.EMPLOYEE_ID = SSD.EMPLOYEE_ID)
        LEFT JOIN HRIS_PAY_SETUP P ON P.PAY_ID = SSD.PAY_ID
        
        WHERE SS.MONTH_ID IN ({$monthCondn})
            AND SS.SALARY_TYPE_ID = {$salaryType}
            AND SSD.PAY_ID IN (6,38,40,42,43,44,45,46,47,54,69,87,88,89,90,94,95,96,97,98,107,115,149) 
        GROUP BY SS.SHEET_NO, SS.MONTH_ID, E.employee_id
        ORDER BY SS.SHEET_NO, SS.MONTH_ID ASC ) hw
        left join hris_employees e on hw.employee_id=e.employee_id
        left join hris_month_code hmc on hmc.month_id=hw.month_id
        where 1=1 and e.employee_id={$employeeId}
        ORDER BY
                  hmc.month_id
        ";

        // print_r($sql);die;

        $statement = $this->adapter->query($sql);
        $result = $statement->execute($boundedParams);
        return Helper::extractDbData($result);
    }

    public function fetchEmployeeTaxSlip($monthId, $employeeId) {

        $boundedParameter = [];
        $boundedParameter['monthId'] = $monthId;
        $boundedParameter['employeeId'] = $employeeId;

        $sql = "SELECT TS.*,
                  P.PAY_TYPE_FLAG,
                  P.PAY_EDESC
                FROM HRIS_TAX_SHEET TS
                LEFT JOIN HRIS_PAY_SETUP P
                ON (TS.PAY_ID         =P.PAY_ID)
                WHERE P.INCLUDE_IN_TAX='Y' AND TS.VAL>0
                AND TS.SHEET_NO       IN
                  (SELECT SHEET_NO FROM HRIS_SALARY_SHEET WHERE MONTH_ID =?
                  )
                AND EMPLOYEE_ID =? ORDER BY P.PRIORITY_INDEX";
        
        return $this->rawQuery($sql, $boundedParameter);
    }

    public function delete($id) {
        return $this->tableGateway->delete([TaxSheet::SHEET_NO => $id]);
    }

    public function deleteBy($by) {
        return $this->tableGateway->delete($by);
    }
}
