<?php

namespace Punishment\Repository;

use Application\Model\Model;
use Application\Repository\HrisRepository;
use Zend\Db\Adapter\AdapterInterface;
use Punishment\Model\PunishmentModel;

class PunishmentRepository extends HrisRepository {


    public function __construct(AdapterInterface $adapter, $tablename = null) {
        parent::__construct($adapter, "HRIS_EMPLOYEE_PUNISHMENT");
      }
  
      public function add($data){
          $array = $data->getArrayCopyForDb();
          $this->tableGateway->insert($array);
      }

      public function fetchAll(){

        $sql = "
            SELECT 
            P.PUNISHMENT_ID AS PUNISHMENT_ID,
            E.FULL_NAME AS EMPLOYEE_NAME,
            HRIS_PUNISHMENT_TYPE(P.PUNISHMENT_TYPE) AS PUNISHMENT_TYPE,
            BS_DATE(P.START_DATE) AS START_DATE,
            BS_DATE(P.END_DATE) AS END_DATE,
            P.REASON AS REASON,
            CASE WHEN P.STATUS = 'E' THEN 'Enabled' ELSE 'Revoked' END AS STATUS,
            A.FULL_NAME AS ANNOUNCED_BY,
            P.CREATED_DATE AS CREATED_DATE
            FROM HRIS_EMPLOYEE_PUNISHMENT P
            LEFT JOIN HRIS_EMPLOYEES E ON P.EMPLOYEE_ID = E.EMPLOYEE_ID
            LEFT JOIN HRIS_EMPLOYEES A ON P.ANNOUNCED_BY = A.EMPLOYEE_ID
            WHERE P.STATUS IN ('E', 'R')
            ORDER BY P.CREATED_DATE DESC
        ";

        return $this->adapter->query($sql)->execute();

      }

      public function addJobHistory($employeeId, $startDate, $serviceEventId, $endDate){

        $endDateQuery = $endDate == null ? "NULL" : "'{$endDate}'";

        $sql = "
            CALL HRIS_INSERT_JOB_HISTORY({$employeeId}, '{$startDate}', {$serviceEventId}, NULL, NULL, 'N', 'N', {$endDateQuery})
        ";

        $this->adapter->query($sql)->execute();
      }

    public function fetchById($id){
        return $this->tableGateway->select([PunishmentModel::PUNISHMENT_ID=>$id])->current();
    }

    public function edit(Model $model, $id) {
        $array = $model->getArrayCopyForDB();
        $this->tableGateway->update($array, [PunishmentModel::PUNISHMENT_ID => $id]);
    }

    public function editJobHistory($jobHistoryId, $serviceEventId, $startDate, $endDate, $employeeId){

        $sql = "UPDATE HRIS_JOB_HISTORY 
                SET SERVICE_EVENT_TYPE_ID = {$serviceEventId}, START_DATE = '{$startDate}', END_DATE = '{$endDate}', EMPLOYEE_ID = {$employeeId} 
                WHERE JOB_HISTORY_ID = {$jobHistoryId}";

        $this->adapter->query($sql)->execute();

    }

    public function delete($id){

        $sql = "UPDATE HRIS_EMPLOYEE_PUNISHMENT SET STATUS = 'D' WHERE PUNISHMENT_ID = {$id}";

        $this->adapter->query($sql)->execute();

        $sql = "UPDATE HRIS_JOB_HISTORY SET STATUS = 'D' WHERE JOB_HISTORY_ID = (SELECT JOB_HISTORY_ID FROM HRIS_EMPLOYEE_PUNISHMENT WHERE PUNISHMENT_ID = {$id})";

        $this->adapter->query($sql)->execute();

        $sql = "UPDATE HRIS_JOB_HISTORY SET STATUS = 'D' WHERE JOB_HISTORY_ID = (SELECT REVOKED_JOB_HISTORY_ID FROM HRIS_EMPLOYEE_PUNISHMENT WHERE PUNISHMENT_ID = {$id})";

        $this->adapter->query($sql)->execute();
    }

    public function revoke($punishmentId, $revokedBy, $revokedRemarks, $revokeHistoryId){

        $sql = "UPDATE HRIS_EMPLOYEE_PUNISHMENT 
                SET REVOKED_BY = $revokedBy, REVOKED_REMARKS = '{$revokedRemarks}', REVOKED_DATE = CURRENT_DATE, 
                STATUS = 'R', REVOKED_JOB_HISTORY_ID = {$revokeHistoryId}
                WHERE PUNISHMENT_ID = {$punishmentId}
            ";

        $this->adapter->query($sql)->execute();

    }

    public function impactPayroll($employeeId, $value){

        $sql = "
            UPDATE HRIS_EMPLOYEES SET INCLUDE_IN_PAYROLL = '{$value}' WHERE EMPLOYEE_ID = {$employeeId}
        ";

        $this->adapter->query($sql)->execute();
    }
}