<?php
namespace Recruitment\Repository;

use Application\Helper\EntityHelper;
use Application\Repository\HrisRepository;
use Zend\Db\Adapter\AdapterInterface;
use Application\Model\Model;
use Recruitment\Model\RecruitmentVacancy;
use Symfony\Component\VarDumper\VarDumper;
use Zend\Db\Sql\Sql;
use Zend\Db\Sql\Expression;

class VacancyRepository extends HrisRepository{
    public function __construct(AdapterInterface $adapter, $tableName = null) {
        parent::__construct($adapter, RecruitmentVacancy::TABLE_NAME);
    }

    
    public function add(Model $model) {
        $array = $model->getArrayCopyForDB();
        if (($array['QUOTA_INTERNAL'] == null)) {
            $array['QUOTA_INTERNAL'] = 0;
        };
        if (($array['QUOTA_OPEN'] == null)) {
            $array['QUOTA_OPEN'] = 0;
        };
        // echo '<pre>'; print_r($array); die();
        $this->tableGateway->insert($array);
        
    }
    
    public function getFilteredRecords($search) {        
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->columns([            
            new Expression("REC.VACANCY_ID AS VACANCY_ID"),
            new Expression("TO_CHAR(REC.VACANCY_EDESC) AS VACANCY_EDESC"),
            new Expression("TO_CHAR(REC.VACANCY_NDESC) AS VACANCY_NDESC"),
            new Expression("REC.OPENING_ID AS OPENING_ID"),
            new Expression("REC.AD_NO AS AD_NO"),
            // new Expression("REC.AGE AS AGE"),
            new Expression("REC.PAYMENT_FLAG AS PAYMENT_FLAG"),
            new Expression("REC.ETHNICITY_FLAG AS ETHNICITY_FLAG"),
            new Expression("TO_CHAR(REC.JOB_SPECIFICATION) AS JOB_SPECIFICATION"),
            new Expression("HD.DEPARTMENT_NAME AS DEPARTMENT_ID"),
            new Expression("REC.POSITION_ID AS POSITION_ID"),
            new Expression("REC.REMARKS AS REMARKS"),
            // new Expression("HG.GENDER_NAME AS GENDER"),
            new Expression("HQ.ACADEMIC_DEGREE_NAME AS QUALIFICATION_ID"),
            new Expression("REC.QUOTA_OPEN AS QUOTA_OPEN"),
            new Expression("REC.QUOTA_INTERNAL AS QUOTA_INTERNAL"),
            new Expression("(CASE WHEN REC.STATUS= 'E' THEN 'ENABLE' ELSE 'DISABLE' END) AS STATUS"),
            // new Expression("REC.STATUS AS STATUS"),
            ], true);

        $select->from(['REC' => RecruitmentVacancy::TABLE_NAME])
                // ->join(['HG' => 'HRIS_GENDERS'],'HG.GENDER_ID=REC.GENDER', 'GENDER_ID', 'left')
                ->join(['HQ' => 'HRIS_ACADEMIC_DEGREES'],'HQ.ACADEMIC_DEGREE_ID=REC.QUALIFICATION_ID', 'ACADEMIC_DEGREE_ID', 'left')
                ->join(['HD' => 'HRIS_DEPARTMENTS'],'HD.DEPARTMENT_ID=REC.DEPARTMENT_ID', 'DEPARTMENT_CODE', 'left')
                ->join(['HOP' => 'HRIS_REC_OPENINGS'], 'HOP.OPENING_ID=REC.OPENING_ID', 'OPENING_NO', 'left')
                // $select->where(["REC.VACANCY_ID" => $id]);
                
                ->where(["REC.STATUS='E' AND HOP.STATUS='E'"]);

        
        // if (($search['Gender'] != null)) {
        //     $select->where([
        //         "REC.GENDER" => $search['Gender']
        //     ]);
        // }

        if (($search['QualificationId'] != null)) {
            $select->where([
                "REC.QUALIFICATION_ID" => $search['QualificationId']
            ]);
        }
        if (($search['AdNo'] != null)) {
            $select->where([
                "REC.AD_NO" => $search['AdNo']
            ]);
        }
        if (($search['DepartmentId'] != null)) {
            $select->where([
                "REC.DEPARTMENT_ID" => $search['DepartmentId']
            ]);
        }

        $select->order("REC.VACANCY_ID ASC");
        $boundedParameter = [];
        
        $statement = $sql->prepareStatementForSqlObject($select);
        // print_r ($statement->getSql()); die();
        $result = $statement->execute($boundedParameter);

        return $result;
    }

    public function fetchById($id) {
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->columns([
            new Expression("REC.VACANCY_ID AS VACANCY_ID"),
            new Expression("TO_CHAR(REC.VACANCY_EDESC) AS VACANCY_EDESC"),  
            new Expression("TO_CHAR(REC.VACANCY_NDESC) AS VACANCY_NDESC"),  
            new Expression("REC.OPENING_ID AS OPENING_ID"),
            new Expression("REC.LEVEL_ID AS LEVEL_ID"),
            new Expression("REC.SERVICE_TYPES_ID AS SERVICE_TYPES_ID"),
            new Expression("REC.SERVICE_EVENTS_ID AS SERVICE_EVENTS_ID"),
            // new Expression("VI.INCLUSION_ID AS INCLUSION_ID"),
            new Expression("REC.PAYMENT_FLAG AS PAYMENT_FLAG"),
            new Expression("REC.ETHNICITY_FLAG AS ETHNICITY_FLAG"),
            new Expression("TO_CHAR(REC.JOB_SPECIFICATION) AS JOB_SPECIFICATION"),  
            new Expression("REC.QUALIFICATION_ID AS QUALIFICATION_ID"),
            new Expression("TO_CHAR(REC.RESPONSIBILITY) AS RESPONSIBILITY"),
            new Expression("REC.EXPERIENCE AS EXPERIENCE"),
            new Expression("TO_CHAR(REC.ROLES) AS ROLES"),
            new Expression("REC.QUOTA_OPEN AS QUOTA_OPEN"),
            new Expression("REC.QUOTA_INTERNAL AS QUOTA_INTERNAL"),
            new Expression("REC.AD_NO AS AD_NO"),
            new Expression("REC.POSITION_ID AS POSITION_ID"),
            new Expression("REC.DEPARTMENT_ID AS DEPARTMENT_ID"),
            new Expression("(CASE WHEN REC.STATUS = 'E' THEN 'ENABLE' ELSE 'DISABLE' END) AS STATUS"),
            new Expression("REC.REMARKS AS REMARKS"),
            new Expression("REC.CREATED_BY AS CREATED_BY"),
            new Expression("REC.CREATED_DT AS CREATED_DT"),
            new Expression("REC.MODIFIED_BY AS MODIFIED_BY"),
            new Expression("REC.MODIFIED_DT AS MODIFIED_DT"),
            new Expression("REC.DELETED_BY AS DELETED_BY"),
            new Expression("REC.DELETED_DT AS DELETED_DT"),
            ], true);

        $select->from(['REC' => RecruitmentVacancy::TABLE_NAME]);
                // ->join(['VI' => 'HRIS_REC_VACANCY_INCLUSION'], 'VI.VACANCY_ID=REC.VACANCY_ID', 'VACANCY_INCLUSION_ID', 'left');
                // ->join(['HVF' => 'HRIS_REC_VACANCY_FILES'], 'HVF.VACANCY_ID=REC.VACANCY_ID', 'FILE_ID', 'left');

        $select->where(["REC.VACANCY_ID='{$id}'"]); //change to this if not working. remove below 2 line.
        // $select->where(["REC.VACANCY_ID" => $id]);
        // $select->order("REC.VACANCY_EDESC DESC");
        $boundedParameter = [];
      
        $statement = $sql->prepareStatementForSqlObject($select);
        
        $result = $statement->execute($boundedParameter);
        // print_r ($statement->getSql()); die();
        return $result->current();
    }
    
    public function edit(Model $model, $id) {
        $array = $model->getArrayCopyForDB();
        if (($array['QUOTA_INTERNAL'] == null)) {
            $array['QUOTA_INTERNAL'] = 0;
        };
        if (($array['QUOTA_OPEN'] == null)) {
            $array['QUOTA_OPEN'] = 0;
        };
        // echo'<pre>';
        // var_dump($array); die;
        // echo'<pre>'; print_r($array); die;

        $this->tableGateway->update($array, [RecruitmentVacancy::VACANCY_ID => $id]);
        // echo '<pre>';print_r($this->db->last_query());
        
    }

    public function delete(Model $model, $id) {
        $array = $model->getArrayCopyForDB();
        $deleted_dt = date('Y-m-d');
        $rewardSql = "update HRIS_REC_VACANCY SET status = 'D', Deleted_By = {$array["DELETED_BY"]} , Deleted_Dt = '{$deleted_dt}' WHERE VACANCY_ID = {$id}";
        
        // print_r($rewardSql); die();
        return EntityHelper::rawQueryResult($this->adapter, $rewardSql);
    }
    
}   