<?php
namespace Recruitment\Controller;

use Application\Controller\HrisController;
use Recruitment\Form\AdmitForm;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Authentication\Storage\StorageInterface;
use Application\Helper\Helper;
use Zend\View\Model\ViewModel;
use Recruitment\Model\AdmitModel;
use Recruitment\Repository\AdmitRepository;
use Application\Helper\EntityHelper;
use Zend\View\Model\JsonModel;
use Exception;

Class AdmitController extends HrisController{

    function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(AdmitRepository::class);
        $this->initializeForm(AdmitForm::class);
        
    }
    public function indexAction(){


        $request = $this->getRequest();
        $detail = $this->repository->fetch();
        $model  = new AdmitModel();

        $model->exchangeArrayFromDB($detail);
        $model->DeclarationText = base64_decode($detail['DECLARATION_TEXT']);
        $model->Terms           = base64_decode($detail['TERMS']);

        /**
         * GET FILE AND ID
         * */
        $file = $model->FileName;
        $id   = $model->AdmitSetupId;

        // echo Helper::UPLOAD_DIR; die;
        
        $this->form->bind($model);

        return new ViewModel(Helper::addFlashMessagesToArray(
                $this, [
                    'form' => $this->form,
                    'data' => $data,
                    'prevFile'=>$file,
                    'prevId' => $id
                ]
            )
        ); 
    }

    public function updateAction() {

        $request = $this->getRequest();

        if ($request->isPost()) {

            $post = array_merge_recursive(
                $request->getPost()->toArray(),
                $request->getFiles()->toArray()
            );


            // echo "<pre>";
            // print_r($post);
            // echo Helper::UPLOAD_DIR;
            // die;

            if ($post['File']['error'] == 0) {

                /* create new name file */
                $filename     = uniqid() . "-" . time();
                $extension    = pathinfo( $post["File"]["name"], PATHINFO_EXTENSION ); // jpg

                $allow_extension = ['jpg','png','jpeg'];
                
                if (!in_array($extension, $allow_extension)) {
                    
                    $this->flashmessenger()->addMessage("{$extension} not accepted, Please upload jpg/jpeg/png types extensions!!!");
                    return $this->redirect()->toRoute("admit");
                
                }

                $basename     = $filename . "." . $extension; // 5dab1961e93a7_1571494241.jpg
                $source       = $post['File']["tmp_name"];
                $destination  = Helper::UPLOAD_DIR."/admit/{$basename}";




                if (isset($_SERVER['HTTPS']) &&
                    ($_SERVER['HTTPS'] == 'on' || $_SERVER['HTTPS'] == 1) ||
                    isset($_SERVER['HTTP_X_FORWARDED_PROTO']) &&
                    $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https') {
                  $protocol = 'https://';
                }
                else {
                  $protocol = 'http://';
                }

                $url          = $protocol.$_SERVER['HTTP_HOST']."/uploads/admit/{$basename}";

                /**
                 * UPLOADING IMAGES
                 * */
                $success      =  move_uploaded_file( $source, $destination);

                if (!empty($post['prev_image'])) {

                    $path  =  Helper::UPLOAD_DIR."/admit/".$prev_image;

                    if (file_exists($path)) {

                        unlink($path);

                    }

                }


                if ($success) {

                    if ($post['AdmitSetupId'] == 0) {

                        $admit_data = new AdmitModel();
                        $admit_data->exchangeArrayFromForm($post);
                        $admit_data->AdmitSetupId = ((int) Helper::getMaxId($this->adapter, AdmitModel::TABLE_NAME, AdmitModel::ADMIT_SETUP_ID)) + 1;
                        $admit_data->DeclarationText = base64_encode($post['DeclarationText']);
                        $admit_data->Terms = base64_encode($post['Terms']);
                        $admit_data->FileName  = $basename;
                        $admit_data->CreatedBy = $this->employeeId;
                        $admit_data->CreatedDt = Helper::getcurrentExpressionDate();
                        $admit_data->Status = 'E'; 
                        $admit_data->url = $url;
                        $this->repository->add($admit_data);

                    } else {

                        $admit_data = new AdmitModel();
                        $admit_data->exchangeArrayFromForm($post);
                        $admit_data->DeclarationText = base64_encode($post['DeclarationText']);
                        $admit_data->Terms = base64_encode($post['Terms']);
                        $admit_data->FileName  = $basename;
                        $admit_data->ModifiedBy = $this->employeeId;
                        $admit_data->ModifiedDt = Helper::getcurrentExpressionDate();
                        $admit_data->url = $url;

                        $this->repository->edit($admit_data, $post['AdmitSetupId']);

                    }
                    

                    $this->flashmessenger()->addMessage("Successfully Updated!!!");
                    return $this->redirect()->toRoute("admit");

                } else {

                    $this->flashmessenger()->addMessage("Unsuccessful Due to fail form validation!!!");
                        return $this->redirect()->toRoute("admit");

                }

            } else {

                $admit_data = new AdmitModel();
                $admit_data->exchangeArrayFromForm($post);
                $admit_data->DeclarationText = base64_encode($post['DeclarationText']);
                $admit_data->Terms = base64_encode($post['Terms']);
                $admit_data->ModifiedBy = $this->employeeId;
                $admit_data->ModifiedDt = Helper::getcurrentExpressionDate();

                $this->repository->edit($admit_data, $post['AdmitSetupId']);

                $this->flashmessenger()->addMessage("Successfully Updated!!!");
                    return $this->redirect()->toRoute("admit");

            }

        }
        

    }

    public function allAction() {


        $empId   = $this->employeeId;
        /**
         * VACANCY AD NO
         * allApplicationUsers(stage_id, vacancy_id)
         * 
         **/
        // $all_data = $this->repository->getAllPersonal();

        // $count_data = count($all_data);

        // for ($i=0; $i < $count_data ; $i++) { 
            
        //     $roll_no = base64_decode($all_data[$i]['ROLL_NO']);

        //     $update_roll_no = Helper::numConverter($roll_no, 'english');

        //     $this->repository->update_data($update_roll_no, $all_data[$i]['APPLICATION_ID']);

        // }

        // echo "<pre>";
        // print_r($all_data);
        // die;
        // $vacancydatas = $this->repository->allApplicationUsers(8, "29,30,31");
        $vacancydatas = $this->repository->allApplicationUsers(8, "34");

        // echo "<pre>";
        // print_r($vacancydatas);
        // die;

        $count = count($vacancydatas);

        for ($y = 0; $y < $count; $y++) {

            $inclusion[$vacancydatas[$y]['APPLICATION_ID']] = $this->repository->admitCardInclusion($vacancydatas[$y]['USER_ID'], $vacancydatas[$y]['APPLICATION_ID']);
            $nepaliName[$vacancydatas[$y]['APPLICATION_ID']] = $this->repository->getnepaliname($vacancydatas[$y]['USER_ID']);
        }
//         echo "<pre>";
//         print_r($inclusion);
//         die;

        // echo "<pre>";
        // print_r(count($vacancydatas));
        // print_r(count($inclusion));
        // die;


        for ($x = 0; $x < $count; $x++) {

            $conversion[$vacancydatas[$x]['APPLICATION_ID']]['ROLL_NO_NEPALI'] = Helper::numConverter(BASE64_DECODE($vacancydatas[$x]['ROLL_NO']), 'nepali');
            $conversion[$vacancydatas[$x]['APPLICATION_ID']]['CITIZENSHIP_NO_NEPALI'] = Helper::numConverter($vacancydatas[$x]['CITIZENSHIP_NO'], 'nepali');
            $conversion[$vacancydatas[$x]['APPLICATION_ID']]['CTZ_ISSUE_DATE_NEPALI'] = Helper::numConverter($vacancydatas[$x]['CTZ_ISSUE_DATE'], 'nepali');
            $conversion[$vacancydatas[$x]['APPLICATION_ID']]['FUNCTIONAL_LEVEL_EDESC_NEPALI'] = Helper::numConverter($vacancydatas[$x]['FUNCTIONAL_LEVEL_EDESC'], 'nepali');

        }

        // echo "<pre>";
        // print_r($conversion);
        // die;


        

        return Helper::addFlashMessagesToArray($this, [
            'vacancydatas' => $vacancydatas,
            'inclusion' => $inclusion,
            'conversion' => $conversion,
            'nepaliname'=> $nepaliName
        ]);

    }
    
}