<?php
namespace SelfService\Controller;

use Application\Controller\HrisController;
use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use Payroll\Repository\PayrollReportRepo;
use Payroll\Repository\SalarySheetDetailRepo;
use Payroll\Repository\SalarySheetRepo;
use Payroll\Repository\SalSheEmpDetRepo;
use Payroll\Repository\TaxSheetRepo;

use SelfService\Repository\ReportRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class Reports extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(ReportRepository::class); 
        $this->repository = new ReportRepository($adapter);
    }

    public function pfstatementAction() {
        
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $postedData = $request->getPost();

                $data = $this->repository->getPFStatement($postedData);

                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);

            } catch (Exception $e) {

                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);

            }
        }
        return $this->stickFlashMessagesTo(['employeeId' => $this->employeeId]);
    }

    public function citstatementAction() {
        
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $postedData = $request->getPost();

                $data = $this->repository->getCITStatement($postedData);

                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);

            } catch (Exception $e) {

                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);

            }
        }
        return $this->stickFlashMessagesTo(['employeeId' => $this->employeeId]);
    }

    public function payslipReportAction() {
        $request = $this->getRequest();
        $salarySheetRepo = new SalarySheetRepo($this->adapter);
        $salaryType = iterator_to_array($salarySheetRepo->fetchAllSalaryType(), false);
        if ($request->isPost()) {
            try {
                $postedData = $request->getPost();
                $salarySheetDetailRepo = new SalarySheetDetailRepo($this->adapter);
                $temp = $salarySheetDetailRepo->fetchEmployeePaySlipPayIds($postedData['monthId'], $this->employeeId,$postedData['salaryType'],$postedData['fiscalYearId']);
                

                $data['ruleList']=$temp;

                $data['data']=$salarySheetDetailRepo->fetchPaySlipPivoted($this->employeeId,$postedData['fiscalYearId'],$postedData['monthId'],$postedData['salaryType']);


                // $data['pay-detail'] = $salarySheetDetailRepo->fetchEmployeePaySlip($postedData['monthId'], $postedData['employeeId']);
              
                return new JsonModel(['success' => true, 'data' => $data, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }


        $salaryTypeList =EntityHelper::getTableList($this->adapter, hris_salary_type, ['SALARY_TYPE_ID' => "SALARY_TYPE_ID", 'SALARY_TYPE_NAME'=>"SALARY_TYPE_NAME"]);
        $data['salaryTypeList']=$salaryTypeList;

        return $data;
    }

    
}
