<?php

namespace SelfService\Controller;

use Application\Controller\HrisController;
use Application\Helper\Helper;
use Exception;
use SelfService\Form\RetirementForm;
use SelfService\Model\RetirementModel;
use SelfService\Repository\RetirementRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Mvc\Controller\Plugin\Redirect;
use Zend\View\Model\JsonModel;

class RetirementController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(RetirementRepository::class);
        $this->initializeForm(RetirementForm::class);
    }

    public function indexAction(){
        
        $retirementRequest = $this->repository->showRetirementApplication($this->employeeId);

        $data = null;
        $documents = null;

        if($retirementRequest){
            $data = $this->repository->fetchById($retirementRequest['RETIREMENT_ID']);
            $documents = iterator_to_array($this->repository->fetchImagesById($retirementRequest['RETIREMENT_ID']), false);
        }

        return Helper::addFlashMessagesToArray($this, [
            'retirementRequest' => $retirementRequest,
            'data' => $data,
            'documents' => $documents,
        ]);
    }

    public function addAction(){

        $retirementRequest = $this->repository->showRetirementApplication($this->employeeId);

        if($retirementRequest != null && $retirementRequest['STATUS'] != 'R'){
            return $this->redirect()->toRoute("retirement"); 
        }

        $request = $this->getRequest();
        
        if ($request->isPost()) {

            $this->form->setData($request->getPost());

            if ($this->form->isValid()) {
                $retirementModel = new RetirementModel();
                $retirementModel->exchangeArrayFromForm($this->form->getData());
                $retirementModel->retirementId = ((int) Helper::getMaxId($this->adapter, RetirementModel::TABLE_NAME, RetirementModel::RETIREMENT_ID)) + 1;
                $retirementModel->createdDate = Helper::getcurrentExpressionDate();
                $retirementModel->isDeputyDirector = $this->repository->isDeputyDirectory($retirementModel->employeeId); 
                $retirementModel->isAgeLimit = 'N';
                $retirementModel->isServiceLimit = 'N';
                $retirementModel->status = 'RQ';
                $retirementModel->remarks = $request->getPost()['remarks'];
                $retirementModel->validFrom = date('Y-m-d', strtotime($retirementModel->validFrom));

                $this->repository->add($retirementModel);

                return new JsonModel(['success'=>true]);
            }
        }

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'employeeId' => $this->employeeId
        ]);

    }

    public function fileUploadAction()
    {
        $request = $this->getRequest();
        $responseData = [];
        $files = $request->getFiles()->toArray();
        try {
            if (sizeof($files) > 0) {
                $ext = pathinfo($files['file']['name'], PATHINFO_EXTENSION);
                if ($ext == 'txt' || $ext == 'pdf' || $ext == 'jpg' || $ext == 'jpeg' || $ext == 'png' || $ext=='docx' || $ext=='odt' || $ext=='doc' ) {
                    $fileName = pathinfo($files['file']['name'], PATHINFO_FILENAME);
                    $unique = Helper::generateUniqueName();
                    $newFileName = $unique . "." . $ext;
                    if (!file_exists(Helper::UPLOAD_DIR . "/retirement_document/")) {
                        mkdir(Helper::UPLOAD_DIR . "/retirement_document/", 0777, true);
                    }
                    $success = move_uploaded_file($files['file']['tmp_name'], Helper::UPLOAD_DIR . "/retirement_document/" . $newFileName);
                    if (!$success) {
                        throw new Exception("Upload unsuccessful.");
                    }
                    $responseData = ["success" => true, "data" => ["fileName" => $newFileName, "oldFileName" => $fileName . "." . $ext]];
                } else { 
                    throw new Exception("Upload unsuccessful.");
                }
            }
        } catch (Exception $e) {
            $responseData = [
                "success" => false,
                "message" => $e->getMessage(),
                "traceAsString" => $e->getTraceAsString(),
                "line" => $e->getLine()
            ];
        }
        return new JsonModel($responseData);
    }

    public function pushFileLinkAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $this->repository->pushFileLink($data);
            return new JsonModel(['success' => true, 'data' => $data, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function pullFilesByIdAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $files = $this->repository->pullFilesbyModificationId($data['id']);
            return new JsonModel(['success' => true, 'data' => $files, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

    public function deleteFileByNameAction(){
        try {
            $request = $this->getRequest();
            $data = $request->getPost();
            $this->repository->deleteFileByName($data['name']);
            return new JsonModel(['success' => true, 'data' => null, 'message' => null]);
        } catch (Exception $e) {
            return new JsonModel(['success' => false, 'data' => null, 'message' => $e->getMessage()]);
        }
    }

}