<?php

namespace SelfService\Repository;

use Application\Model\Model;
use Application\Helper\Helper;
use Application\Repository\RepositoryInterface;
use SelfService\Model\LoanRequest;
use SelfService\Model\LoanEmiDetail;
use Setup\Model\HrEmployees;
use Setup\Model\Loan;
use Application\Repository\HrisRepository;
use PhpOffice\PhpSpreadsheet\Calculation\MathTrig\Exp;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Expression;
use Zend\Db\Sql\Sql;
use Zend\Db\TableGateway\TableGateway; 


class LoanRequestRepository extends HrisRepository implements RepositoryInterface {

    protected $tableGateway;
    
    public function __construct(AdapterInterface $adapter) {
        $this->adapter = $adapter;
        $this->tableGateway = new TableGateway(LoanRequest::TABLE_NAME, $adapter);
        $this->emiTableGateway = new TableGateway(LoanEmiDetail::TABLE_NAME, $adapter);
    }

    public function add(Model $model) {
        $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function delete($id) {
        $this->tableGateway->update([LoanRequest::STATUS => 'C'], [LoanRequest::LOAN_REQUEST_ID => $id]);
    }

    public function edit(Model $model, $id) {
        $this->tableGateway->update($model->getArrayCopyForDB(), [LoanRequest::LOAN_REQUEST_ID => $id]);
    }

    public function fetchAll() {
        
    }

    public function fetchById($id) {
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->columns([
            new Expression("INITCAP(TO_CHAR(LR.LOAN_DATE, 'DD-MON-YYYY')) AS LOAN_DATE"),
            new Expression("BS_DATE(LR.LOAN_DATE) AS LOAN_DATE_BS"),
            new Expression("ADD_MONTHS(LR.LOAN_DATE, LR.REPAYMENT_MONTHS) AS LOAN_DATE_END"),
	        new Expression("BS_DATE(ADD_MONTHS(LR.LOAN_DATE, LR.REPAYMENT_MONTHS)) AS LOAN_DATE_END_BS"),
            new Expression("L.INTEREST_RATE AS LOAN_INTEREST"),
            new Expression("LR.STATUS AS STATUS"),
            new Expression("LR.REPAYMENT_MONTHS AS REPAYMENT_MONTHS"),
            new Expression("LR.APPLIED_LOAN AS APPLIED_LOAN"),
            new Expression("LR.PERIOD AS PERIOD"),
            new Expression("LR.REPAYMENT_INSTALLMENTS AS REPAYMENT_INSTALLMENTS"),
            new Expression("LR.INTEREST_RATE AS INTEREST_RATE"),
            new Expression("LR.MONTHLY_INSTALLMENT_AMOUNT AS MONTHLY_INSTALLMENT_AMOUNT"),
            new Expression("INITCAP(TO_CHAR(LR.REQUESTED_DATE, 'DD-MON-YYYY')) AS REQUESTED_DATE"),
            new Expression("INITCAP(TO_CHAR(LR.APPROVED_DATE, 'DD-MON-YYYY')) AS APPROVED_DATE"),
            new Expression("INITCAP(TO_CHAR(LR.RECOMMENDED_DATE, 'DD-MON-YYYY')) AS RECOMMENDED_DATE"),
            new Expression("LR.REQUESTED_AMOUNT AS REQUESTED_AMOUNT"),
            new Expression("LR.LOAN_REQUEST_ID AS LOAN_REQUEST_ID"),
            new Expression("LR.REASON AS REASON"),
            new Expression("LR.EMPLOYEE_ID AS EMPLOYEE_ID"),
            new Expression("LR.FILE_PATH AS FILE_PATH"),
            new Expression("LR.RECOMMENDED_BY AS RECOMMENDED_BY"),
            new Expression("LR.APPROVED_BY AS APPROVED_BY"),
            new Expression("LR.APPROVED_REMARKS AS APPROVED_REMARKS"),
            new Expression("LR.RECOMMENDED_REMARKS AS RECOMMENDED_REMARKS"),
            new Expression("LR.LOAN_ID AS LOAN_ID"),
            new Expression("LR.MONTH_ID AS MONTH_ID"),
            new Expression("LR.FISCAL_YEAR_ID AS FISCAL_YEAR_ID"),
            new Expression("LR.MONTHLY_INTEREST_RATE AS MONTHLY_INTEREST_RATE"),
            new Expression("LR.CIT AS CIT"),

                ], true);

        $select->from(['LR' => LoanRequest::TABLE_NAME])
                ->join(['E' => HrEmployees::TABLE_NAME], "E." . HrEmployees::EMPLOYEE_ID . "=LR." . LoanRequest::EMPLOYEE_ID, ["FIRST_NAME" => new Expression("INITCAP(E.FIRST_NAME)"), "MIDDLE_NAME" => new Expression("INITCAP(E.MIDDLE_NAME)"), "LAST_NAME" => new Expression("INITCAP(E.LAST_NAME)")])
                ->join(['L' => Loan::TABLE_NAME], "L." . Loan::LOAN_ID . "=LR." . LoanRequest::LOAN_ID, [Loan::LOAN_CODE, "LOAN_NAME" => new Expression("INITCAP(L.LOAN_NAME)")])
                ->join(['E1' => "HRIS_EMPLOYEES"], "E1.EMPLOYEE_ID=LR.RECOMMENDED_BY", ['FN1' => new Expression("INITCAP(E1.FIRST_NAME)"), 'MN1' => new Expression("INITCAP(E1.MIDDLE_NAME)"), 'LN1' => new Expression("INITCAP(E1.LAST_NAME)")], "left")
                ->join(['E2' => "HRIS_EMPLOYEES"], "E2.EMPLOYEE_ID=LR.APPROVED_BY", ['FN2' => new Expression("INITCAP(E2.FIRST_NAME)"), 'MN2' => new Expression("INITCAP(E2.MIDDLE_NAME)"), 'LN2' => new Expression("INITCAP(E2.LAST_NAME)")], "left");

        $select->where([
            "LR.LOAN_REQUEST_ID =" . $id
        ]);
        $select->order("LR.REQUESTED_DATE DESC");
        $statement = $sql->prepareStatementForSqlObject($select);
        // echo "<pre>";
        // print_r($statement->getSql());die;
        $result = $statement->execute();
        return $result->current();
    }

    public function getAllByEmployeeId($employeeId) {
        $sql = new Sql($this->adapter);
        $select = $sql->select();
        $select->columns([
            new Expression("INITCAP(TO_CHAR(LR.LOAN_DATE, 'DD-MON-YYYY')) AS LOAN_DATE_AD"),
            new Expression("BS_DATE(LR.LOAN_DATE) AS LOAN_DATE_BS"),
            new Expression("LR.STATUS AS STATUS"),
            new Expression("LR.LOAN_STATUS AS LOAN_STATUS"),
            new Expression("INITCAP(TO_CHAR(LR.REQUESTED_DATE, 'DD-MON-YYYY')) AS REQUESTED_DATE_AD"),
            new Expression("BS_DATE(LR.REQUESTED_DATE) AS REQUESTED_DATE_BS"),
            new Expression("INITCAP(TO_CHAR(LR.APPROVED_DATE, 'DD-MON-YYYY')) AS APPROVED_DATE"),
            new Expression("INITCAP(TO_CHAR(LR.RECOMMENDED_DATE, 'DD-MON-YYYY')) AS RECOMMENDED_DATE"),
            new Expression("LR.REQUESTED_AMOUNT AS REQUESTED_AMOUNT"),
            new Expression("LR.LOAN_REQUEST_ID AS LOAN_REQUEST_ID"),
            new Expression("LR.REASON AS REASON"),
            new Expression("LR.RECOMMENDED_REMARKS AS RECOMMENDED_REMARKS"),
            new Expression("LR.APPROVED_REMARKS AS APPROVED_REMARKS"),
            new Expression("LR.LOAN_ID AS LOAN_ID")
                ], true);

        $select->from(['LR' => LoanRequest::TABLE_NAME])
                ->join(['E' => HrEmployees::TABLE_NAME], "E." . HrEmployees::EMPLOYEE_ID . "=LR." . LoanRequest::EMPLOYEE_ID, ["FIRST_NAME" => new Expression("INITCAP(E.FIRST_NAME)"), "MIDDLE_NAME" => new Expression("INITCAP(E.MIDDLE_NAME)"), "LAST_NAME" => new Expression("INITCAP(E.LAST_NAME)")])
                ->join(['L' => Loan::TABLE_NAME], "L." . Loan::LOAN_ID . "=LR." . LoanRequest::LOAN_ID, [Loan::LOAN_CODE, "LOAN_NAME" => new Expression("INITCAP(L.LOAN_NAME)")])
                ->join(['E1' => "HRIS_EMPLOYEES"], "E1.EMPLOYEE_ID=LR.RECOMMENDED_BY", ['FN1' => new Expression("INITCAP(E1.FIRST_NAME)"), 'MN1' => new Expression("INITCAP(E1.MIDDLE_NAME)"), 'LN1' => new Expression("INITCAP(E1.LAST_NAME)")], "left")
                ->join(['E2' => "HRIS_EMPLOYEES"], "E2.EMPLOYEE_ID=LR.APPROVED_BY", ['FN2' => new Expression("INITCAP(E2.FIRST_NAME)"), 'MN2' => new Expression("INITCAP(E2.MIDDLE_NAME)"), 'LN2' => new Expression("INITCAP(E2.LAST_NAME)")], "left");

        $select->where([
            "E.EMPLOYEE_ID=" . $employeeId
        ]);
        $select->where([
            "days_between(LR.REQUESTED_DATE, current_date) < (
                      CASE
                        WHEN LR.STATUS = 'C'
                        THEN 20
                        ELSE 365
                      END) AND LR.REQUESTED_AMOUNT > 0"
        ]);
        $select->order("LR.REQUESTED_DATE DESC");
        $statement = $sql->prepareStatementForSqlObject($select);
        $result = $statement->execute();
        return $result;
    }
    
    public function getDefaultInterestRate($loan_id){
        $sql = "SELECT INTEREST_RATE FROM HRIS_LOAN_MASTER_SETUP WHERE LOAN_ID = :loan_id";

        $boundedParameter = [];
        $boundedParameter['loan_id'] = $loan_id;
        return $this->rawQuery($sql, $boundedParameter);
    }

    public function getLoanDetails(){
        $sql = "SELECT LOAN_ID, LOAN_NAME, IS_RATE_FLEXIBLE, INTEREST_RATE, MIN_AMOUNT, MAX_AMOUNT, REPAYMENT_PERIOD, IS_ACCUMULATION, ACCUMULATION_MONTH FROM HRIS_LOAN_MASTER_SETUP";
        $statement = $this->adapter->query($sql);
        return $statement->execute();
    }

    public function getLoanDetailById($loanId){
        $sql = "SELECT * FROM HRIS_LOAN_MASTER_SETUP WHERE LOAN_ID = {$loanId}";
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current();
    }

    public function pullFilebyId($id)
    {
        $boundedParams = [];
        $boundedParams['id'] = $id;
        $sql = "select FILE_PATH from HRIS_EMPLOYEE_LOAN_REQUEST where LOAN_REQUEST_ID = $id limit 1";
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current();;
    }

    public function getCitValOfLatestMonth($empId) {

        $sql = "SELECT 
                    SSD.VAL 
                FROM HRIS_SALARY_SHEET_DETAIL SSD 
                LEFT JOIN HRIS_SALARY_SHEET SS ON (SS.SHEET_NO = SSD.SHEET_NO)
                WHERE 
                    EMPLOYEE_ID = {$empId} AND 
                    PAY_ID = 18 
                ORDER BY SS.MONTH_ID DESC 
                LIMIT 1";

        
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current()['VAL'];
    }

    public function getTechGradeOfLatestMonth($empId) {

        $sql = "SELECT 
                    SSD.VAL 
                FROM HRIS_SALARY_SHEET_DETAIL SSD 
                LEFT JOIN HRIS_SALARY_SHEET SS ON (SS.SHEET_NO = SSD.SHEET_NO)
                WHERE 
                    EMPLOYEE_ID = {$empId} AND 
                    PAY_ID = 56 
                ORDER BY SS.MONTH_ID DESC 
                LIMIT 1";

        
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current()['VAL'];
    }

    public function getLoanInfo($empId){
        $sql = "SELECT E.EMPLOYEE_CODE, E.FULL_NAME,
                (SELECT 
                    BASIC_SALARY 
                FROM HRIS_EMPLOYEES_GRADE_CEILING_MASTER_SETUP 
                WHERE SERVICE_TYPE_ID = E.SERVICE_TYPE_ID 
                    AND FUNCTIONAL_LEVEL_ID = IFNULL(E.ACTING_FUNCTIONAL_LEVEL_ID, E.FUNCTIONAL_LEVEL_ID)
                    AND POSITION_ID = E.POSITION_ID
                ) AS BASIC_SALARY,

                (SELECT FLAT_VALUE FROM HRIS_FLAT_VALUE_DETAIL 
                 WHERE EMPLOYEE_ID = E.EMPLOYEE_ID 
                    AND FLAT_ID = 16 
                    AND FISCAL_YEAR_ID = (SELECT FISCAL_YEAR_ID 
                                          FROM HRIS_FISCAL_YEARS
                                          WHERE CURRENT_DATE BETWEEN START_DATE AND END_DATE
                                          )
                ) AS FLAT_VALUE,

                (SELECT \"1DAY_SALARY\" FROM HRIS_EMPLOYEES_GRADE_CEILING_MASTER_SETUP
                 WHERE SERVICE_TYPE_ID = E.SERVICE_TYPE_ID 
                    AND FUNCTIONAL_LEVEL_ID = E.FUNCTIONAL_LEVEL_ID
                    AND POSITION_ID = E.POSITION_ID
                ) AS ONE_DAY,

                BS_DATE(E.BIRTH_DATE) AS DOB_BS,
                BS_DATE(E.JOIN_DATE) AS JOIN_DATE_BS,
                BS_DATE(CURRENT_DATE) AS CURRENT_DATE_BS

                FROM HRIS_EMPLOYEES E 

                WHERE E.EMPLOYEE_ID = {$empId}";

        // print_r($sql);die;
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current();;
    }

    public function getDetailLoanInfo($empId){
        $sql = "Select ssd.val, ps.pay_edesc, ssd.pay_id from hris_salary_sheet_detail ssd
        left join hris_pay_setup ps on (ps.pay_id = ssd.pay_id)
        where ssd.employee_id = {$empId}
        and ssd.sheet_no = (select max(sheet_no) from hris_salary_sheet_detail where employee_id = {$empId}
        and sheet_no in (select sheet_no from hris_salary_sheet where salary_type_id = 1
        and month_id < (select month_id from hris_month_code where current_date
        between from_date and to_date)))
        and ssd.pay_id in (106,104,97,84,18,99,27,100,82,103,85,83,96,98)" ;
        // echo "<pre>"; print_r($sql); die;
        return $this->rawQuery($sql);
    }

    public function validateLoanRequest($empId,$loanAmount,$period, $loanId, $installment, $citVal){
        $parameter = "";
        if($empId){
            $parameter.=$empId.",";
        }else{
            $parameter.="null,";
        }
        if($loanAmount){
            $parameter.=$loanAmount.",";
        }else{
            $parameter.="null,null,";
        }
        if($period){
            $parameter.=$period.",";
        }else{
            $parameter.="null,null,";
        }
        if($installment){
            $parameter.=$installment.",";
        }else{
            $parameter.="null,";
        }
        if($loanId){
            $parameter.=$loanId.",";
        }else{
            $parameter.="null,";
        }
        if($citVal){
            $parameter.=$citVal;
        }else{
            $parameter.="null";
        }
        $sql="SELECT HRIS_FN_LOAN_CHECK($parameter) AS ERROR FROM DUMMY";
        print_r($sql);die;
        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current();
    }

    public function emiAdd($emiModel){
        // echo "<pre>";print_r($emiModel);die;
        $this->emiTableGateway->insert($emiModel->getArrayCopyForDB());
    }

    public function emiDelete($id) { 
        $this->emiTableGateway->update([LoanEmiDetail::STATUS => 'C'], [LoanEmiDetail::LOAN_REQUEST_ID => $id]);
    }

    public function fetchLoanDetailView($loanId){
        $sql = "select EMD.*, MC.MONTH_EDESC || '-' || MC.YEAR as PAYMENT_MONTH_EDESC from hris_employee_emi_detail EMD
        left join hris_month_code MC on (MC.month_id = EMD.PAYMENT_MONTH)
        WHERE loan_request_id = $loanId order by sno asc";
        
        return $this->rawQuery($sql);

        
    }

    public function getCurrentFiscalYears() {

        $sql = "SELECT FISCAL_YEAR_ID FROM HRIS_FISCAL_YEARS WHERE CURRENT_DATE BETWEEN START_DATE AND END_DATE";

        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current()['FISCAL_YEAR_ID'];

    }

    public function getCurrentFiscalYearsDetail() {

        $sql = "SELECT * FROM HRIS_FISCAL_YEARS WHERE CURRENT_DATE BETWEEN START_DATE AND END_DATE";

        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current();

    }

    public function getCurrentFiscalYearsDetails() {

        $sql = "SELECT * FROM HRIS_FISCAL_YEARS WHERE STATUS = 'E' ORDER BY FISCAL_YEAR_ID DESC";

        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result;

    }

    public function getCurrentMonth($fiscal_year) {

        $sql = "SELECT 
                    MONTH_ID 
                FROM HRIS_MONTH_CODE 
                WHERE FISCAL_YEAR_ID = {$fiscal_year} AND CURRENT_DATE BETWEEN FROM_DATE AND TO_DATE";

        $statement = $this->adapter->query($sql);
        $result = $statement->execute();
        return $result->current()['MONTH_ID'];

    }

    public function validateSameLoanRequest($loanId, $employeeId)
    {
        if ($loanId) {

            $sql = "SELECT * FROM HRIS_EMPLOYEE_LOAN_REQUEST WHERE LOAN_ID = {$loanId} AND EMPLOYEE_ID = {$employeeId}";

            $statement = $this->adapter->query($sql);
            $result = $statement->execute();
            return $result->current(); 
        }
        
        return false;
    }

    public function statement($postData)
    {
        $from = "'".date('Y-m-d', strtotime($postData['from']))."'";
        $to   = "'".date('Y-m-d', strtotime($postData['to']))."'";
        $sql = "SELECT 
                    *
                FROM HRIS_LOAN_PAYMENT_DETAIL
                WHERE 
                    LOAN_REQUEST_ID = {$postData['loanRequestId']} AND
                    EMPLOYEE_ID = {$postData['employeeId']} AND
                    PAID_FLAG = 'Y' AND
                    CREATED_DT BETWEEN {$from} AND {$to}
                ";

        $payment_details = $this->rawQuery($sql);

        for ($i=0; $i < count($payment_details) ; $i++) { 
            
            $sql1 = "SELECT 
                        *
                    FROM HRIS_EMPLOYEE_EMI_DETAIL EMI
                    LEFT JOIN HRIS_MONTH_CODE C ON C.MONTH_ID = EMI.PAYMENT_MONTH
                    LEFT JOIN HRIS_FISCAL_YEARS Y ON Y.FISCAL_YEAR_ID = C.FISCAL_YEAR_ID
                    WHERE 
                        EMI.LOAN_REQUEST_ID = {$postData['loanRequestId']} AND
                        EMI.EMPLOYEE_ID = {$postData['employeeId']} AND
                        EMI.PAID_FLAG = 'Y' AND
                        EMI.SNO = {$payment_details[$i]['SNO']}
                    ";

            $payment_detail1 = $this->rawQuery($sql1);

            $payment_details[$i]['LOAN_AMOUNT'] = $payment_detail1[0]['LOAN_AMOUNT'];
            $payment_details[$i]['INSTALLMENT'] = $payment_detail1[0]['INSTALLMENT'];
            $payment_details[$i]['INTEREST'] = $payment_detail1[0]['INTEREST'];
            $payment_details[$i]['PRINCIPAL_REPAID'] = $payment_detail1[0]['PRINCIPAL_REPAID'];
            $payment_details[$i]['REMAINING_PRINCIPAL'] = $payment_detail1[0]['REMAINING_PRINCIPAL'];
            $payment_details[$i]['MONTH_EDESC'] = $payment_detail1[0]['MONTH_EDESC'];
            $payment_details[$i]['YEAR'] = $payment_detail1[0]['YEAR'];
            
        }

        return $payment_details;

    }

    public function loanAmountRemaining($postData) {

        $sql = "SELECT 
                        *
                    FROM HRIS_EMPLOYEE_EMI_DETAIL EMI
                    LEFT JOIN HRIS_MONTH_CODE C ON C.MONTH_ID = EMI.PAYMENT_MONTH
                    LEFT JOIN HRIS_FISCAL_YEARS Y ON Y.FISCAL_YEAR_ID = C.FISCAL_YEAR_ID
                    WHERE 
                        EMI.LOAN_REQUEST_ID = {$postData['loanRequestId']} AND
                        EMI.EMPLOYEE_ID = {$postData['employeeId']} AND
                        EMI.PAID_FLAG = 'N'
                    ORDER BY EMI.SNO ASC LIMIT 1
                    ";

        return $this->rawQuery($sql);

    }

    public function pullHasSubLoan($data)
    {
        $result = $this->rawQuery("SELECT * FROM HRIS_LOAN_MASTER_SETUP WHERE LOAN_ID = {$data['loanId']}");

        if ($result[0]['HAS_SUB_LOAN'] == 'Y') {

            $sql = "SELECT 
                        *
                FROM HRIS_LOAN_MASTER_SETUP_SUB
                WHERE 
                    LOAN_ID = {$result[0]['LOAN_ID']}
                ";

            return $this->rawQuery($sql);

        } else {

            return false;

        }
        
    }

    public function getLoanListEligible($employeeId)
    {

        $sql = "SELECT 
                    *
                FROM HRIS_LOAN_MASTER_SETUP L
                WHERE 
                    ELIGIBLE_SERVICE_PERIOD <= (SELECT 
                                                    YEARS_BETWEEN(TO_DATE(JOIN_DATE), TO_DATE(CURRENT_DATE)) 
                                                FROM HRIS_EMPLOYEES 
                                                WHERE EMPLOYEE_ID = {$employeeId}
                                                )";

        $result = $this->rawQuery($sql);

        return $result;
    }

    public function fetchLoanRequestByLoanId($status, $loanId)
    {
        $sql = "SELECT LOAN_REQUEST_ID, EMPLOYEE_ID, REQUESTED_AMOUNT, REPAYMENT_MONTHS FROM HRIS_EMPLOYEE_LOAN_REQUEST WHERE LOAN_ID = {$loanId} AND LOAN_STATUS = '{$status}' AND MONTHLY_INSTALLMENT_AMOUNT > 0 AND EMPLOYEE_ID IS NOT NULL";

        $statement = $this->rawQuery($sql);
        return $statement; 
    }
}
