<?php

namespace SelfService\Repository;

use Application\Model\Model;
use Application\Repository\HrisRepository;
use SelfService\Model\RetirementModel;
use Zend\Db\Adapter\AdapterInterface;

class RetirementRepository extends HrisRepository {

    public function __construct(AdapterInterface $adapter, $tableName = null) {
        parent::__construct($adapter, RetirementModel::TABLE_NAME);
    }

    public function add(Model $data){
        $array = $data->getArrayCopyForDb();
        $this->tableGateway->insert($array);
        $this->linkWithFiles();
    }

    public function isDeputyDirectory($employeeId){

        $sql = "
            select  
            case when functional_level_id = 1 then 'Y'
            else 'N' end as IS_DD
            from hris_employees where employee_id = {$employeeId};
        ";

        return $this->adapter->query($sql)->execute()->current()['IS_DD'];

    }

    public function pushFileLink($data){
        $fileName = $data['file'];
        $id = $data['id'] == null ? 0 : $data['id'] ;
        $oldfile = $data['fileName'];
        $sql = "INSERT INTO HRIS_RETIREMENT_DOCUMENT(FILE_ID, FILE_NAME, FILE_IN_DIR_NAME, UPLOADED_DATE, RETIREMENT_ID)
        VALUES( (select ifnull(max(FILE_ID)+1, 1) from HRIS_RETIREMENT_DOCUMENT), '{$fileName}', '{$oldfile}', current_date, {$id})";
        return $this->rawQuery($sql);
    }

    public function pullFilesbyModificationId($id){
        $sql = "select * from HRIS_RETIREMENT_DOCUMENT where RETIREMENT_ID = $id and status = 'E'";
        return $this->rawQuery($sql);
    }

    public function deleteFileByName($name){
        $sql = "update HRIS_RETIREMENT_DOCUMENT set status = 'D' where FILE_NAME = '{$name}'";
        return $this->rawQuery($sql);
    }

    public function linkWithFiles(){
        if(!empty($_POST['fileUploadList'])){
            $filesList = $_POST['fileUploadList'];
            $filesList = "'".implode("','" , $filesList)."'";
            $sql = "UPDATE HRIS_RETIREMENT_DOCUMENT SET RETIREMENT_ID = (SELECT MAX(RETIREMENT_ID) FROM HRIS_RETIREMENT) 
                    WHERE FILE_NAME IN($filesList)";

            $statement = $this->adapter->query($sql);
            $statement->execute();
        }
    }

    public function showRetirementApplication($employeeId){

        $sql = "SELECT * FROM HRIS_RETIREMENT WHERE EMPLOYEE_ID = {$employeeId} ORDER BY CREATED_DATE DESC LIMIT 1";

        return $this->adapter->query($sql)->execute()->current();

    }

    public function retirementToApprove($employeeId, $isHr = false){

        $orderQuery = " WHERE 1 = 2";

        $sql = "
            select employee_id from hris_employees e 
            left join hris_functional_levels f on e.functional_level_id = f.functional_level_id
            where location_id = (select location_id from hris_employees where employee_id = {$employeeId}) 
            order by order_id asc limit 1;
        ";

        if($employeeId == $this->adapter->query($sql)->execute()->current()['EMPLOYEE_ID']){
            // Is manager condition
            $orderQuery = " WHERE ORDER_ID >= 8 AND E.LOCATION_ID = 
                        (SELECT LOCATION_ID FROM HRIS_EMPLOYEES WHERE EMPLOYEE_ID = {$employeeId})";
        }

        $sql = "
            select functional_level_id from hris_employees where employee_id = {$employeeId}
        ";

        // If executive director
        if($this->adapter->query($sql)->execute()->current()['FUNCTIONAL_LEVEL_ID'] == 16){
            $orderQuery = " WHERE ORDER_ID < 8";
        }

        $sql = "SELECT
                R.RETIREMENT_ID AS RETIREMENT_ID,
                HRIS_RETIREMENT_REASON(R.REASON) AS REASON,
                LEAVE_STATUS_DESC(R.STATUS) AS STATUS,
                R.VALID_FROM AS VALID_FROM,
                BS_DATE(R.VALID_FROM) AS VALID_FROM_N,
                R.CREATED_DATE AS CREATED_DATE,
                BS_DATE(R.CREATED_DATE) AS CREATED_DATE_N,
                E.FULL_NAME AS FULL_NAME,
                E.EMPLOYEE_CODE AS EMPLOYEE_CODE,
                F.FUNCTIONAL_LEVEL_NO
                FROM HRIS_RETIREMENT R
                LEFT JOIN HRIS_EMPLOYEES E ON E.EMPLOYEE_ID = R.EMPLOYEE_ID
                LEFT JOIN HRIS_FUNCTIONAL_LEVELS F ON 
                (SELECT CASE WHEN ACTING_FUNCTIONAL_LEVEL_ID IS NULL THEN FUNCTIONAL_LEVEL_ID ELSE FUNCTIONAL_LEVEL_ID END AS ID 
                FROM HRIS_EMPLOYEES WHERE EMPLOYEE_ID = E.EMPLOYEE_ID) = F.FUNCTIONAL_LEVEL_ID {$orderQuery};
            ";

        $boundedParameter = [];
        $boundedParameter['employeeId1'] = $employeeId;
        $boundedParameter['employeeId2'] = $employeeId;
        $boundedParameter['employeeId3'] = $employeeId;
        return $this->rawQuery($sql,$boundedParameter);

    }

    public function fetchById($id){
        $sql = "
            SELECT 
            E.FULL_NAME AS FULL_NAME,
            HRIS_RETIREMENT_REASON(R.REASON) AS REASON,
            R.IS_DEPUTY_DIRECTORY AS IS_DEPUTY_DIRECTOR,
            BS_DATE(R.VALID_FROM) AS VALID_FROM,
            R.REMARKS AS REMARKS,
            BS_DATE(R.CREATED_DATE) AS CREATED_DATE,
            R.IS_AGE_LIMIT AS IS_AGE_LIMIT,
            LEAVE_STATUS_DESC(R.STATUS) AS STATUS,
            R.IS_SERVICE_LIMIT AS IS_SERVICE_LIMIT,
            R.APPROVED_REMARKS AS APPROVED_REMARKS,
            R.RETIREMENT_ID AS RETIREMENT_ID,
            R.APPROVED_REMARKS AS APPROVED_REMARKS
            FROM HRIS_RETIREMENT R
            LEFT JOIN HRIS_EMPLOYEES E ON R.EMPLOYEE_ID = E.EMPLOYEE_ID
            WHERE RETIREMENT_ID = {$id}
        ";

        return $this->adapter->query($sql)->execute()->current();
    }

    public function fetchImagesById($id){

        $sql = "
            SELECT FILE_NAME FROM HRIS_RETIREMENT_DOCUMENT WHERE STATUS = 'E' AND RETIREMENT_ID = {$id}
        ";

        return $this->adapter->query($sql)->execute();

    }

    public function approveReject($data, $employeeId){

        $status = $data['status'];
        $approvedRemarks = $data['approvalRemarks'];

        $sql = "
            UPDATE HRIS_RETIREMENT 
            SET STATUS = '{$status}', APPROVED_REMARKS = '{$approvedRemarks}', APPROVED_BY = {$employeeId}, APPROVED_DATE = CURRENT_DATE
            WHERE RETIREMENT_ID = {$data['retirementId']}
        ";

        $this->adapter->query($sql)->execute();

    }

}