<?php

namespace Setup\Controller;

use Application\Controller\HrisController;
use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use Payroll\Repository\BasicSalaryRepository;
use Payroll\Repository\FlatValueRepository;
use Payroll\Repository\PayrollRepository;
use Setup\Repository\LeaveYearRepository;
use Setup\Form\JobHistoryForm;
use Setup\Model\Branch;
use Setup\Model\Company;
use Setup\Model\Department;
use Setup\Model\Designation;
use Setup\Model\HrEmployees;
use Setup\Model\JobHistory;
use Setup\Model\Position;
use Setup\Model\ServiceEventType;
use Setup\Repository\EmployeeRepository;
use Setup\Repository\JobHistoryRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Predicate\Predicate;
use PrintLayout\Controller\PrintController;
use PrintLayout\Repository\PrintRepository;
use Recruitment\Repository\UserApplicationRepository;
use Zend\View\Model\JsonModel;

class LeaveYearController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(LeaveYearRepository::class);
    }

    public function indexAction() {

        $request = $this->getRequest();
        if ($request->isPost()) {

            $data = iterator_to_array($this->repository->fetchAll(), false);
            return new JsonModel(['success'=>true, 'data'=>$data]);
        
        }

    }

    public function addAction(){
        
        $request = $this->getRequest();
        if ($request->isPost()) {

            $postData = $request->getPost();
            
            $leaveYearId = ((int) Helper::getMaxId($this->adapter, 'HRIS_LEAVE_YEARS', 'LEAVE_YEAR_ID')) + 1;
            $startDate = date('Y-m-d', strtotime($postData['fromDateYear']));
            $endDate = date('Y-m-d', strtotime($postData['toDateYear']));
            $leaveYear = $postData['leaveYearName'];
            $this->repository->insertYear($leaveYearId, $startDate, $endDate, $leaveYear);


            for($i=0; $i<count($postData['monthName']); $i++){

                $monthId = ((int) Helper::getMaxId($this->adapter, 'HRIS_LEAVE_MONTH_CODE', 'MONTH_ID')) + 1;
                $monthName = $postData['monthName'][$i];
                $fromDate = date('Y-m-d', strtotime($postData['fromDate'][$i]));
                $toDate = date('Y-m-d', strtotime($postData['toDate'][$i]));
                $this->repository->insertMonth($leaveYearId, $monthId, $monthName, $fromDate, $toDate, $leaveYear, $i+1);

            }

            $this->flashmessenger()->addMessage("Leave year added Successfully.");
            return $this->redirect()->toRoute("leaveYear");
        }

    }

    public function editAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("leaveYear", array("action"=>"index"));
        }

        if ($request->isPost()) {

            $postData = $request->getPost();

            $startDate = date('Y-m-d', strtotime($postData['fromDateYear']));
            $endDate = date('Y-m-d', strtotime($postData['toDateYear']));
            $leaveYear = $postData['leaveYearName'];

            $this->repository->updateYear($id, $startDate, $endDate, $leaveYear);

            for($i=0; $i<count($postData['monthId']); $i++){

                $fromDate = date('Y-m-d', strtotime($postData['fromDate'][$i]));
                $toDate = date('Y-m-d', strtotime($postData['toDate'][$i]));
                $this->repository->updateMonth($postData['monthId'][$i], $fromDate, $toDate);

            }

            $this->flashmessenger()->addMessage("Leave year updated Successfully.");
            return $this->redirect()->toRoute("leaveYear");
        }

        $leaveYearData = $this->repository->fetchLeaveYear($id);
        $leaveMonthData = iterator_to_array($this->repository->fetchLeaveMonth($id), false);

        $leaveYearData['START_DATE'] = date('d-M-Y', strtotime($leaveYearData['START_DATE']));
        $leaveYearData['END_DATE'] = date('d-M-Y', strtotime($leaveYearData['END_DATE']));

        return Helper::addFlashMessagesToArray(
            $this, [
                    'yearData' => $leaveYearData,
                    'monthData' => $leaveMonthData,
                    'id' => $id,
            ]);
    }

    public function deleteAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("leaveYear", array("action"=>"index"));
        }

        $this->repository->deleteYear($id);
        $this->repository->deleteMonth($id);

        $this->flashmessenger()->addMessage("Leave year updated Successfully.");
        return $this->redirect()->toRoute("leaveYear");
    }

}