<?php

namespace Setup\Controller;

use Application\Controller\HrisController;
use Application\Helper\Helper;
use Exception;
use Setup\Form\OrganizationTypeForm;
use Setup\Model\OrganizationType;
use Setup\Repository\OrganizationTypeRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class OrganizationTypeController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(OrganizationTypeRepository::class);
        $this->initializeForm(OrganizationTypeForm::class);
    }

    public function indexAction() {
        $request = $this->getRequest();
        if ($request->isPost()) {
            try {
                $result = $this->repository->fetchActiveRecord();
                $organizationTypeList = iterator_to_array($result, false);
                // print_r($organizationTypeList);die;
                return new JsonModel(['success' => true, 'data' => $organizationTypeList, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }
        return $this->stickFlashMessagesTo(['acl' => $this->acl]);
    }

    // private function prepareForm() {
    //     $type = $this->form->get('type');
    //     $typeList = ["PERMANENT" => "Permanent", "PROBATION" => "Probation", "CONTRACT" => "Contract", "TEMPORARY" => "Temporary", "RESIGNED" => "Resigned", "RETIRED" => "Retired"];
    //     $type->setValueOptions($typeList);
    // }

    public function addAction() {
        $request = $this->getRequest();
        if ($request->isPost()) {
            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                try {
                    $organizationType = new OrganizationType();
                    $organizationType->exchangeArrayFromForm($this->form->getData());
                    $organizationType->organizationTypeId = (int) Helper::getMaxId($this->adapter, OrganizationType::TABLE_NAME, OrganizationType::ORGANIZATION_TYPE_ID) + 1;
                    $organizationType->createdDt = date('Y-m-d');
                    $organizationType->createdBy = $this->employeeId;
                    $organizationType->status = 'E';
                    $organizationType->organizationTypeNameNdesc=$organizationType->organizationTypeName;

                    // print_r($organizationType->organizationTypeName);die;
                    $this->repository->add($organizationType);
                    $this->flashmessenger()->addMessage("Organization Type Successfully Added!!!");
                    return $this->redirect()->toRoute("organizationType");
                } catch (Exception $e) {
                    
                }
            }
        }
        return $this->stickFlashMessagesTo([
                    'form' => $this->form,
                'customRenderer' => Helper::renderCustomView()
        ]);
    }

    public function editAction() {
        $id = (int) $this->params()->fromRoute("id");
        if ($id === 0) {
            return $this->redirect()->toRoute();
        }
        // $this->prepareForm();
        $request = $this->getRequest();
        $organizationType = new OrganizationType();
        if (!$request->isPost()) {
            $organizationType->exchangeArrayFromDb($this->repository->fetchById($id)->getArrayCopy());
            $this->form->bind($organizationType);
        } else {

            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                $organizationType->exchangeArrayFromForm($this->form->getData());
                $organizationType->modifiedDt = Helper::getcurrentExpressionDate();
                $organizationType->modifiedBy = $this->employeeId;
                $organizationType->organizationTypeNameNdesc=$organizationType->organizationTypeName;

                $this->repository->edit($organizationType, $id);
                $this->flashmessenger()->addMessage("Organization Type Successfully Updated!!!");
                return $this->redirect()->toRoute("organizationType");
            }
        }
        return $this->stickFlashMessagesTo(['form' => $this->form, 'id' => $id, 'customRenderer' => Helper::renderCustomView()]);
    }

    public function deleteAction() {
        $id = (int) $this->params()->fromRoute("id");
        if (!$id) {
            return $this->redirect()->toRoute('organizationType');
        }
        $this->repository->delete($id);
        $this->flashmessenger()->addMessage("Organization Type Successfully Deleted!!!");
        return $this->redirect()->toRoute('organizationType');
    }

}
