<?php

namespace Setup\Controller;

use Application\Controller\HrisController;
use Application\Helper\Helper;
use Exception;
use Setup\Form\TrainingFeedbackChoiceForm;
use Setup\Model\TrainingFeedbackChoiceModel;
use Setup\Repository\TrainingFeedbackChoiceRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\View\Model\JsonModel;

class TrainingFeedbackChoiceController extends HrisController {


    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(TrainingFeedbackChoiceRepository::class);
        $this->initializeForm(TrainingFeedbackChoiceForm::class);
    }

    public function indexAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {
            try {
                $result = iterator_to_array($this->repository->fetchAll(), false);

                return new JsonModel(['success' => true, 'data' => $result, 'error' => '']);
            } catch (Exception $e) {
                return new JsonModel(['success' => false, 'data' => [], 'error' => $e->getMessage()]);
            }
        }

    }

    public function addAction(){

        $request = $this->getRequest();

        if ($request->isPost()) {
            $this->form->setData($request->getPost());
            if ($this->form->isValid()) {
                $model = new TrainingFeedbackChoiceModel();
                $model->exchangeArrayFromForm($this->form->getData());
                $model->createdDate = Helper::getcurrentExpressionDate();
                $model->createdBy = $this->employeeId;
                $model->choiceId = ((int) Helper::getMaxId($this->adapter, TrainingFeedbackChoiceModel::TABLE_NAME, TrainingFeedbackChoiceModel::CHOICE_ID)) + 1;
                $model->status = 'E';
                $this->repository->add($model);

                $this->flashmessenger()->addMessage("Training Choice Successfully added.");
                return $this->redirect()->toRoute("trainingFeedbackChoice");
            }
        }

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
        ]);

    }

    public function editAction(){

        $id = (int) $this->params()->fromRoute("id");
        
        if ($id == 0) {

            return $this->redirect()->toRoute('trainingFeedbackChoice');
        }

        $request = $this->getRequest();
        $choice = new TrainingFeedbackChoiceModel();

        if($request->isPost()){
            $this->form->setData($request->getPost());

            if ($this->form->isValid()) {

                $choice->exchangeArrayFromForm($this->form->getData());
                $choice->modifiedDate = Helper::getcurrentExpressionDate();
                $choice->modifiedBy = $this->employeeId;

                $this->repository->edit($choice, $id);
                $this->flashmessenger()->addMessage("Training Choice Successfully Updated!!!");
                return $this->redirect()->toRoute("trainingFeedbackChoice");
            }

        }

        $choice->exchangeArrayFromDB($this->repository->fetchById($id));
        $this->form->bind($choice);

        return Helper::addFlashMessagesToArray($this, [
            'form' => $this->form,
            'id' => $id
        ]);
    }

    public function deleteAction(){

        $id = (int) $this->params()->fromRoute("id");
        if (!$id) {
            return $this->redirect()->toRoute('trainingFeedbackChoice');
        }
        
        $this->repository->delete($id);
        
        $this->flashmessenger()->addMessage("Training Feedback Choice Successfully Deleted!!!");
        return $this->redirect()->toRoute('trainingFeedbackChoice');

    }

}
