<?php

namespace Setup\Repository;

use Application\Helper\EntityHelper;
use Application\Model\Model;
use Application\Repository\RepositoryInterface;
use Setup\Model\OrganizationType;
use Traversable;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Select;
use Zend\Db\TableGateway\TableGateway;

class OrganizationTypeRepository implements RepositoryInterface {

    private $tableGateway;

    public function __construct(AdapterInterface $adapter) {
        $this->tableGateway = new TableGateway(OrganizationType::TABLE_NAME, $adapter);
    }

    public function add(Model $model) {
        // print_r($model->getArrayCopyForDB());die;
        $this->tableGateway->insert($model->getArrayCopyForDB());
    }

    public function edit(Model $model, $id) {
        $array = $model->getArrayCopyForDB();
        // print_r($array);die;
        unset($array[OrganizationType::ORGANIZATION_TYPE_ID]);
        unset($array[OrganizationType::CREATED_DT]);
        $this->tableGateway->update($array, [OrganizationType::ORGANIZATION_TYPE_ID => $id]);
    }

    public function fetchAll() {
        return $this->tableGateway->select();
    }

    public function fetchById($id) {
        $rowset = $this->tableGateway->select(function(Select $select)use($id) {
            $select->columns(EntityHelper::getColumnNameArrayWithOracleFns(OrganizationType::class, [OrganizationType::ORGANIZATION_TYPE_NAME]), false);
            $select->where([OrganizationType::ORGANIZATION_TYPE_ID => $id]);
        });
        return $rowset->current();
    }

    public function fetchActiveRecord(): Traversable {
        return $rowset = $this->tableGateway->select(function(Select $select) {
            $select->columns(EntityHelper::getColumnNameArrayWithOracleFns(OrganizationType::class, [OrganizationType::ORGANIZATION_TYPE_NAME]), false);
            $select->where([OrganizationType::STATUS => 'E']);
            $select->order(OrganizationType::ORGANIZATION_TYPE_NAME . " ASC");
        });
    }

    public function delete($id) {
        $this->tableGateway->update([OrganizationType::STATUS => 'D'], [OrganizationType::ORGANIZATION_TYPE_ID => $id]);
    }

}
