<?php

namespace Setup\Controller;

use Application\Controller\HrisController;
use Application\Custom\CustomViewModel;
use Application\Helper\EntityHelper;
use Application\Helper\Helper;
use Exception;
use Payroll\Repository\BasicSalaryRepository;
use Payroll\Repository\FlatValueRepository;
use Payroll\Repository\PayrollRepository;
use Setup\Repository\FiscalYearRepository;
use Setup\Form\JobHistoryForm;
use Setup\Model\Branch;
use Setup\Model\Company;
use Setup\Model\Department;
use Setup\Model\Designation;
use Setup\Model\HrEmployees;
use Setup\Model\JobHistory;
use Setup\Model\Position;
use Setup\Model\ServiceEventType;
use Setup\Repository\EmployeeRepository;
use Setup\Repository\JobHistoryRepository;
use Zend\Authentication\Storage\StorageInterface;
use Zend\Db\Adapter\AdapterInterface;
use Zend\Db\Sql\Predicate\Predicate;
use PrintLayout\Controller\PrintController;
use PrintLayout\Repository\PrintRepository;
use Recruitment\Repository\UserApplicationRepository;
use Zend\View\Model\JsonModel;

class FiscalYearController extends HrisController {

    public function __construct(AdapterInterface $adapter, StorageInterface $storage) {
        parent::__construct($adapter, $storage);
        $this->initializeRepository(FiscalYearRepository::class);
    }

    public function indexAction() {

        $request = $this->getRequest();
        if ($request->isPost()) {

            $data = iterator_to_array($this->repository->fetchAll(), false);
            return new JsonModel(['success'=>true, 'data'=>$data]);
        
        }

    }

    public function addAction(){
        
        $request = $this->getRequest();
        if ($request->isPost()) {

            $postData = $request->getPost();
            
            $fiscalYearId = ((int) Helper::getMaxId($this->adapter, 'HRIS_FISCAL_YEARS', 'FISCAL_YEAR_ID')) + 1;
            $startDate = date('Y-m-d', strtotime($postData['fromDateYear']));
            $endDate = date('Y-m-d', strtotime($postData['toDateYear']));
            $fiscalYear = $postData['fiscalYearName'];
            $this->repository->insertYear($fiscalYearId, $startDate, $endDate, $fiscalYear);


            for($i=0; $i<count($postData['monthName']); $i++){

                $monthId = ((int) Helper::getMaxId($this->adapter, 'HRIS_MONTH_CODE', 'MONTH_ID')) + 1;
                $monthName = $postData['monthName'][$i];
                $fromDate = date('Y-m-d', strtotime($postData['fromDate'][$i]));
                $toDate = date('Y-m-d', strtotime($postData['toDate'][$i]));
                if(($i+1) < 10){
                    $monthNo = $i + 4;
                    $year = explode('/', $fiscalYear)[0];
                }else{
                    $monthNo = $i - 8;
                    $year = explode('/', $fiscalYear)[1];
                }
                $this->repository->insertMonth($fiscalYearId, $monthId, $monthName, $fromDate, $toDate, $year, $monthNo,$i+1);

            }

            $this->flashmessenger()->addMessage("Fiscal year added Successfully.");
            return $this->redirect()->toRoute("fiscalYear");
        }

    }

    public function editAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("fiscalYear", array("action"=>"index"));
        }

        if ($request->isPost()) {

            $postData = $request->getPost();

            $startDate = date('Y-m-d', strtotime($postData['fromDateYear']));
            $endDate = date('Y-m-d', strtotime($postData['toDateYear']));
            $fiscalYear = $postData['fiscalYearName'];

            $this->repository->updateYear($id, $startDate, $endDate, $fiscalYear);

            for($i=0; $i<count($postData['monthId']); $i++){

                $fromDate = date('Y-m-d', strtotime($postData['fromDate'][$i]));
                $toDate = date('Y-m-d', strtotime($postData['toDate'][$i]));
                $this->repository->updateMonth($postData['monthId'][$i], $fromDate, $toDate);

            }

            $this->flashmessenger()->addMessage("Fiscal year updated Successfully.");
            return $this->redirect()->toRoute("fiscalYear");
        }

        $fiscalYearData = $this->repository->fetchFiscalYear($id);
        $fiscalMonthData = iterator_to_array($this->repository->fetchFiscalMonth($id), false);

        $fiscalYearData['START_DATE'] = date('d-M-Y', strtotime($fiscalYearData['START_DATE']));
        $fiscalYearData['END_DATE'] = date('d-M-Y', strtotime($fiscalYearData['END_DATE']));

        return Helper::addFlashMessagesToArray(
            $this, [
                    'yearData' => $fiscalYearData,
                    'monthData' => $fiscalMonthData,
                    'id' => $id,
            ]);
    }

    public function deleteAction(){

        $request = $this->getRequest();

        $id = (int) $this->params()->fromRoute('id');

        if ($id === 0 && !$request->isPost())
        {
            return $this->redirect()->toRoute("fiscalYear", array("action"=>"index"));
        }

        $this->repository->deleteYear($id);
        $this->repository->deleteMonth($id);

        $this->flashmessenger()->addMessage("Fiscal year updated Successfully.");
        return $this->redirect()->toRoute("fiscalYear");
    }

}